//
//  BackendMessage.m
//  pwSafe
//
//  Created by Jorge P. Vasquez on 23/02/13.
//
//

#import "BackendMessage.h"
#import <Parse/Parse.h>
#import <objc/runtime.h>

@interface BackendMessageDelegate : NSObject <UIAlertViewDelegate>

@property NSURL *url;

@end

static NSString * const kBackendMessagesKeyName = @"BackendMessages";
static NSString * const kBackendMessagesLastCheckKeyName = @"BackendMessagesLastCheck";
#define kBackendMessageMinCheckInterval ((NSTimeInterval) 86400)

void showBackendMessage(NSString *messageName, void (^completion)(BOOL show))
{
  NSDate *now = [NSDate date];
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
  NSDictionary *lastCheckDict = [defaults objectForKey:kBackendMessagesLastCheckKeyName];
  NSDate *lastCheck;
  if(lastCheckDict && (lastCheck = lastCheckDict[messageName]))
  {
    NSTimeInterval elapsed = -[lastCheck timeIntervalSinceNow];
    if(elapsed < kBackendMessageMinCheckInterval)
    {
      A77Log(@"Not rechecking now, las checked not too long ago: %f seconds", elapsed);
      if(completion)
        completion(NO);
      return;
    }
  }
  if(!lastCheckDict)
  {
    [defaults setObject:@{ messageName : now } forKey:kBackendMessagesLastCheckKeyName];
  }
  else
  {
    NSMutableDictionary *newLastCheckDict = [lastCheckDict mutableCopy];
    newLastCheckDict[messageName] = now;
    [defaults setObject:newLastCheckDict forKey:kBackendMessagesLastCheckKeyName];
  }
  
  PFQuery *query = [PFQuery queryWithClassName:@"Message"];
  
  completion = completion ? [completion copy] : nil;
  
  [query whereKey:@"name" equalTo:messageName];
  [query findObjectsInBackgroundWithBlock:^(NSArray *objects, NSError *error) {
    if(!objects.count)
    {
      A77Log(@"Could not find message %@: %@/%@", messageName, objects, error);
      if(completion)
        completion(NO);
      return;
    }
    PFObject *motd = objects[0];
    if(![motd[@"active"]boolValue])
    {
      A77Log(@"message %@ innactive", messageName);
      if(completion)
        completion(NO);
      return;
    }
    NSDate *minDate = motd[@"minDate"];
    NSDate *maxDate = motd[@"maxDate"];
    if(minDate || maxDate)
    {
      if(minDate && [minDate compare:now] != NSOrderedAscending)
      {
        A77Log(@"message %@ still not valid (starts on %@)", messageName, minDate);
        if(completion)
          completion(NO);
        return;
      }
      if(maxDate && [maxDate compare:now] != NSOrderedDescending)
      {
        A77Log(@"message %@ expired (ended on %@)", messageName, maxDate);
        if(completion)
          completion(NO);
        return;
      }
    }
    
    NSDictionary *backendMessages = [defaults objectForKey:kBackendMessagesKeyName];
    NSDate *lastVersionShown;
    if(backendMessages && (lastVersionShown = backendMessages[messageName]))
    {
      NSDate *version = motd.updatedAt;
      if([version isEqualToDate:lastVersionShown])
      {
        A77Log(@"Message %@ already show", messageName);
        if(completion)
          completion(NO);
        return;
      }
    }
    
    NSString *title = motd[@"title"];
    NSString *text = motd[@"message"];
    NSString *urlStr = motd[@"url"];
    NSURL *url = urlStr.length ? [NSURL URLWithString:urlStr] : nil;
    
    if(!title.length || !text.length)
    {
      A77Log(@"Message %@ incomplete: %@/%@", messageName, title, text);
      if(completion)
        completion(NO);
      return;
    }
    
    UIAlertView *alert;
    if(url && [[UIApplication sharedApplication]canOpenURL:url])
    {
      BackendMessageDelegate *dg = [[BackendMessageDelegate alloc]init];
      dg.url = url;
      alert = [[UIAlertView alloc]initWithTitle:title message:text delegate:dg cancelButtonTitle:@"Dismiss" otherButtonTitles:@"Open", nil];
      objc_setAssociatedObject(alert, @"url", dg, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
    }
    else
    {
      alert = [[UIAlertView alloc]initWithTitle:title message:text delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    }
    [alert show];
    
    if(backendMessages)
    {
      NSMutableDictionary *newBackendMessages = [backendMessages mutableCopy];
      newBackendMessages[messageName] = motd.updatedAt;
      [defaults setObject:newBackendMessages forKey:kBackendMessagesKeyName];
    }
    else
    {
      backendMessages = @{messageName : motd.updatedAt };
      [defaults setObject:backendMessages forKey:kBackendMessagesKeyName];
    }
    if(completion)
      completion(YES);
  }];
}

@implementation BackendMessageDelegate

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex
{
  if(buttonIndex == alertView.cancelButtonIndex)
    return;
  UIApplication *app = [UIApplication sharedApplication];
  NSURL *url = self.url;
  if(!url || ![app openURL:url])
    A77Log(@"Could not find or open URL on alertview: %@", url);
  return;
}


@end

