//
//  CreateGroupViewController.m
//  iPasswordSafe
//
//  Created by Jorge Peixoto Vasquez on 5/31/11.
//  Copyright 2011 app77. All rights reserved.
//

#import "CreateGroupViewController.h"
#import "SelectGroupViewController.h"
#import "SafeFile.h"
#import "AppTheme.h"

#define TextViewTag 44

@implementation CreateGroupViewController

@synthesize  result = m_result;

- (id)initWithTarget:(id)target selector:(SEL)selector file:(SafeFile *)file
{
  self = [super initWithStyle:UITableViewStyleGrouped];
  if (!self) 
    return nil;
  
  m_target = target;
  m_selector = selector;
  m_file = file;
  m_newParentGroupPath = @"";
  
  self.navigationItem.title = @"New group";
  
  return self;
}

- (void)dealloc
{
  if(self.isViewLoaded)
  {
    self.tableView.delegate = nil;
    self.tableView.dataSource = nil;
  }
}

- (NSString *)groupName
{
  if(m_newParentGroupPath.length)
    return [NSString stringWithFormat:@"%@.%@", m_newParentGroupPath, m_newGroupName];
  else
    return m_newGroupName;
}

- (void)onCancel:(id)sender
{
  m_result = NO;
  if(m_target)
  {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
    [m_target performSelector:m_selector withObject:self];
#pragma clang diagnostic pop
    m_target = nil;
  }
}

- (void)onDone:(id)sender
{
  m_result = YES;
  if(m_target)
  {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
    [m_target performSelector:m_selector withObject:self];
#pragma clang diagnostic pop
    m_target = nil;
  }
}

#pragma mark - View lifecycle

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)viewWillAppear:(BOOL)animated
{
  [super viewWillAppear:animated];
  
  if(m_selectGroupVC)
  {
    m_newParentGroupPath = [m_selectGroupVC selectedGroup];
    m_selectGroupVC = nil;
    self.navigationItem.rightBarButtonItem.enabled = m_newGroupName && m_newGroupName.length > 0 && ![m_file.sortedGroups containsObject:self.groupName];
  }

  [self.tableView reloadRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:1 inSection:0]] withRowAnimation:UITableViewRowAnimationNone];
}

- (void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
  UITextField *textField = (UITextField *)[cell.contentView viewWithTag:TextViewTag];
  [textField becomeFirstResponder];
}


- (void)viewDidLoad
{
  [super viewDidLoad];
  [self.tableView themeViewDidLoadInIpadPos:TVP_FORMSHEET];
  
  UIBarButtonItem *btn  = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(onCancel:)];
  self.navigationItem.leftBarButtonItem = btn;
  
  btn = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(onDone:)];
  self.navigationItem.rightBarButtonItem = btn;
  btn.enabled = NO;
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  return 2;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  UITableViewCell *cell;
  
  NSUInteger row = indexPath.row;

  if(row == 0)
  {
    static NSString *CellIdentifier = @"Cell0";    

    cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) 
    {
      cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:CellIdentifier];
    
      cell.selectionStyle = UITableViewCellSelectionStyleNone;
      
      NSUInteger w = UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ? 350 : 170;
      UITextField *textField = [[UITextField alloc]initWithFrame:CGRectMake(120, 12, w, 21)];
      textField.borderStyle = UITextBorderStyleNone;
      textField.adjustsFontSizeToFitWidth = YES;
      textField.textColor = cell.detailTextLabel.textColor;
      textField.autocapitalizationType = UITextAutocapitalizationTypeNone;
      textField.autocorrectionType = UITextAutocorrectionTypeNo;
      textField.enablesReturnKeyAutomatically = NO;
      textField.returnKeyType = UIReturnKeyDone;
      textField.clearsOnBeginEditing = NO;
      textField.tag = TextViewTag;
      textField.delegate = self;
      textField.clearButtonMode = UITextFieldViewModeWhileEditing;

      [textField addTarget:self action:@selector(onTextChanged:) forControlEvents:UIControlEventEditingChanged];
      [cell.contentView addSubview:textField];
    }

    cell.textLabel.text = @"Name";
    UITextField *textField = (UITextField *)[cell.contentView viewWithTag:TextViewTag];
    textField.text = m_newGroupName;
    [textField becomeFirstResponder];
  }
  else
  {
    static NSString *CellIdentifier = @"Cell1";    
    NSAssert(row == 1, @"Unknown row");
    cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) 
      cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:CellIdentifier];
    
    cell.textLabel.text = @"Parent";
    cell.detailTextLabel.text = [[m_newParentGroupPath componentsSeparatedByString:@"."]lastObject];
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
  }
  
  [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];  
  
  return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  if(indexPath.row != 1)
    return;
  
  m_selectGroupVC = [[SelectGroupViewController alloc] initWithSafeFile:m_file currentGroup:m_newParentGroupPath];
  [self.navigationController pushViewController:m_selectGroupVC animated:YES];
}

#pragma mark - Text view events

- (void)textFieldDidBeginEditing:(UITextField *)textField
{
  if(!iOS6())
    textField.textColor = kThemeTintColor;
}

- (void)textFieldDidEndEditing:(UITextField *)textField
{
  if(!iOS6())
    textField.textColor = [UIColor blackColor];
}

- (void)onTextChanged:(id)sender
{
  UITextField *txtField = (UITextField *)sender;
  NSString *text = txtField.text;
  
  m_newGroupName = text ? [text copy] : nil;
  
  self.navigationItem.rightBarButtonItem.enabled = text && text.length > 0 && ![m_file.sortedGroups containsObject:self.groupName];
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
  if(self.navigationItem.rightBarButtonItem.enabled)
    [self onDone:textField];
  return NO;
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
  return ![string isEqualToString:@"."];
}

@end
