//
//  EditSafePwPoliciesVC.mm
//  pwSafe
//
//  Created by Jorge P. Vasquez on 03/10/12.
//
//

#import "EditSafePwPoliciesVC.h"
#import "SafeFile.h"
#import "PwGenParsViewController.h"
#import "NewPwPolicyVC.h"
#import "AppTheme.h"

#include "PWPolicy.h"
#include "PWSprefs.h"
#include "PWScore.h"

@interface EditSafePwPoliciesVC () <UINavigationControllerDelegate>
{
  SafeFile *_file;
  NSArray *_names;
  void (^_handler)(EditSafePwPoliciesVC *vc);
  NSString *_editedPolicyName;
  NSIndexPath *_indexPathToSelect;
  UIBarButtonItem *_rightBarBtn;
}
@end

@implementation EditSafePwPoliciesVC

- (id)initWithSafeFile:(SafeFile *)file handler:(void (^)(EditSafePwPoliciesVC *vc))handler
{
  self = [super initWithStyle:UITableViewStyleGrouped];
  if (!self)
    return nil;

  _file = file;
  _handler = [handler copy];
  
  self.navigationItem.title = @"Password policies";
  _rightBarBtn = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(onDone:)];
  self.navigationItem.rightBarButtonItem = _rightBarBtn;
  self.navigationItem.leftBarButtonItem = self.editButtonItem;
  UIBarButtonItem *addBtn = [UIBarButtonItem buttonWithImageNamed:imageName(@"Plus") target:self action:@selector(onAddPolicy:)];
  addBtn.accessibilityLabel = @"Add";
  addBtn.style = UIBarButtonItemStylePlain;
  self.toolbarItems = @[
    [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil],
    addBtn,
  ];

  return self;
}

- (void)dealloc
{
  if(self.isViewLoaded)
  {
    self.tableView.delegate = nil;
    self.tableView.dataSource = nil;
  }
}

- (void)didReceiveMemoryWarning
{
  [super didReceiveMemoryWarning];
  _names = nil;
}

- (void)onDone:(id)sender
{
  _handler(self);
}

- (void)onAddPolicy:(id)sender
{
  NewPwPolicyVC *vc = [[NewPwPolicyVC alloc]initWithHandler:^(NewPwPolicyVC *vc) {
    [self.navigationController popViewControllerAnimated:YES];
    if(!vc.ok)
      return;
    NSString *name = vc.name;

    PWPolicy policy;
    [_file defaultPwPolicy:policy];
    [_file addPwPolicy:policy named:name];
    _names = _file.namedPwPoliciesNames;
    UITableView *tableView = self.tableView;
    [tableView reloadData];
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:[_names indexOfObject:name] inSection:1];
    [tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionMiddle];

    _indexPathToSelect = indexPath;
    self.navigationController.delegate = self;
  }];
  [self.navigationController pushViewController:vc animated:YES];
}

- (void)navigationController:(UINavigationController *)navigationController didShowViewController:(UIViewController *)viewController animated:(BOOL)animated
{
  NSAssert(viewController == self && _indexPathToSelect, @"Only with this condition there's a delegate");
  navigationController.delegate = nil;
  [self tableView:self.tableView didSelectRowAtIndexPath:_indexPathToSelect];
  _indexPathToSelect = nil;
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated
{
  [self.navigationItem setRightBarButtonItem:editing ? nil : _rightBarBtn animated:animated];
  [super setEditing:editing animated:animated];
}

#pragma mark - View lifecycle

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  [self.tableView themeViewDidLoadInIpadPos:TVP_FORMSHEET];
}

- (void)viewWillAppear:(BOOL)animated
{
  [self.navigationController setToolbarHidden:NO animated:animated];
  [super viewWillAppear:animated];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  if(section == 0)
    return 1; // Default policy
  if(!_names)
    _names = _file.namedPwPoliciesNames;
  
  BOOL visible = _names.count > 0;
  BOOL currentlyVisible = self.navigationItem.leftBarButtonItem != nil;
  
  if(visible != currentlyVisible)
    [self.navigationItem setLeftBarButtonItem:visible ? self.editButtonItem : nil animated:YES];

  return _names.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  static NSString *CellIdentifier = @"Cell";
  UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
  if(!cell)
  {
    cell = [[UITableViewCell alloc]initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    cell.accessibilityTraits = UIAccessibilityTraitButton;
  }
  NSUInteger section = indexPath.section;
  NSUInteger row = indexPath.row;
  if(section == 0)
  {
    cell.textLabel.text = @"Default policy";
    cell.detailTextLabel.text = @"";
  }
  else
  {
    if(!_names)
      _names = _file.namedPwPoliciesNames;
    NSString *name = _names[row];
    cell.textLabel.text = name;
    PWPolicy pol;
    [_file pwPolicy:pol named:name];
    cell.detailTextLabel.text = [NSString stringWithFormat:@"Used by %zu %@", pol.usecount, pol.usecount == 1 ? @"entry" : @"entries"];
  }
  
  [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];
  return cell;
}

// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
  NSUInteger section = indexPath.section;
  if(section == 0)
    return NO;
  NSUInteger row = indexPath.row;
  if(!_names)
    _names = _file.namedPwPoliciesNames;
  NSString *name = _names[row];
  PWPolicy pol;
  [_file pwPolicy:pol named:name];
  return pol.usecount == 0;
}

// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
  if (editingStyle == UITableViewCellEditingStyleDelete)
  {
    NSUInteger section = indexPath.section;
    NSAssert(section == 1, @"Must be here");
    (void)section;
    NSUInteger row = indexPath.row;
    // Delete the row from the data source
    if(!_names)
      _names = _file.namedPwPoliciesNames;
    NSString *name = _names[row];
    _names = nil;
    [_file deletePwPolicyNamed:name];
    [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationLeft];
    
    if(self.isEditing && [self tableView:tableView numberOfRowsInSection:1] == 0)
      [self setEditing:NO animated:YES];
  }
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
  if(section == 0)
    return nil;
  return @"Named policies";
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  PwGenParsViewController *vc;
  NSUInteger section = indexPath.section;
  if(section == 0)
  {
    vc = [[PwGenParsViewController alloc]initForCurrentSafe:_file];
    vc.navigationItem.title = @"Default policy";
    _editedPolicyName = nil;
  }
  else
  {
    NSUInteger row = indexPath.row;
    if(!_names)
      _names = _file.namedPwPoliciesNames;
    NSString *name = _names[row];
    vc = [[PwGenParsViewController alloc]initWithPolicyNamed:name inFile:_file];
    vc.navigationItem.title = name;
    _editedPolicyName = name;
  }
  [vc setOnDoneTarget:self action:@selector(onPwPolicyEditComplete:)];
  
  [self.navigationController pushViewController:vc animated:YES];
}

- (void)onPwPolicyEditComplete:(PwGenParsViewController *)genParsVC
{
  [self.navigationController popViewControllerAnimated:YES];
  if(genParsVC.canceled)
    return;
  
  if(!_editedPolicyName)
    [genParsVC updateSafe];
  else
    [genParsVC updatePolicyWithName:_editedPolicyName];
}

@end
