
//
//  FileSystemFolders.m
//  pwSafe
//
//  Created by Jorge Peixoto Vasquez on 03/07/12.
//  Copyright (c) 2012 app77. All rights reserved.
//

#import "FileSystemFolders.h"

static NSString * const kLocalSafeDirSync = @"Local";

NSString *localSafeDir()
{
  @synchronized(kLocalSafeDirSync)
  {
    static NSString *dir = nil;
    if(!dir)
    {
#ifdef PWSAFE_COZI
      NSArray *libsDirs = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
      NSString *libDir = libsDirs[0];
      dir = [libDir stringByAppendingPathComponent:@"Safes"];
#else
      NSArray *docDirs = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
      dir = docDirs[0];
#endif
    }
    return dir;
  }
}

static NSString * const kDropboxSafeDirSync = @"Dropbox";

NSString *dropboxDir()
{
  @synchronized(kDropboxSafeDirSync)
  {
    static NSString *dropboxDir = nil;
    if(!dropboxDir)
    {
      NSArray *libsDirs = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
      NSString *libDir = libsDirs[0];
      dropboxDir = [libDir stringByAppendingPathComponent:@"Dropbox"];
    }
    return dropboxDir;
  }
}

static NSString * const kOldDropboxSafeDirSync = @"OldDropbox";

NSString *oldDropboxDir()
{
  @synchronized(kOldDropboxSafeDirSync)
  {
    static NSString *oldDropboxDir = nil;
    if(!oldDropboxDir)
    {
      NSArray *cachesDirs = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
      NSString *cacheDir = cachesDirs[0];
      oldDropboxDir = [cacheDir stringByAppendingPathComponent:@"Dropbox"];
    }
    return oldDropboxDir;
  }
}

void iCloudURL(URLBlock block)
{
  
  URLBlock userBlock = [block copy];
  block = [^(NSURL *url){dispatch_async(dispatch_get_main_queue(), ^{userBlock(url);});} copy];
  
  dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
#ifdef PWSAFE_COZI
    static NSString *const kiCloudId = @"W2LRL3T5P8.com.cozi.passwordvault";
#else
    static NSString *const kiCloudId = @"F2XAAD6WWR.com.app77.pwsafe";
#endif
    
//#ifdef DEBUG_MODE
//    static dispatch_once_t onceToken;
//    dispatch_once(&onceToken, ^{
//      NSLog(@"Simulating slow iCloud init");
//      sleep(3);
//      NSLog(@"Simulating slow iCloud init complete");
//    });
//#endif
    
    NSFileManager *fm = [NSFileManager defaultManager];
    NSURL *ubiquityURL;
    
    if([fm respondsToSelector:@selector(ubiquityIdentityToken)])
      ubiquityURL = [fm ubiquityIdentityToken] ? [fm URLForUbiquityContainerIdentifier:kiCloudId] : nil;
    else
      ubiquityURL = [fm URLForUbiquityContainerIdentifier:kiCloudId];
    
    if(!ubiquityURL)
    {
      block(nil);
      return;
    }
    
#ifdef PWSAFE_COZI
    block(ubiquityURL); // Cozi does not expose iCloud documents individually
#else
    block([ubiquityURL URLByAppendingPathComponent:@"Documents" isDirectory:YES]);
#endif
  });
}

static NSString * const kiCloudSafeTempDirSync = @"iCloudTemp";

NSString *iCloudTempDir()
{
  @synchronized(kiCloudSafeTempDirSync)
  {
    static NSString *dir = nil;
    if(!dir)
    {
      NSArray *cachesDirs = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
      dir = [cachesDirs[0]stringByAppendingPathComponent:@"iCloud"];
    }
    return dir;
  }
}

void moveIntoICloud(NSString *filePath, void (^completion)(BOOL moveOk, NSError *error))
{
  void (^completionCopy)(BOOL moveOk, NSError *error) = completion ? [completion copy] : nil;
  NSURL *urlSandbox = [NSURL fileURLWithPath:filePath];
  NSString *fileName = [filePath lastPathComponent];
  iCloudURL(^(NSURL *url) {
    NSURL *urlICloud = [url URLByAppendingPathComponent:fileName isDirectory:NO];
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
      NSFileManager *fileManager = [NSFileManager defaultManager];
      NSError *error = nil;
      BOOL ok = [fileManager setUbiquitous:YES itemAtURL:urlSandbox destinationURL:urlICloud error:&error];
      if(!ok)
        A77Log(@"Error moving local file into iCloud: %@/%ld", error.domain, (long)error.code);
      
      if(!completionCopy)
        return;
      dispatch_async(dispatch_get_main_queue(), ^{
        completionCopy(ok, ok ? nil : error);
      });
    });
  });
}
