//
//  FileViewController.m
//  iPasswordSafe
//
//  Created by Jorge Peixoto Vasquez on 3/7/11.
//  Copyright 2011 app77. All rights reserved.
//

#import "FileViewController.h"
#import "GroupViewController.h"
#import "SafeCtx.h"
#import "SafeCtxStorage.h"
#import "CreateGroupViewController.h"
#import "DetailViewController.h"
#import "NewEditSafeViewController.h"
#import "MBProgressHUD.h"
#import "EditableSimpleTableViewCell.h"
#import "AppTheme.h"
#import "AddHelpTextUITableViewController.h"
#import "EditSafePwPoliciesVC.h"
#import "PwHistoryVC.h"
#import "DismissableNavigationController.h"
#import "MobiusAdVc.h"
#import "BackendMessage.h"
#import <Parse/Parse.h>

#import <MessageUI/MessageUI.h>
#import <objc/runtime.h>

#import "SafeFile.h"
#ifdef PWSAFE_WITH_ICLOUD
#  import "SafeFile_iCloud.h"
#endif
#import "SafeFile+Print.h"
#import "NewPDFPasswordVC.h"

#ifndef PWSAFE_NO_MOBIUS
#import "MobiusSDK.h"
#endif

#define FILE_VC_ACTIONSHEET_SETTINGS_TAG 5001
#define FILE_VC_ACTIONSHEET_DELETE_TAG 5002
#define FILE_VC_ACTIONSHEET_SHARE_TAG 5003

#define FILE_VC_TEXTFIELD_TAG 6001

@interface FileViewController () <UIActionSheetDelegate, MFMailComposeViewControllerDelegate, UIDocumentInteractionControllerDelegate, PwHistoryVCDelegate>
{
  NSMutableDictionary *m_state;
  BOOL m_hasAlreadyAppeared;  
  UIActionSheet *m_currentActionSheet;
  NSIndexPath *m_groupToDelete;
  UIDocumentInteractionController *m_docIntControllerForOpenIn;
  void (^m_blockToRunAfterViewAppears)();
  NSString *m_attachmentTypeInEmail; // Just for logging
  BOOL m_writingMyself;
}

@end

static NSInteger kUngroupedRowIdx;
static NSInteger kSearchAllItemsRowIdx;
static NSInteger kAllItemsRow;

@implementation FileViewController

@synthesize initialGroup = m_initialGroup;
@synthesize initialSearchTerm = m_initialSearchTerm;

+ (void)initialize
{
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
  {
    kAllItemsRow = 0;
    kSearchAllItemsRowIdx = -1;
    kUngroupedRowIdx = 1;
  }
  else
  {
    kAllItemsRow = 1;
    kSearchAllItemsRowIdx = 0;
    kUngroupedRowIdx = 2;
  }
}

static NSString * const kOpenedGroup = @"OpenedGroup";
static NSString * const kIsGroupOpened = @"IsGroupOpened";
static NSString * const kHelpText = @"Tap here to add a\nnew group";
static NSInteger const kHelpBaseTag = 13567;

#pragma mark -
#pragma mark Actions

-(void)onAddComplete:(id)sender
{
  CreateGroupViewController *newGroupVC = sender;
  if(newGroupVC.result)
  {
    m_writingMyself = YES;
    [m_file addGroup:newGroupVC.groupName];
    [self reloadTableViewPreservingSelection];
    [self adjustAddHelpText:kHelpText viewTagBase:kHelpBaseTag show:[self tableView:self.tableView numberOfRowsInSection:1] == 0];
    [self adjustEditBtnVisibilityAnimated:YES];
  }
  [self dismissModalViewControllerAnimated:YES];
}

- (void)onAddNewItem:(id)sender
{
  CreateGroupViewController *newGroupVC = [[CreateGroupViewController alloc] initWithTarget:self selector:@selector(onAddComplete:) file:m_file];
  
  DismissableNavigationController *nav = [[DismissableNavigationController alloc]initWithRootViewController:newGroupVC];
  nav.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    nav.modalPresentationStyle = UIModalPresentationFormSheet;
  
  [self presentModalViewController:nav animated:YES];
}

- (void)onCloseSafe:(id)sender
{
  [m_state removeAllObjects];
  [m_safeCtx expireNow];
  m_safeCtx.opened = NO;
  [UIPasteboard generalPasteboard].string = @"";
  
  UINavigationController *nav = self.navigationController;
  UIViewController *currentVC = nav.viewControllers.lastObject;
  if(currentVC != nav.viewControllers[0])
  {
    [currentVC dismissModalViewControllerAnimated:YES];
    [nav popToRootViewControllerAnimated:YES];
  }
  
  UIView *rootView = [MBProgressHUD rootView];
  MBProgressHUD *hud = [[MBProgressHUD alloc]initWithView:rootView];
  hud.customView = [[UIImageView alloc]initWithImage:[UIImage imageNamed:@"Check.png"]];
  hud.mode = MBProgressHUDModeCustomView;
  hud.labelText = @"Clipboard cleared";
  hud.animationType = MBProgressHUDAnimationZoom;
  hud.yOffset = -50;
  hud.opacity = .8;
  hud.removeFromSuperViewOnHide = YES;
  [rootView addSubview:hud];
  [hud show:YES];
  int32_t secs = UIAccessibilityIsVoiceOverRunning() ? 3 : 1;
  dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, secs * NSEC_PER_SEC);
  dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
    [hud hide:YES];
  });
}

- (void)presentCurrentActionSheet:(id)sender
{
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
  {
    if([sender isKindOfClass:[UIBarButtonItem class]])
    {
      [m_currentActionSheet showFromBarButtonItem:(UIBarButtonItem *)sender animated:YES];
    }
    else
    {
      NSAssert([sender isKindOfClass:[UIView class]], @"Must be a UIView");
      UIView *view = (UIView *)sender;
      [m_currentActionSheet showFromRect:view.frame inView:view.superview animated:YES];
    }
  }
  else
  {
    [m_currentActionSheet showFromToolbar:self.navigationController.toolbar];
  }
  
}

- (void) onSettings:(id)sender
{
  if(m_currentActionSheet)
  {
    [m_currentActionSheet dismissWithClickedButtonIndex:m_currentActionSheet.cancelButtonIndex animated:YES];
  }

#ifdef PWSAFE_VAULT
  NSString *safeStr = @"Vault";
#else
  NSString *safeStr = @"Safe";
#endif

#ifdef PWSAFE_NO_MOBIUS
  BOOL mobiusInstalled = NO;
  BOOL mobiusRegistered = NO;
#else
  BOOL mobiusInstalled = [MobiusSDK isMobiusEngineAppInstalled];
  BOOL mobiusRegistered = [[MobiusSDK sharedMobius]isAccountRegistered:m_safeCtx.safeUUID];
#endif
  
  m_currentActionSheet = [[UIActionSheet alloc]initWithTitle:[safeStr stringByAppendingString:@" settings"]
                                                    delegate:self
                                           cancelButtonTitle:@"Cancel"
                                      destructiveButtonTitle:nil
                                           otherButtonTitles:[safeStr stringByAppendingString:@" password"], @"Password policies", @"Password history", (mobiusInstalled ? ( mobiusRegistered ? @"Re-register with Mobius" : @"Register with Mobius") : nil),  nil
  ];
  m_currentActionSheet.tag = FILE_VC_ACTIONSHEET_SETTINGS_TAG;

  [self presentCurrentActionSheet:sender];
}

- (void)deleteGroup
{
  NSString *groupName = [self titleForCellAtIndexPath:m_groupToDelete forHumans:NO];
  NSAssert(groupName, @"Must find");
  m_writingMyself = YES;
  NSArray *sortedGroups = m_file.sortedGroups;
  NSUInteger nGroupsBeforeDeletion = sortedGroups.count;
  if([sortedGroups containsObject:@""] && nGroupsBeforeDeletion > 0)
    nGroupsBeforeDeletion--;
  
  NSArray *deletedGroupsNames = [m_file deleteGroup:groupName];

  A77Log(@"Deleting group %@", groupName);
  A77Log(@"All groups = %@", sortedGroups);
  
  NSMutableArray *deletedGroups = [NSMutableArray arrayWithCapacity:deletedGroupsNames.count];
  
  BOOL deletedFirst = NO;
  BOOL deletedLast = NO;
  for(NSUInteger i = 0; i < deletedGroupsNames.count; i++)
  {
    NSUInteger row = m_groupToDelete.row + i;
    [deletedGroups addObject:[NSIndexPath indexPathForRow:row inSection:m_groupToDelete.section]];
    if(row == 0)
      deletedFirst = YES;
    if(row == nGroupsBeforeDeletion - 1)
      deletedLast = YES;
  }

  A77Log(@"Deleting groups %@", deletedGroupsNames);
  
  UITableView *tableView = self.tableView;
  [tableView deleteRowsAtIndexPaths:deletedGroups withRowAnimation:UITableViewRowAnimationLeft];
  NSUInteger nGroupsAfterDeletion = nGroupsBeforeDeletion - deletedGroupsNames.count;
  
  if(iOS6() && nGroupsAfterDeletion)
  {
    double delayInSeconds = 0.35;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
    NSIndexPath *groupToDelete = m_groupToDelete;
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
      BOOL reloadFirst = NO;
      if(deletedFirst)
      {
        reloadFirst = YES;
        [tableView reloadRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:0 inSection:groupToDelete.section]] withRowAnimation:UITableViewRowAnimationNone];
      }
      if(deletedLast && (!reloadFirst || nGroupsAfterDeletion > 1))
        [tableView reloadRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:nGroupsAfterDeletion - 1 inSection:groupToDelete.section]] withRowAnimation:UITableViewRowAnimationNone];
    });
  }
  
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad && self.tableView.indexPathForSelectedRow == nil)
  {
    NSIndexPath *allItemsIdx = [NSIndexPath indexPathForRow:0 inSection:0];
    [self.tableView selectRowAtIndexPath:allItemsIdx animated:YES scrollPosition:UITableViewScrollPositionTop];
    [self tableView:self.tableView didSelectRowAtIndexPath:allItemsIdx];
  }
  
  m_groupToDelete = nil;
  [self adjustAddHelpText:kHelpText viewTagBase:kHelpBaseTag show:[self tableView:self.tableView numberOfRowsInSection:1] == 0];
  [self adjustEditBtnVisibilityAnimated:YES];
}

- (void)historyVC:(PwHistoryVC *)historyVC setEditing:(BOOL)editing
{
  [self dismissModalViewControllerAnimated:YES];
  
  NSAssert(!editing, @"Should only be called in edition end");
  BOOL changed = NO;
  BOOL enabled = historyVC.historyEnabled;
  uint max = historyVC.maxEntries;
  
  [m_file beginMultipleChanges];
  
  if(!!m_file.defaultKeepPwHistory != !!enabled)
  {
    m_file.defaultKeepPwHistory = enabled;
    changed = YES;
  }
  if(m_file.defaultPwHistoryEntries != max)
  {
    m_file.defaultPwHistoryEntries = max;
    changed = YES;
  }
  
  if(historyVC.shouldUpdateAllEntries)
  {
    changed = YES;
    [m_file enablePwHistoryOnAllEntries:enabled withMaxEntries:max];
    A77Log(@"Changing password history to %@ on all entries", enabled ? @"on" : @"off");
  }
  
  m_writingMyself = changed;
  [m_file endMultipeChanges:changed];
}

- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
  NSAssert(m_currentActionSheet == actionSheet, @"Just now one");
  m_currentActionSheet = nil;
  
  if(actionSheet.cancelButtonIndex == buttonIndex)
    return;
  
  if(actionSheet.tag == FILE_VC_ACTIONSHEET_SETTINGS_TAG)
  {
    if(buttonIndex == 0)
    {
      // Safe password
      NewEditSafeViewController *newSafeVC = [[NewEditSafeViewController alloc]initWithTarget:self selector:@selector(onSafeEdit:) name:m_safeCtx.safeName];

      UINavigationController *nav = [[UINavigationController alloc]initWithRootViewController:newSafeVC];
      nav.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
      if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        nav.modalPresentationStyle = UIModalPresentationFormSheet;
      [self presentModalViewController:nav animated:YES];
      A77Log(@"Editing safe password");
      
      return;
    }
    if(buttonIndex == 1)
    {
      // Safe password policies
      EditSafePwPoliciesVC *vc = [[EditSafePwPoliciesVC alloc]initWithSafeFile:m_file handler:^(EditSafePwPoliciesVC *vc) {
        [self dismissModalViewControllerAnimated:YES];
      }];
      UINavigationController *nav = [[UINavigationController alloc]initWithRootViewController:vc];
      nav.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
      if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        nav.modalPresentationStyle = UIModalPresentationFormSheet;
      [self presentModalViewController:nav animated:YES];
      A77Log(@"Editing safe password policies");
      
      return;
    }
    if(buttonIndex == 2)
    {
      // Safe history
      PwHistoryVC *vc = [[PwHistoryVC alloc]initWithSafeFile:m_file];
      vc.delegate = self;
      
      UINavigationController *nav = [[UINavigationController alloc]initWithRootViewController:vc];
      nav.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
      if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        nav.modalPresentationStyle = UIModalPresentationFormSheet;
      [self presentModalViewController:nav animated:YES];
      A77Log(@"Editing safe pass history settings");
      
      return;
    }
#ifndef PWSAFE_NO_MOBIUS
    if(buttonIndex == 3)
    {
      // Register with Mobius
      [[MobiusSDK sharedMobius]registerAccount:m_safeCtx.safeUUID name:m_safeCtx.safeName secret:m_safeCtx.password];
    }
#endif
  }
  else if(actionSheet.tag == FILE_VC_ACTIONSHEET_DELETE_TAG)
  {
    if(buttonIndex == 0)
      [self deleteGroup];
  }
  else if(actionSheet.tag == FILE_VC_ACTIONSHEET_SHARE_TAG)
  {
    if(buttonIndex == 0)
      [self printSafe];
    else
    {
#ifdef PWSAFE_NO_EMAIL_SAFE
      buttonIndex++;
#else
      if(buttonIndex == 1)
        [self emailSafe];
      else
#endif
      if(buttonIndex == 2)
        [self emailSafeAsPDF];
      else if(buttonIndex == 3)
        [self exportPDFToOtherApp];
    }
  }
}

- (void)onSafeEdit:(id)sender
{
  NewEditSafeViewController *newSafeVC = (NewEditSafeViewController *)sender;
  if(newSafeVC.result)
  {
    NSString *newPassword = newSafeVC.safePassword;
    if(![newPassword isEqualToString:m_safeCtx.password])
    {
      m_writingMyself = YES;
      [m_file changePassword:newPassword];
      m_safeCtx.password = newPassword;
    }
  }
  
  [self dismissModalViewControllerAnimated:YES];
}

#pragma mark -
#pragma mark Initialization

static void *kSafeNameObservationCtx = &kSafeNameObservationCtx;

- (id)initWithSafeCtx:(SafeCtx *)ctx safeFile:(SafeFile *)file state:(NSMutableDictionary *)state
{
  self = [super initWithStyle:UITableViewStyleGrouped];
  if(!self)
    return nil;
  
  m_safeCtx = ctx;
  m_file = file;
  m_state = state;
  m_safeCtx.opened = YES;

  file.delegate = self;
  
  self.title = m_safeCtx.safeName;
  [m_safeCtx addObserver:self forKeyPath:@"safeName" options:NSKeyValueObservingOptionNew context:kSafeNameObservationCtx];
  
  // toolbar
  UIBarButtonItem *spacer = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
  UIBarButtonItem *closeBtn = [UIBarButtonItem buttonWithImageNamed:imageName(@"Close") target:self action:@selector(onCloseSafe:)];
  closeBtn.accessibilityLabel = @"Close";
  UIBarButtonItem *shareBtn = [UIBarButtonItem buttonWithImageNamed:imageName(@"Action") target:self action:@selector(onShareSafe:)];
  shareBtn.accessibilityLabel = @"Action";
  UIBarButtonItem *settingsBtn = [UIBarButtonItem buttonWithImageNamed:imageName(@"Settings") target:self action:@selector(onSettings:)];
  settingsBtn.accessibilityLabel = @"Settings";
  UIBarButtonItem *addBtn = [UIBarButtonItem buttonWithImageNamed:imageName(@"Plus") target:self action:@selector(onAddNewItem:)];
  addBtn.accessibilityLabel = @"Add";
  addBtn.style = UIBarButtonItemStylePlain;
  self.toolbarItems = @[closeBtn, spacer, shareBtn, spacer, settingsBtn, spacer, addBtn];
  
  return self;
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
  if(context != kSafeNameObservationCtx)
  {
    [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    return;
  }
  self.title = change[NSKeyValueChangeNewKey];
}

- (void)loadSavedState
{
  NSNumber *isGroupOpened = m_state[kIsGroupOpened];
  if(!isGroupOpened || ![isGroupOpened boolValue])
    return;
  NSString *groupName = m_state[kOpenedGroup];
  
  GroupViewController *groupViewController = [[GroupViewController alloc] initWithFile:m_file ctx:m_safeCtx group:groupName searchMode:NO state:m_state];
  // ...
  // Pass the selected object to the new view controller.
  [self.navigationController pushViewController:groupViewController animated:NO];
  
  [groupViewController loadSavedState];
  
}

#pragma mark - Reload TV

- (void)reloadTableViewPreservingSelection
{
  UITableView *tv = self.tableView;
  BOOL iPad = UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad;
  NSIndexPath *selectedIdx = nil;
  NSString *selectedGroup = nil;
  if(iPad)
  {
    selectedIdx = tv.indexPathForSelectedRow;
    DebugLog(@"Selected IDX = %ld/%ld", (long)selectedIdx.section, (long)selectedIdx.row);
    if(selectedIdx.section)
      selectedGroup = [self titleForCellAtIndexPath:selectedIdx forHumans:NO];
  }
  
  [tv reloadData];
  
  if(iPad)
  {
    if(selectedIdx.section)
    {
      NSInteger row = 0;
      for(NSString *group in m_file.sortedGroups)
      {
        if(group.length == 0)
          continue;
        if([group isEqualToString:selectedGroup])
        {
          selectedIdx = [NSIndexPath indexPathForRow:row inSection:1];
          break;
        }
        row++;
      }
    }
    
    if(selectedIdx)
    {
      DebugLog(@"Restoring selected IDX = %ld/%ld", (long)selectedIdx.section, (long)selectedIdx.row);
      [tv selectRowAtIndexPath:selectedIdx animated:NO scrollPosition:UITableViewScrollPositionNone];
    }
  }
  
}

#pragma mark - Safe file delegate

-(void)safeFileChangesWrittenToDisk:(SafeFile *)file
{
  if(!file.lastWriteError && !m_writingMyself)
    [self reloadTableViewPreservingSelection];
  [m_currentGroupVC safeFileWasInternallyModified];
  m_writingMyself = NO;
}

#pragma mark -
#pragma mark View lifecycle

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)viewWillAppear:(BOOL)animated 
{
  [self adjustEditBtnVisibilityAnimated:NO];
  [m_state removeObjectForKey:kOpenedGroup];
  m_state[kIsGroupOpened] = @NO;
  m_currentGroupVC = nil;
  [super viewWillAppear:animated];
  [self.navigationController setToolbarHidden:NO animated:animated];
}

-(void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  
  [self adjustAddHelpText:kHelpText viewTagBase:kHelpBaseTag show:[self tableView:self.tableView numberOfRowsInSection:1] == 0];
  
  if(!m_hasAlreadyAppeared)
  {
    m_hasAlreadyAppeared = YES;
    BOOL openInAllItems = (m_file.entries.count == 1 && [(m_file.entries)[@""]count] == 0) ||
                          UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || 
                          [[NSUserDefaults standardUserDefaults] integerForKey:@"openInAllItems"];
    
    if(openInAllItems)
    {
      NSIndexPath *allItemsPath = [NSIndexPath indexPathForRow:(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ? 0 : 1) inSection:0];
      [self.tableView selectRowAtIndexPath:allItemsPath animated:YES scrollPosition:UITableViewScrollPositionTop];
      [self tableView:self.tableView didSelectRowAtIndexPath:allItemsPath];
    }
    
    if(m_initialGroup.length)
    {
      NSUInteger idx = [m_file.sortedGroups indexOfObject:m_initialGroup];
      if(idx != NSNotFound)
      {
        NSAssert(idx > 0, @"0 is the index for the root, unnamed group");
        NSIndexPath *idxPath = [NSIndexPath indexPathForRow:idx - 1 inSection:1];
        [self.tableView selectRowAtIndexPath:idxPath animated:YES scrollPosition:UITableViewScrollPositionTop];
        [self tableView:self.tableView didSelectRowAtIndexPath:idxPath];
        if(m_initialSearchTerm.length)
          m_currentGroupVC.searchDisplayController.searchBar.text = m_initialSearchTerm;
      }
    }
    
    NSUInteger nEntries = 0;
    for(NSArray *entries in m_file.entries.allValues)
      nEntries += entries.count;    
    
    A77Log(@"Opened safe with %lu entries", (unsigned long)nEntries);
    
    if(nEntries > 300)
      showBackendMessage(@"openBigSafe", nil);
    
#ifndef PWSAFE_NO_MOBIUS
    BOOL mobiusInstalled = [MobiusSDK isMobiusEngineAppInstalled];
    NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
    NSString *safeUUID = m_safeCtx.safeUUID;
    
    if(mobiusInstalled && ![[MobiusSDK sharedMobius]isAccountRegistered:safeUUID])
    {
      NSArray *safesForWhichRegisterHasBeenOfered = [prefs objectForKey:kMobiusSafesRegOffer];
      if(safesForWhichRegisterHasBeenOfered && [safesForWhichRegisterHasBeenOfered containsObject:safeUUID])
        return;
      
      UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"Register with Mobius" message:@"To register this safe with Mobius Engine, so that it can be opened by facial recognition, tap the gear icon below and choose Register with Mobius." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
      [alert show];
      if(!safesForWhichRegisterHasBeenOfered)
        safesForWhichRegisterHasBeenOfered = [NSArray arrayWithObject:safeUUID];
      else
        safesForWhichRegisterHasBeenOfered = [safesForWhichRegisterHasBeenOfered arrayByAddingObject:safeUUID];
      
      [prefs setObject:safesForWhichRegisterHasBeenOfered forKey:kMobiusSafesRegOffer];
      
      return;
    }
    
    BOOL alreadyViewed = [MobiusAdVc mobiusAdAlreadyViewed];
    
    if(alreadyViewed || mobiusInstalled)
    {
      A77Log(@"Mobius: already shown, not showing again");
      return;
    }
    
    NSInteger nSkipped = [prefs integerForKey:kMobiusNSkipped];
    if(nSkipped < 2)
    {
      nSkipped++;
      [prefs setInteger:nSkipped forKey:kMobiusNSkipped];
      A77Log(@"Mobius: skipped %ld time, skipping again", (long)nSkipped - 1);
      return;
    }
    
    A77Log(@"Mobius: reached time to see it, get a ticket");
    
    [PFCloud callFunctionInBackground:@"canShowAd" withParameters:@{@"adName":@"mobius"} block:^(NSDictionary *ret, NSError *error) {
      if(!ret || ![ret isKindOfClass:[NSDictionary class]] || ![ret[@"allowed"]boolValue])
      {
        A77Log(@"Could not get auth to run mobius ad: %@ / %@", ret, error);
        return;
      }
      
      MobiusAdVc *ad = [[MobiusAdVc alloc]init];
      ad.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
      if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        ad.modalPresentationStyle = UIModalPresentationFormSheet;
      [self presentModalViewController:ad animated:YES];
    }];
    
#endif
  }
  
  if(m_blockToRunAfterViewAppears)
  {
    void (^block)() = m_blockToRunAfterViewAppears;
    m_blockToRunAfterViewAppears = nil;
    block();
  }
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  [self.tableView themeViewDidLoadInIpadPos:TVP_LEFT];
  UIBarButtonItem *backButton = [[UIBarButtonItem alloc] initWithTitle:@"Groups" style:UIBarButtonItemStyleBordered target:nil action:nil];
  self.navigationItem.backBarButtonItem = backButton;
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
  [super didRotateFromInterfaceOrientation:fromInterfaceOrientation];
  [self adjustHelpFramesViewTagBase:kHelpBaseTag];
}

- (void)viewWillDisappear:(BOOL)animated
{
  [super viewWillDisappear:animated];
  [self adjustAddHelpText:kHelpText viewTagBase:kHelpBaseTag show:NO];
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated
{
  [super setEditing:editing animated:animated];
  
  if(editing)
    return;
  if(UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad)
    return;
  
  NSString *openedGroup = m_state[kOpenedGroup];
  NSIndexPath *idxPath;
  if(openedGroup == nil)
  {
    idxPath = [NSIndexPath indexPathForRow:kAllItemsRow inSection:0];
  }
  else if(openedGroup.length == 0)
  {
    idxPath = [NSIndexPath indexPathForRow:kUngroupedRowIdx inSection:0];
  }
  else
  {
    NSUInteger idx = [m_file.sortedGroups indexOfObject:openedGroup];
    NSAssert(idx >= 1 && idx != NSNotFound, @"Zero is the empty group name, i.e., ungroupped");
    idxPath = [NSIndexPath indexPathForRow:idx - 1 inSection:1];
  }
  [self.tableView selectRowAtIndexPath:idxPath animated:animated scrollPosition:UITableViewScrollPositionNone];
}

- (void) adjustEditBtnVisibilityAnimated:(BOOL)animated
{
  BOOL visible = [self tableView:self.tableView numberOfRowsInSection:1] > 0;
  [self.navigationItem setRightBarButtonItem:visible ? self.editButtonItem : nil animated:animated];
}

#pragma mark -
#pragma mark Table view data source

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath 
{
  if (editingStyle != UITableViewCellEditingStyleDelete) 
    return;

  NSAssert(indexPath.section == 1, @"Can only delete groups");
  m_groupToDelete = indexPath;
  NSString *fullGroupName = [self titleForCellAtIndexPath:indexPath forHumans:NO];
  
  NSInteger nEntries = [m_file.entries[fullGroupName]count];
  BOOL verify = nEntries > 0;
  if(!verify)
  {
    // Check for children groups
    NSString *prefix = [fullGroupName stringByAppendingString:@"."];
    verify = NSNotFound != [m_file.sortedGroups indexOfObjectPassingTest:^BOOL(NSString *groupName, NSUInteger idx, BOOL *stop) {
      *stop = [groupName rangeOfString:prefix].location == 0;
      return *stop;
    }];
  }
  
  if(verify)
  {
    // Verify deletion 
    if(m_currentActionSheet)
    {
      [m_currentActionSheet dismissWithClickedButtonIndex:m_currentActionSheet.cancelButtonIndex animated:YES];
    }

    NSString *groupName = [[fullGroupName componentsSeparatedByString:@"."]lastObject];
    
    m_currentActionSheet = [[UIActionSheet alloc]initWithTitle:[NSString stringWithFormat:@"Delete group '%@' and all its %ld %@ and subgroups? (This action cannot be undone.)", groupName, (long)nEntries, nEntries == 1 ? @"entry" : @"entries"] delegate:self cancelButtonTitle:@"Cancel" destructiveButtonTitle:@"Delete" otherButtonTitles:nil];
    m_currentActionSheet.tag = FILE_VC_ACTIONSHEET_DELETE_TAG;
    [m_currentActionSheet showFromToolbar:self.navigationController.toolbar];
  }
  else
  {
    [self deleteGroup];
  }
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView 
{
  return 2;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
  if(!section)
    return nil;
  if(section == 1)
    return @"Groups";
  return nil;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section 
{
  // Return the number of rows in the section.
  if(section == 0)
    return UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ? 2 : 3;

  // Do not unnecessarily use the sorted groups here
  NSArray *allKeys = [m_file.entries allKeys];
  if([allKeys containsObject:@""])
    return allKeys.count - 1;
  else
    return allKeys.count;
}


- (NSString *)titleForCellAtIndexPath:(NSIndexPath *)indexPath forHumans:(BOOL)forHumans
{
  NSInteger row = indexPath.row;
  
  if(indexPath.section == 0)
  {
    if(row == kSearchAllItemsRowIdx)
      return forHumans ? @"Search all items" : nil;
    if(row == kAllItemsRow)
      return forHumans ? @"All items" : nil;
    NSAssert2(row == kUngroupedRowIdx, @"Internal error @ %s:%d", __FILE__, __LINE__);
    return forHumans ? @"Ungrouped" : @"";
  }
  NSAssert2(indexPath.section == 1, @"Internal error @ %s:%d", __FILE__, __LINE__);
  int n = 0;
  static const unichar arrowChar = 0x21B3;
  NSString *arrow = [NSString stringWithCharacters:&arrowChar length:1];
  NSString *blank = @"  ";
  
  for(NSString *group in m_file.sortedGroups)
  {
    if([group isEqualToString:@""])
      continue;
    if(n++ != row)
      continue;
    NSArray *groupPath = [group componentsSeparatedByString:@"."];
    if(!forHumans || groupPath.count == 1)
      return group;
    NSMutableString *prefix = [NSMutableString stringWithString:@""];
    NSUInteger nBlanks = groupPath.count - 2;
    for(NSUInteger i = 0; i < nBlanks; i++)
      [prefix appendString:blank];

    return [NSString stringWithFormat:@"%@%@ %@", prefix, arrow, groupPath[groupPath.count - 1]];
  }

  NSAssert2(NO, @"Looking for non-existent group:%ld/%ld", (long)indexPath.section, (long)row);
  return nil;
}

static const void *const kOldGroupNameKey = &kOldGroupNameKey;

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath 
{
  static NSString *CellIdentifier = @"Cell";
  
  EditableSimpleTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
  if (cell == nil) 
    cell = [[EditableSimpleTableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];

  __block NSString *humanGroupName = [[self titleForCellAtIndexPath:indexPath forHumans:YES]copy];
  __block NSString *fullGroupName = [self titleForCellAtIndexPath:indexPath forHumans:NO];
  
  cell.textField.text = humanGroupName;
  NSUInteger nEntries;
  if(fullGroupName)
  {
    nEntries = [(m_file.entries)[fullGroupName]count];
  }
  else
  {
    nEntries = 0;
    for(NSArray *entries in m_file.entries.allValues)
      nEntries += entries.count;
  }

  cell.detailText = [NSString stringWithFormat:@"%lu %@", (unsigned long)nEntries, nEntries != 1 ? @"entries" : @"entry"];
  
  if(UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad)
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
  cell.accessibilityTraits = UIAccessibilityTraitButton;
  cell.isAccessibilityElement = YES;
  cell.accessibilityLabel = humanGroupName;
  NSInteger row = indexPath.row;
  NSInteger section = indexPath.section;
  if(section == 0)
  {
    cell.accessibilityHint = row == kSearchAllItemsRowIdx ? @"Search all items" :
                             row == kAllItemsRow ? @"Shows all entries." : @"Shows ungrouped entries";
  }
  else
    cell.accessibilityHint = @"Opens group.";

  __weak FileViewController *wself = self;
  __weak EditableSimpleTableViewCell *wcell = cell;

#ifdef DEBUG_MODE
  NSUInteger n_refs = CFGetRetainCount((__bridge CFTypeRef)self);
#endif
  
  void (^textChangedHandler)(NSString *text, NSString *originalText);
  textChangedHandler = ^(NSString *text, NSString *originalText) {
    FileViewController *sSelf = wself;
    EditableSimpleTableViewCell *cell = wcell;
    
    if(!sSelf || !cell)
      return;
    
    NSCAssert([originalText isEqualToString:humanGroupName], @"Must be the same"); // Use NSCAssert to avoid retaining self
//    NSCAssert(this.isEditing, @"Must be editing, otherwise setEditing:animated: is wrong");
    if([humanGroupName isEqualToString:text])
      return;
    
    NSString *newFullGroupName;
    
    if(![humanGroupName isEqualToString:fullGroupName])
    {
      // Remove leading arrow      
      static const unichar arrowChar = 0x21B3;
      NSString *arrow = [NSString stringWithCharacters:&arrowChar length:1];
      NSUInteger idx = [text rangeOfString:arrow].location;
      if(idx != NSNotFound)
        text = [text substringFromIndex:idx + 1];
      // Trim again
      text = [text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
      // Add prefix
      idx = [fullGroupName rangeOfString:@"." options:NSBackwardsSearch].location;
      NSCAssert(idx != NSNotFound, @"Must have a dot");
      newFullGroupName = [[fullGroupName substringToIndex:idx + 1]stringByAppendingString:text];
    }
    else
    {
      newFullGroupName = text;
    }

    if([fullGroupName isEqualToString:newFullGroupName])
    {
      cell.textField.text = humanGroupName;
      return;
    }
    
    sSelf->m_writingMyself = YES;
    [sSelf->m_file renameGroupFrom:fullGroupName to:newFullGroupName];
    [sSelf->m_currentGroupVC groupRenamedFrom:fullGroupName to:newFullGroupName];
    
    NSUInteger idx = [sSelf->m_file.sortedGroups indexOfObject:newFullGroupName];
    NSCAssert(idx >= 1 && idx != NSNotFound, @"Must be a non-empty group");
    NSIndexPath *idxPath = [NSIndexPath indexPathForRow:idx - 1 inSection:1];

    NSCAssert([newFullGroupName isEqualToString:[wself titleForCellAtIndexPath:idxPath forHumans:NO]], @"Must have changed name by now");
    humanGroupName = [[wself titleForCellAtIndexPath:idxPath forHumans:YES]copy];
    cell.textField.text = humanGroupName;
    cell.accessibilityLabel = humanGroupName;
    
    NSIndexPath *curIndexPath = [tableView indexPathForCell:cell];
    
    if(![idxPath isEqual:curIndexPath])
    {
      [tableView beginUpdates];
      
      [tableView moveRowAtIndexPath:curIndexPath toIndexPath:idxPath];
      
      // Move children too
      NSString *prefix = [newFullGroupName stringByAppendingString:@"."];
      NSIndexSet *children = [sSelf->m_file.sortedGroups indexesOfObjectsPassingTest:^BOOL(NSString *child, NSUInteger idx, BOOL *stop) {
        return [child rangeOfString:prefix].location == 0;
      }];
      NSUInteger n = children.count;
      for(NSUInteger i = 0; i < n; i++)
      {
        NSIndexPath *from = [NSIndexPath indexPathForRow:curIndexPath.row + i + 1 inSection:curIndexPath.section];
        NSIndexPath *to = [NSIndexPath indexPathForRow:idxPath.row + i + 1 inSection:idxPath.section];
        [tableView moveRowAtIndexPath:from toIndexPath:to];
      };
      
      [tableView endUpdates];
    }
    
    NSString *currentlyOpenedGroup = sSelf->m_state[kOpenedGroup];
    if(currentlyOpenedGroup)
    {
      NSString *newOpenedGroupName;
      
      if([currentlyOpenedGroup isEqual:fullGroupName])
      {
        newOpenedGroupName = newFullGroupName;
      }
      else if(0 == [currentlyOpenedGroup rangeOfString:[fullGroupName stringByAppendingString:@"."]].location)
      {
        // Current group is son of fullGroupName
        newOpenedGroupName = [newFullGroupName stringByAppendingString:[currentlyOpenedGroup substringFromIndex:fullGroupName.length]];
      }
    
      if(newOpenedGroupName)
      {
        sSelf->m_state[kOpenedGroup] = newOpenedGroupName;
        if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        {
          NSUInteger idx = [sSelf->m_file.sortedGroups indexOfObject:newOpenedGroupName];
          NSCAssert(idx >= 1 && idx != NSNotFound, @"Must be a non-empty group");
          NSIndexPath *idxPath = [NSIndexPath indexPathForRow:idx - 1 inSection:1];
          [wself.tableView selectRowAtIndexPath:idxPath animated:YES scrollPosition:UITableViewScrollPositionNone];
        }
      }
    }
    
    fullGroupName = newFullGroupName;
    
  };
  [cell setTextChanged:textChangedHandler];
  
#ifdef DEBUG_MODE
  NSAssert(n_refs == CFGetRetainCount((__bridge CFTypeRef)self), @"Above block should not retain self");
#endif

  [cell setTextValidator:^BOOL(NSString *text) {
    FileViewController *sself = wself;
    if(!sself)
      return NO;

    if([fullGroupName isEqualToString:text])
      return YES;
    // Don't allow repeated group name
    if([sself->m_file.entries.allKeys containsObject:text])
      return NO;
    
    // Don't allow dots
    return [text rangeOfString:@"."].location == NSNotFound;
  }];
  
#ifdef DEBUG_MODE
  NSAssert(n_refs == CFGetRetainCount((__bridge CFTypeRef)self), @"Above block should not retain self");
#endif

  [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];

  return cell;
}


- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
  return indexPath.section;
}

#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath 
{
  DebugLog(@"Selected %ld/%ld", (long)indexPath.section, (long)indexPath.row);
  // Navigation logic may go here. Create and push another view controller.
  NSString *groupName = [self titleForCellAtIndexPath:indexPath forHumans:NO];
  DebugLog(@"Group name is %@", groupName);
  BOOL startInSearchMode = (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad && indexPath.row == 0 && indexPath.section == 0) || m_initialSearchTerm.length != 0;

  if(groupName)
    m_state[kOpenedGroup] = groupName;
  else
    [m_state removeObjectForKey:kOpenedGroup];
  
  DebugLog(@"State updated");
  
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
  {
    DetailViewController *detailVC = [DetailViewController sharedDetailViewControllerWithState:m_state];
    UIViewController *currentVC = detailVC.viewControllers.lastObject;
    [currentVC dismissModalViewControllerAnimated:YES];
    [detailVC popToRootViewControllerAnimated:YES];
    m_currentGroupVC = [detailVC groupViewController];
    [m_currentGroupVC setSafeFile:m_file ctx:m_safeCtx group:groupName searchMode:startInSearchMode];
    [m_currentGroupVC reload];
  }
  else
  {
    // Pass the selected object to the new view controller.
    m_currentGroupVC = [[GroupViewController alloc] initWithFile:m_file ctx:m_safeCtx group:groupName searchMode:startInSearchMode state:m_state];
    [self.navigationController pushViewController:m_currentGroupVC animated:YES];
  }

  m_state[kIsGroupOpened] = @YES;
  
  DebugLog(@"All done");
}

#pragma mark - Share

static BOOL canOpenFileWithExtension(NSString *extension, UIView *view)
{
  UIDocumentInteractionController *controller = [UIDocumentInteractionController interactionControllerWithURL:[NSURL fileURLWithPath:[NSString stringWithFormat:@"file.%@", extension]]];
  BOOL ok = [controller presentOpenInMenuFromRect:CGRectMake(-500, -500, 2, 2) inView:view animated:NO];
  [controller dismissMenuAnimated:NO];
  return ok;
}

- (void)onShareSafe:(id)sender
{
  if(m_currentActionSheet)
  {
    [m_currentActionSheet dismissWithClickedButtonIndex:m_currentActionSheet.cancelButtonIndex animated:YES];
  }
  
  m_currentActionSheet = [[UIActionSheet alloc]init];
  [m_currentActionSheet addButtonWithTitle:@"Print"];
#ifndef PWSAFE_NO_EMAIL_SAFE
  [m_currentActionSheet addButtonWithTitle:@"Email Safe"];
#endif
  [m_currentActionSheet addButtonWithTitle:@"Email PDF"];
  if(canOpenFileWithExtension(@"pdf", self.view))
    [m_currentActionSheet addButtonWithTitle:@"Open PDF in..."];
  [m_currentActionSheet addButtonWithTitle:@"Cancel"];
  m_currentActionSheet.cancelButtonIndex = m_currentActionSheet.numberOfButtons - 1;
  m_currentActionSheet.delegate = self;
  
  m_currentActionSheet.tag = FILE_VC_ACTIONSHEET_SHARE_TAG;
  [self presentCurrentActionSheet:sender];
}

- (void)printSafe
{
  UIPrintInteractionController *printController = [UIPrintInteractionController sharedPrintController];
  printController.printPageRenderer = [[SafeFilePageRenderer alloc]initWithSafeFile:m_file name:m_safeCtx.safeName];
  printController.showsPageRange = NO;
  
  UIPrintInfo *printInfo = [UIPrintInfo printInfo];
  printInfo.outputType = UIPrintInfoOutputGeneral;
  printInfo.orientation = UIPrintInfoOrientationPortrait;
  printInfo.jobName = m_safeCtx.safeName;
  
  printController.printInfo = printInfo;
  
  UIPrintInteractionCompletionHandler handler = ^(UIPrintInteractionController *printInteractionController, BOOL completed, NSError *error) {
    if(completed)
    {
      A77Log(@"Safe printed");
      return;
    }
    A77Log(@"Safe print failure");
    A77Log(@"Failed to print: %@", error);
    UIView *rootView = [MBProgressHUD rootView];
    MBProgressHUD *hud = [[MBProgressHUD alloc]initWithView:rootView];
    hud.customView = [[UIImageView alloc]initWithImage:[UIImage imageNamed:@"Bad.png"]];
    hud.mode = MBProgressHUDModeCustomView;
    hud.labelText = @"Failed to print";
    hud.detailsLabelText = error.localizedDescription;
    hud.animationType = MBProgressHUDAnimationZoom;
    hud.yOffset = -50;
    hud.opacity = .8;
    hud.removeFromSuperViewOnHide = YES;
    [rootView addSubview:hud];
    [hud show:YES];
    uint32_t secs = UIAccessibilityIsVoiceOverRunning() ? 3 : 1;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, secs * NSEC_PER_SEC);
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
      [hud hide:YES];
    });
  };
  
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    [printController presentFromBarButtonItem:self.toolbarItems[2] animated:YES completionHandler:handler];
  else
    [printController presentAnimated:YES completionHandler:handler];
}

#ifndef PWSAFE_NO_EMAIL_SAFE

- (void)emailSafe
{
  NSString *filePath = m_file.safePath;
  [m_file waitForSavingComplete];
  [self emailFile:filePath withMimeType:@"application/octet-stream" subject:[@"pwSafe file: " stringByAppendingString:m_safeCtx.safeName]];
}

#endif

- (void)emailSafeAsPDF
{
  [self makeTempPDF:^(NSString *pdfPath) {
    if(!pdfPath)
      return;

    void (^sendEmail)() = ^{
#ifdef PWSAFE_VAULT
      NSString *safeStr = @"vault";
#else
      NSString *safeStr = @"safe";
#endif
      [self emailFile:pdfPath withMimeType:@"application/pdf" subject:[NSString stringWithFormat:@"PDF of %@ %@", safeStr, m_safeCtx.safeName]];
      [[NSFileManager defaultManager]removeItemAtPath:pdfPath error:nil];
    };
    
    if(self.modalViewController)
    {
      if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone)
      {
        NSAssert(!m_blockToRunAfterViewAppears, @"Can't have another one");
        m_blockToRunAfterViewAppears = [sendEmail copy];
      }
      else
      {
        void (__block ^trySendMail)() = [^
        {
          if(self.modalViewController)
          {
            DebugLog(@"Not yet");
            double delayInSeconds = 0.05;
            dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-retain-cycles"
            dispatch_after(popTime, dispatch_get_main_queue(), trySendMail);
#pragma clang diagnostic pop
          }
          else
          {
            sendEmail();
            NSAssert(CFGetRetainCount((__bridge CFTypeRef)trySendMail) == 1, @"Must free the block");
            trySendMail = nil;
          }
        } copy];
        trySendMail();
      }
    }
    else
    {
      sendEmail();
    }
  }];
}

- (void)exportPDFToOtherApp
{
  [self makeTempPDF:^(NSString *pdfPath) {
    if(!pdfPath)
      return;
    void (^openIn)() = ^{
      m_docIntControllerForOpenIn = [UIDocumentInteractionController interactionControllerWithURL:[NSURL fileURLWithPath:pdfPath]];
      m_docIntControllerForOpenIn.delegate = self;
      [m_docIntControllerForOpenIn presentOpenInMenuFromBarButtonItem:self.toolbarItems[2] animated:YES];
    };
    if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone && self.modalViewController)
    {
      NSAssert(!m_blockToRunAfterViewAppears, @"Can't have another one");
      m_blockToRunAfterViewAppears = [openIn copy];
    }
    else
    {
      openIn();
    }
  }];
}

- (void)documentInteractionControllerDidDismissOpenInMenu:(UIDocumentInteractionController *)controller
{
  A77Log(@"Dismissed open in menu");
  m_docIntControllerForOpenIn = nil;
}

-(void)documentInteractionController:(UIDocumentInteractionController *)controller willBeginSendingToApplication:(NSString *)application
{
  A77Log(@"Open as PDF in another app");
  A77Log(@"Will open as PDF in %@", application);
}

-(void)documentInteractionController:(UIDocumentInteractionController *)controller didEndSendingToApplication:(NSString *)application
{
  [[NSFileManager defaultManager]removeItemAtURL:controller.URL error:nil];
  m_docIntControllerForOpenIn = nil;
}

// This functions reports errors itself
- (void)makeTempPDF:(void (^)(NSString *tempPDFPath))callback
{
  NSString *tempDir = NSTemporaryDirectory();
  NSString *tempFilePath = [tempDir stringByAppendingPathComponent:[m_safeCtx.safeName stringByAppendingPathExtension:@"pdf"]];
  [[NSFileManager defaultManager]removeItemAtPath:tempFilePath error:nil];
  
  NSString *countryCode = [[[NSLocale currentLocale]objectForKey:NSLocaleCountryCode]lowercaseString];
  CGSize pageSize;
  if([countryCode isEqualToString:@"us"] || [countryCode isEqualToString:@"ca"] || [countryCode isEqualToString:@"mx"])
    pageSize = CGSizeMake(8.5 * 72, 11 * 72); // Letter
  else
    pageSize = CGSizeMake(8.26772 * 72, 11.6929 * 72); // A4
  
  NewPDFPasswordVC *pdfPassVC = [[NewPDFPasswordVC alloc]initWithTitle:@"PDF Password" handler:^(NewPDFPasswordVC *vc) {
    [self dismissModalViewControllerAnimated:YES];
    
    if(!vc.result)
    {
      A77Log(@"User gave no password for PDF");
      return;
    }
    
    [m_file saveAsPDFToPath:tempFilePath password:vc.password name:m_safeCtx.safeName pageSize:pageSize callback:^(BOOL ok, NSError *error, NSString *pdfPath) {
      if(!ok)
      {
        A77Log(@"Error rendering temporary PDF");
        A77Log(@"Error rendering temporary PDF: %@", error);
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"Error"
                                                       message:[NSString stringWithFormat:@"Error rendering temporary PDF file: %@", error.localizedDescription]
                                                      delegate:nil cancelButtonTitle:@"OK"
                                             otherButtonTitles: nil];
        
        [alert show];
        callback(nil);
        return;
      }
      callback(tempFilePath);
    }];
  }];
  UINavigationController *nav = [[UINavigationController alloc]initWithRootViewController:pdfPassVC];
  nav.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
  
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    nav.modalPresentationStyle = UIModalPresentationFormSheet;
  [self presentModalViewController:nav animated:YES];
}

- (void)emailFile:(NSString *)path withMimeType:(NSString *)mime subject:(NSString *)subject
{
  MFMailComposeViewController *mailViewController = [[MFMailComposeViewController alloc]init];
  if(![MFMailComposeViewController canSendMail] || !mailViewController)
  {
    NSString *deviceName = UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ? @"iPad" : @"iPhone";
    UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"No Email Acount" message:[NSString stringWithFormat:@"You need to configure your email account in the %@ Settings app.", deviceName] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
    
    return;
  }
  
  NSError *error = nil;

  m_attachmentTypeInEmail = path.pathExtension;
  
  NSData *attachmentData = [NSData dataWithContentsOfFile:path options:0 error:&error];
  if(!attachmentData)
  {
    A77Log(@"Failed to attach file of type %@ to email", m_attachmentTypeInEmail);
    A77Log(@"Failed to attach file of type %@ to email: %@", path.pathExtension, error);
    UIView *rootView = [MBProgressHUD rootView];
    MBProgressHUD *hud = [[MBProgressHUD alloc]initWithView:rootView];
    hud.customView = [[UIImageView alloc]initWithImage:[UIImage imageNamed:@"Bad.png"]];
    hud.mode = MBProgressHUDModeCustomView;
    hud.labelText = error.localizedDescription;
    hud.animationType = MBProgressHUDAnimationZoom;
    hud.yOffset = -50;
    hud.opacity = .8;
    hud.removeFromSuperViewOnHide = YES;
    [rootView addSubview:hud];
    [hud show:YES];
    uint32_t secs = UIAccessibilityIsVoiceOverRunning() ? 3 : 1;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, secs * NSEC_PER_SEC);
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
      [hud hide:YES];
    });
    return;
  }
  NSString *fileName = path.lastPathComponent;
  [mailViewController addAttachmentData:attachmentData mimeType:mime fileName:fileName];
  [mailViewController setSubject:subject];
#ifdef PWSAFE_COZI
  [mailViewController setMessageBody:@"Sent with Cozi Password Vault - http://itunes.apple.com/app/cozi-password-vault/id553658546?mt=8" isHTML:NO];
#else
  [mailViewController setMessageBody:@"Sent with pwSafe - http://j.mp/pwSafe" isHTML:NO];
#endif
  
  mailViewController.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    mailViewController.modalPresentationStyle = UIModalPresentationPageSheet;
  mailViewController.mailComposeDelegate = self;
  
  if(iOS6())
  {
    [self presentModalViewController:mailViewController animated:YES];
  }
  else
  {
    [self presentViewController:mailViewController animated:YES completion:^{
      [[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleLightContent];
    }];
  }
}

- (void)mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error
{
  NSString *resultStr;
  switch(result)
  {
    case MFMailComposeResultSent:
      resultStr = @"sent";
      break;
    case MFMailComposeResultCancelled:
      resultStr = @"cancelled";
      break;
    case MFMailComposeResultSaved:
      resultStr = @"saved";
      break;
    case MFMailComposeResultFailed:
      resultStr = @"failed";
      A77Log(@"Error sending email with attachment of type %@: %@", m_attachmentTypeInEmail, error);
      break;
  }
  A77Log(@"Email %@ with attachement of type %@", resultStr, m_attachmentTypeInEmail);
  [self dismissModalViewControllerAnimated:YES];
}

#pragma mark -
#pragma mark Memory management

- (void)viewDidUnload 
{
  [super viewDidUnload];
  
  m_currentActionSheet.delegate = nil;
  [m_currentActionSheet dismissWithClickedButtonIndex:[m_currentActionSheet cancelButtonIndex] animated:NO];
  m_currentActionSheet = nil;
}

- (void)dealloc 
{
  m_file.delegate = nil;
  m_currentActionSheet.delegate = nil;
  [m_safeCtx removeObserver:self forKeyPath:@"safeName"];
  if(self.isViewLoaded)
  {
    self.tableView.delegate = nil;
    self.tableView.dataSource = nil;
  }
}

@end

