//
//  InactiveScreenCover.m
//  pwSafe
//
//  Created by Jorge Peixoto Vasquez on 09/07/12.
//  Copyright (c) 2012 app77. All rights reserved.
//

#import "InactiveScreenCover.h"
#import "AppTheme.h"

@implementation InactiveScreenCover
{
  UIView *m_inactiveScreenView;
}

@synthesize disableHideScreenOnInactive = m_disableHideScreenOnInactive;
@synthesize imageName = m_imageName;
@synthesize landscapeImageName = m_landscapeImageName;
@synthesize imageMargin = m_imageMargin;
@synthesize imageMode = m_imageMode;

+ (InactiveScreenCover *)sharedCover
{
  static InactiveScreenCover *cover = nil;
  if(!cover)
    cover = [[InactiveScreenCover alloc]init];
  return cover;
}

- (id)init
{
  if(!(self = [super init]))
    return nil;
  
  [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(appDidBecomeActive:) name:UIApplicationDidBecomeActiveNotification object:[UIApplication sharedApplication]];
  [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(appWillResignActive:) name:UIApplicationWillResignActiveNotification object:[UIApplication sharedApplication]];
  m_imageMode = UIViewContentModeScaleAspectFit;
  
  return self;
}

- (void)appDidBecomeActive:(NSNotification *)notif
{
  if(m_inactiveScreenView)
  {
    UIView *inactiveScreen = m_inactiveScreenView;
    m_inactiveScreenView = nil;
    
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, .3 * NSEC_PER_SEC);
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
      [UIView animateWithDuration:.3 animations:^{
        inactiveScreen.alpha = 0;
      } completion:^(BOOL finished) {
        [inactiveScreen removeFromSuperview];
      }];
    });
  }
}

- (void)appWillResignActive:(NSNotification *)notif
{
  if(m_disableHideScreenOnInactive)
    return;
  
  NSAssert(!m_inactiveScreenView, @"Must not exist");
  UIWindow *window = ([UIApplication sharedApplication].windows)[0];
  UIView *rootView = window.rootViewController.view;
  NSAssert(rootView, @"Must have a root view");
  CGRect frame = rootView.frame;
  m_inactiveScreenView = [[UIView alloc]init];
  m_inactiveScreenView.backgroundColor = kThemeDarkBgColor;
  m_inactiveScreenView.opaque = YES;
  CGAffineTransform transform = rootView.transform;
  m_inactiveScreenView.transform = transform; // Rotation transform
  m_inactiveScreenView.frame = frame;
  
  int nHalfPis = rint(atan2(transform.b, transform.a) / M_PI_2);
  BOOL rotated = nHalfPis & 1;

  NSString *imageName = (rotated && m_landscapeImageName) ? m_landscapeImageName : m_imageName;
  UIImageView *imgView = [[UIImageView alloc]initWithImage:[UIImage imageNamed:imageName]];
  if(rotated)
  {
    CGRect auxFrame;
    auxFrame.size.width = frame.size.height;
    auxFrame.size.height = frame.size.width;
    auxFrame.origin = frame.origin;
    frame = auxFrame;
  }

  frame.origin.x = m_imageMargin.left;
  frame.origin.y = m_imageMargin.top;  
  frame.size.width -= m_imageMargin.left + m_imageMargin.right;
  frame.size.height -= m_imageMargin.top + m_imageMargin.bottom;
  imgView.opaque = NO;
  imgView.backgroundColor = nil;
  imgView.frame = frame;
  imgView.contentMode = m_imageMode;
  [m_inactiveScreenView addSubview:imgView];
  
  [window addSubview:m_inactiveScreenView];
}

@end
