//
//  KeepAliveTimePickerViewController.m
//  iPasswordSafe
//
//  Created by Jorge Peixoto Vasquez on 3/10/11.
//  Copyright 2011 app77. All rights reserved.
//

#import "KeepAliveTimePickerViewController.h"
#import "SafeCtx.h"
#import "AppTheme.h"

@implementation KeepAliveTimePickerViewController

static const int32_t KeepAliveOptions[] =
{
  0, // No keep alive
  1, 2, 5, 10, 15, 30, 60, 120, 360, 720, 1440, 2880, 4320,
};

static const uint KeepAliveOptionsCount = sizeof(KeepAliveOptions) / sizeof(KeepAliveOptions[0]);

@synthesize safeCtx;

- (id)initWithSafeCtx:(id)ctx
{
  self = [super initWithStyle:UITableViewStyleGrouped];
  if (!self)
    return nil;
  safeCtx = ctx;
  
#ifdef PWSAFE_VAULT
  self.navigationItem.title = @"Keep vault open for";
#else
  self.navigationItem.title = @"Keep safe open for";
#endif
  
  return self;
}

- (void)dealloc
{
  if(self.isViewLoaded)
  {
    self.tableView.delegate = nil;
    self.tableView.dataSource = nil;
  }
}

- (void)didReceiveMemoryWarning
{
  // Releases the view if it doesn't have a superview.
  [super didReceiveMemoryWarning];
  
  // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)viewWillAppear:(BOOL)animated
{
  for(int i = 0; i < KeepAliveOptionsCount; i++)
  {
    if(safeCtx.keepAlivePeriod == KeepAliveOptions[i])
    {
      [self.tableView reloadData];
      [self.tableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:i inSection:0] atScrollPosition:UITableViewScrollPositionMiddle animated:animated];
      return;
    }
  }
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  [self.tableView themeViewDidLoadInIpadPos:TVP_FORMSHEET];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  return KeepAliveOptionsCount;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  static NSString *CellIdentifier = @"Cell";
  
  UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
  if (cell == nil) 
    cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];

  int32_t rowMinutes = KeepAliveOptions[indexPath.row];
  cell.accessoryType = safeCtx.keepAlivePeriod == rowMinutes ? UITableViewCellAccessoryCheckmark : UITableViewCellAccessoryNone;
  NSString *text;
  if(rowMinutes == 0)
    text = @"Never";
  else if(rowMinutes == 1)
    text = @"1 minute";
  else if(rowMinutes < 60)
    text = [NSString stringWithFormat:@"%d minutes", rowMinutes];
  else if(rowMinutes == 60)
    text = @"1 hour";
  else
    text = [NSString stringWithFormat:@"%d hours", rowMinutes / 60];
  
  cell.textLabel.text = text;

  [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];
  
  return cell;
}

- (NSString *)tableView:(UITableView *)tableView titleForFooterInSection:(NSInteger)section
{
#ifdef PWSAFE_COZI
  return @"This is the maximum keep-alive period. If iOS decides to remove Cozi Password Vault from memory before the selected amount of time, you'll have to type your password again. Your password won't be saved to permanent storage.";
#else
  return @"This is the maximum keep-alive period. If iOS decides to remove pwSafe from memory before the selected amount of time, and you haven't set a master password, you'll have to type your password again. Your password won't be saved to permanent storage unless you have a master password which can be used as a cryptographic key.";
#endif
}


#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  uint rowMinutes = KeepAliveOptions[indexPath.row];
  safeCtx.keepAlivePeriod = rowMinutes;
  [self.navigationController popViewControllerAnimated:YES];
}

@end
