//
//  MacAppAdd.m
//  pwSafe
//
//  Created by Jorge Peixoto Vasquez on 31/05/12.
//  Copyright (c) 2012 app77. All rights reserved.
//

#import "MacAppAdd.h"
#import "GradientView.h"
#import "pwSafeTheme.h"

#import <MessageUI/MessageUI.h>

@interface MacAppAdd () <MFMailComposeViewControllerDelegate>
{
  MFMailComposeViewController *m_mailVC;
  UIView *m_hiddenAddView;
}
@property (weak, nonatomic) IBOutlet UIView *addView;
@property (weak, nonatomic) IBOutlet UIButton *dismissBtn;
@property (weak, nonatomic) IBOutlet UIButton *emailLinkBtn;

@end

@implementation MacAppAdd
@synthesize addView = m_addView;


- (void)viewDidLoad
{
  [super viewDidLoad];
  GradientView *view = (GradientView *)self.view;
  view.colors = @[(id)[UIColor lightGrayColor].CGColor, 
                 (id)[UIColor darkGrayColor].CGColor];
  
  if(!iOS6())
  {
    for(UIButton *btn in @[self.dismissBtn, self.emailLinkBtn])
    {
      [btn setTitleShadowColor:[UIColor darkTextColor] forState:UIControlStateNormal];
      [btn setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
      btn.titleLabel.shadowOffset = CGSizeMake(0, 1 );
    }
  }
}

- (IBAction)dismiss:(id)sender
{
  [self dismissModalViewControllerAnimated:YES];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  BOOL res = (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
  return res;
}

- (IBAction)onEmailLink:(id)sender 
{
  NSAssert(!m_mailVC, @"Only once at a time");
  m_mailVC = [[MFMailComposeViewController alloc]init];  
  if(![MFMailComposeViewController canSendMail] || !m_mailVC)
  {
    NSString *deviceName = UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ? @"iPad" : @"iPhone";
    UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"No Email Acount" message:[NSString stringWithFormat:@"You need to configure your email account in the %@ Settings app.", deviceName] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
    
    return;
  }
  m_mailVC.mailComposeDelegate = self;
  
  [m_mailVC setSubject:@"pwSafe for Mac"];
  [m_mailVC setMessageBody:@"pwSafe for Mac can be downloaded from http://j.mp/pwSafeMac" isHTML:NO];

  UIView *mailView = m_mailVC.view;
  CGSize mySize = self.view.bounds.size;
  mailView.frame = CGRectMake(0, 0, mySize.width, mySize.height);
  mailView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;

  m_hiddenAddView = m_addView;
  
  [UIView transitionFromView:m_addView toView:mailView duration:.6 options:UIViewAnimationOptionTransitionFlipFromRight completion:^(BOOL finished) {
    if(!iOS6())
      [[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleLightContent];
  }];
}

- (void)mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error
{
  UIView *mailView = controller.view;

  [UIView transitionFromView:mailView toView:m_hiddenAddView duration:.6 options:UIViewAnimationOptionTransitionFlipFromLeft completion:^(BOOL finished) {
    if(finished)
    {
      m_hiddenAddView = nil;
      m_mailVC = nil;
    }
  }];
}

@end
