//
//  MasterPasswordViewController.m
//  pwSafe
//
//  Created by Jorge Peixoto Vasquez on 12/08/11.
//  Copyright (c) 2011 app77. All rights reserved.
//

#import "MasterPasswordViewController.h"
#import "AppTheme.h"

#import <QuartzCore/QuartzCore.h>
#include <CommonCrypto/CommonCryptor.h>
#include <CommonCrypto/CommonDigest.h>

@implementation MasterPasswordViewController
{
  UITextField *m_passwordField;
  NSString *m_instructions;
  MasterPasswordCallbackT m_callback;
  UIPopoverController *m_ipadPopover;
  UIViewController *m_viewOnWhichToPresent;
  UILabel *m_instructionsLabel;
  CGSize m_size;
  BOOL m_badPassword;
}

@synthesize instructions = m_instructions;


- (id)initWithTitle:(NSString *)title instructions:(NSString *)instructions accessibilityLabel:(NSString *)accessibilityLabel allowCancel:(BOOL)allowCancel callback:(MasterPasswordCallbackT)callback
{
  self = [super init];
  if (!self) 
    return nil;

  self.navigationItem.title = title;
  m_instructions = [instructions copy];
  m_callback = [callback copy];
  if(allowCancel)
    self.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(onCancel:)];
  self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(onDone:)];
  self.navigationItem.rightBarButtonItem.enabled = NO;
  
  return self;
}

- (void)setInstructions:(NSString *)instructions
{
  m_instructions = [instructions copy];
  m_instructionsLabel.text = instructions;
}

#define MASTER_PW_WIDTH 320.0f
#define MASTER_PW_MARGIN 20.0f

- (UIFont *)passwordFont
{
  return [UIFont respondsToSelector:@selector(preferredFontForTextStyle:)] ?
  [UIFont preferredFontForTextStyle:UIFontTextStyleBody]:
  [UIFont systemFontOfSize:20];
}

- (UIFont *)instructionsFont
{
  return [UIFont respondsToSelector:@selector(preferredFontForTextStyle:)] ?
  [UIFont preferredFontForTextStyle:UIFontTextStyleSubheadline]:
  [UIFont boldSystemFontOfSize:17];;
}

- (void)configureLabel:(UILabel *)label
{
  label.font = [self instructionsFont];
  
  if(iOS6())
  {
    if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
      label.textColor = [UIColor colorWithRed:77.0/255 green:85.0/255 blue:108.0/255 alpha:1];
    else
      label.textColor = [UIColor colorWithRed:96.0/255 green:105.0/255 blue:127.0/255 alpha:1];
    label.shadowColor = [UIColor whiteColor];
    label.shadowOffset = CGSizeMake(0, 1);
  }
  else
  {
    label.textColor = [UIColor colorWithWhite:79.f/255.f alpha:1];
  }

  label.opaque = NO;
  label.backgroundColor = nil;
  label.textAlignment = UITextAlignmentCenter;
}

- (void)configurePasswordBg:(UIView *)pwBg
{
  pwBg.backgroundColor = [UIColor whiteColor];
  pwBg.opaque = YES;
  CALayer *layer = pwBg.layer;
  if(iOS6())
  {
    layer.cornerRadius = kThemeGroupedCellBgCornerRadius;
    layer.borderWidth = 1;
  }
  else
  {
    layer.borderWidth = 1 / [UIScreen mainScreen].scale;
  }
  layer.borderColor = kThemeGroupedCellBgStrokeColor.CGColor;
}

// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView
{
  UIView *view = [[UIView alloc]init];
  self.view = view;
  view.opaque = YES;
  
  view.autoresizingMask = 0;
  
  UIFont *pwFont = [self passwordFont];  
  UIFont *inFont = [self instructionsFont];
  
  CGRect frame;
  
  frame.size.width = MASTER_PW_WIDTH - 2 * MASTER_PW_MARGIN;
  frame.size.height = [@"17tq" sizeWithFont:inFont].height;
  frame.origin.x = MASTER_PW_MARGIN;
  frame.origin.y = MASTER_PW_MARGIN;
  BOOL isIpad = UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad;
  if(!iOS6())
  {
    frame.origin.y += self.navigationController.navigationBar.frame.size.height;
    if(!isIpad)
      frame.origin.y += [UIApplication sharedApplication].statusBarFrame.size.height;
  }
  
  m_instructionsLabel = [[UILabel alloc]initWithFrame:frame];
  m_instructionsLabel.text = m_instructions;
  [self configureLabel:m_instructionsLabel];
  [view addSubview:m_instructionsLabel];

  frame.origin.y += MASTER_PW_MARGIN + frame.size.height;  
  frame.size.height = [@"17tq" sizeWithFont:pwFont].height + 12.0f;
  
  if(!iOS6() && !isIpad)
  {
    CGFloat thin = 1 / [UIScreen mainScreen].scale;
    frame.origin.x = -thin;
    frame.size.width = MASTER_PW_WIDTH + 2 * thin;
  }
  
  UIView *pwBg = [[UIView alloc]initWithFrame:frame];
  [self configurePasswordBg:pwBg];
  [view addSubview:pwBg];
  
  m_passwordField = [[UITextField alloc]initWithFrame:CGRectMake(6.0f, 6.0f, frame.size.width - 12.0f, frame.size.height - 12.0f)];
  m_passwordField.secureTextEntry = YES;
  m_passwordField.borderStyle = UITextBorderStyleNone;
  m_passwordField.backgroundColor = [UIColor whiteColor];
  m_passwordField.opaque = YES;
  m_passwordField.font = pwFont;
  m_passwordField.delegate = self;
  m_passwordField.returnKeyType = UIReturnKeyGo;
  [m_passwordField addTarget:self action:@selector(onPasswordChanged:) forControlEvents:UIControlEventEditingChanged];
  [pwBg addSubview:m_passwordField];

  frame = CGRectMake(0, 0, MASTER_PW_WIDTH, frame.origin.y + frame.size.height + MASTER_PW_MARGIN);
  m_size = frame.size;
  if(!iOS6())
    m_size.height -= 44.f;
  
  view.frame = frame;

  if(iOS6() || !isIpad)
    self.view.backgroundColor = kThemeBackgroundPatternColor;
}

- (CGSize)contentSizeForViewInPopover
{
  return m_size;
}

- (void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  [m_passwordField becomeFirstResponder];
}

- (void)onCancel:(id)sender
{
  m_callback(self, nil);
}

- (void)onDone:(id)sender
{
  [self textFieldShouldReturn:m_passwordField];
}

- (void)onPasswordChanged:(id)sender
{
  self.navigationItem.rightBarButtonItem.enabled = m_passwordField.text.length > 0;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
  NSString *password = textField.text;
  if(!password || password.length == 0)
    return NO;
  m_callback(self, password);
  return NO;
}

- (void)presentInViewController:(UIViewController *)viewController
{
  m_viewOnWhichToPresent = viewController;
  
  UINavigationController *nav = [[UINavigationController alloc]initWithRootViewController:self];
  
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) 
  {
    UIView *view = viewController.view;
    m_ipadPopover = [[UIPopoverController alloc]initWithContentViewController:nav];
    
    if(!iOS6())
      nav.navigationBar.barStyle = UIBarStyleBlack;
    
    m_ipadPopover.delegate = self;
    CGSize viewSize = view.bounds.size;
    CGRect rect = CGRectMake(viewSize.width / 2 - 50, viewSize.height / 2 - 50, 100, 100);
    
    if(UIInterfaceOrientationIsLandscape(viewController.interfaceOrientation))
      rect.origin.y -= 352 / 2;
    else
      rect.origin.y -= 264 / 2;
    
    [m_ipadPopover presentPopoverFromRect:rect inView:view permittedArrowDirections:0 animated:YES];
  }
  else
  {
    nav.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
    UIViewController *modal;
    while((modal = [viewController modalViewController]))
    {
      viewController = modal;
    }
    [viewController presentModalViewController:nav animated:YES];
  }
}

- (void)dismiss
{
  [self.view endEditing:YES];
  m_viewOnWhichToPresent = nil;
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) 
  {
    [m_ipadPopover dismissPopoverAnimated:YES];
    m_ipadPopover = nil;
  }
  else
  {
    [self.navigationController dismissModalViewControllerAnimated:YES];
  }
}

- (void)badPassword
{
  m_passwordField.text = @"";  
  
  if(m_badPassword)
    return;
  
  UIColor *oldColor = m_instructionsLabel.textColor;
  m_instructionsLabel.text = @"Bad password";
  m_instructionsLabel.textColor = [UIColor colorWithRed:.6 green:0 blue:0 alpha:1];
  double delayInSeconds = 2.0;
  dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
  dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
    m_instructionsLabel.text = m_instructions;
    m_instructionsLabel.textColor = oldColor;
    m_badPassword = NO;
  });
  
  m_badPassword = YES;
} 

- (void)viewDidUnload
{
  m_passwordField = nil;
  m_instructionsLabel = nil;
  
  [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  return (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (BOOL)popoverControllerShouldDismissPopover:(UIPopoverController *)popoverController
{
  return NO;
}

- (void)popoverControllerDidDismissPopover:(UIPopoverController *)popoverController
{
  m_ipadPopover = nil;
  if(m_viewOnWhichToPresent)
  {
    dispatch_async(dispatch_get_main_queue(), ^{
      [self presentInViewController:m_viewOnWhichToPresent];
    });
  }
}

- (void)promptAgain:(NSString *)newInstructions accessibilityLabel:(NSString *)newAccessibilityLabel
{
  CGSize viewSize = self.view.frame.size; 
  
  CGRect frame = m_instructionsLabel.frame;
  frame.origin.x += viewSize.width;
  UILabel *newLabel = [[UILabel alloc]initWithFrame:frame];
  newLabel.text = newInstructions;
  [self configureLabel:newLabel];
  [self.view addSubview:newLabel];
  
  UIView *pwBg = m_passwordField.superview;
  frame = pwBg.frame;
  frame.origin.x += viewSize.width;
  UIView *newPwBg = [[UIView alloc]initWithFrame:frame];
  [self configurePasswordBg:newPwBg];
  [self.view addSubview:newPwBg];
  
  [UIView animateWithDuration:.5 delay:0 options:UIViewAnimationOptionCurveEaseInOut animations:^() {
    CGRect frame = m_instructionsLabel.frame;
    newLabel.frame = frame;
    frame.origin.x -= viewSize.width;
    m_instructionsLabel.frame = frame;
    
    frame = pwBg.frame;
    newPwBg.frame = frame;
    frame.origin.x -= viewSize.width;
    pwBg.frame = frame;
  } completion:^(BOOL finished) {
    if(!finished)
      return;
    
    [m_instructionsLabel removeFromSuperview];
    m_instructionsLabel = newLabel;
    
    [m_passwordField removeFromSuperview];
    [newPwBg addSubview:m_passwordField];
    m_passwordField.text = nil;
    m_passwordField.accessibilityLabel = newAccessibilityLabel;
    [pwBg removeFromSuperview];
    
    [m_passwordField becomeFirstResponder];
  }];
}

#pragma mark - Class methods

NSString * const kMasterPasswordEnabledKey = @"MasterPassword_Enabled";
NSString * const kMasterPasswordSaltKey = @"MasterPassword_Salt";
NSString * const kMasterPasswordHashKey = @"MasterPassword_Hash";
NSString * const kMasterPasswordEncryptionKey = @"MasterPassword_EncryptionKey";
NSString * const kMasterPasswordEncryptionKeySalt = @"MasterPassword_EncryptionKeySalt";
NSString * const kMasterPasswordExpireMinutesKey = @"masterPasswordExpireMinutes";

static NSMutableData *encryptionKey = nil;

#define MASTER_PW_SALT_LEN 16
#define MASTER_PW_HASH_ITERATIONS 128

static UIViewController *rootViewController()
{
  UIWindow *mainWindow = [[UIApplication sharedApplication]windows][0];
  return mainWindow.rootViewController;
}

+ (BOOL)isEnabled
{
  NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
  return [prefs boolForKey:kMasterPasswordEnabledKey] && [prefs objectForKey:kMasterPasswordHashKey] && [prefs objectForKey:kMasterPasswordSaltKey];
}

static NSData *newSalt()
{
  u_char saltBytes[MASTER_PW_SALT_LEN];
  
  int ret = SecRandomCopyBytes(kSecRandomDefault, MASTER_PW_SALT_LEN, saltBytes);
  if(ret)
    return nil;  

  return [NSData dataWithBytes:saltBytes length:sizeof(saltBytes)];
}

static NSData *calcHash(NSString *password, NSData *salt)
{
  NSData *passwordBytes = [password dataUsingEncoding:NSUTF8StringEncoding];
  uint8_t result[CC_SHA256_DIGEST_LENGTH];
  memset(result, 0, CC_SHA256_DIGEST_LENGTH);

  int i = MASTER_PW_HASH_ITERATIONS;
  while(i--)
  {
    CC_SHA256_CTX ctx;
    
    CC_SHA256_Init(&ctx);
    CC_SHA256_Update(&ctx, passwordBytes.bytes, (CC_LONG)passwordBytes.length);
    CC_SHA256_Update(&ctx, salt.bytes, (CC_LONG)salt.length);
    CC_SHA256_Update(&ctx, result, sizeof(result));
    CC_SHA256_Final(result, &ctx);    
  }
  
  // DebugLog(@"Hash for '%@' and salt %08x = %08x", password, ((const uint32_t *)salt.bytes)[0], ((const uint32_t *)(result))[0]);
  
  return [NSData dataWithBytes:result length:sizeof(result)];
}

static void loadEncryptionKey(NSString *password)
{
  if(encryptionKey)
  {
    memset(encryptionKey.mutableBytes, 0, encryptionKey.length);
    encryptionKey = nil;
  }
  
  NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
  
  NSData *encEncKey = [prefs objectForKey:kMasterPasswordEncryptionKey];
  NSData *salt = [prefs objectForKey:kMasterPasswordEncryptionKeySalt];
  if(!encEncKey || encEncKey.length != kCCKeySizeAES128 || !salt || salt.length == 0)
    return;

  NSData *passwordHash = calcHash(password, salt);
  
  CCCryptorRef cctx = NULL;
  CCCryptorStatus cret = CCCryptorCreate(kCCDecrypt, kCCAlgorithmAES128, kCCOptionECBMode, passwordHash.bytes, kCCKeySizeAES128, NULL, &cctx);
  if(cret != kCCSuccess)
    return;
  
  NSMutableData *newKey = [NSMutableData dataWithLength:kCCKeySizeAES128];
  if(!newKey)
  {
    CCCryptorRelease(cctx);  
    return;
  }
  
  size_t written = 0;
  cret = CCCryptorUpdate(cctx, encEncKey.bytes, kCCKeySizeAES128, newKey.mutableBytes, kCCKeySizeAES128, &written);
  if(cret != kCCSuccess)
  {
    CCCryptorRelease(cctx);  
    return;
  }
  
  int unused;
  cret = CCCryptorFinal(cctx, &unused, sizeof(unused), &written);
  CCCryptorRelease(cctx);  
  if(cret != kCCSuccess || written != 0)
  {
    memset(newKey.mutableBytes, 0, kCCKeySizeAES128);    
    return;
  }
  
  // DebugLog(@"Loaded encryption key %08x for password %@ which hashed to %08x = %08x", *(uint32_t *)newKey.bytes, password, *(uint32_t *)passwordHash.bytes, *(uint32_t *)encEncKey.bytes);
    
  encryptionKey = newKey;
}

static BOOL saveEncryptionKey(NSString *password)
{
  NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
  [prefs removeObjectForKey:kMasterPasswordEncryptionKey];
  
  if(!encryptionKey)
    return NO;
  
  NSData *salt = newSalt();
  NSData *passwordHash = calcHash(password, salt);
  
  CCCryptorRef cctx = NULL;
  CCCryptorStatus cret = CCCryptorCreate(kCCEncrypt, kCCAlgorithmAES128, kCCOptionECBMode, passwordHash.bytes, kCCKeySizeAES128, NULL, &cctx);
  if(cret != kCCSuccess)
    return NO;
  
  size_t written = 0;
  NSMutableData *encryptedKey = [NSMutableData dataWithLength:kCCKeySizeAES128];
  
  cret = CCCryptorUpdate(cctx, encryptionKey.bytes, kCCKeySizeAES128, encryptedKey.mutableBytes, kCCKeySizeAES128, &written);
  if(cret != kCCSuccess)
  {
    CCCryptorRelease(cctx);  
    return NO;
  }
  
  int unused;
  
  cret = CCCryptorFinal(cctx, &unused, sizeof(unused), &written);
  CCCryptorRelease(cctx);  
  if(cret != kCCSuccess || written != 0)
    return NO;
  
  // DebugLog(@"Saving encryption key %08x for password %@ which hashed to %08x = %08x", *(uint32_t *)encryptionKey.bytes, password, *(uint32_t *)passwordHash.bytes, *(uint32_t *)encryptedKey.bytes);

  [prefs setObject:encryptedKey forKey:kMasterPasswordEncryptionKey];
  [prefs setObject:salt forKey:kMasterPasswordEncryptionKeySalt];
  
  return YES;
}

static BOOL verifyPassword(NSString *password)
{
  NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
  
  NSData *salt = [prefs objectForKey:kMasterPasswordSaltKey];
  if(![salt isKindOfClass:[NSData class]])
    return NO;
  NSData *currentHash = [prefs objectForKey:kMasterPasswordHashKey];
  if(![currentHash isKindOfClass:[NSData class]])
    return NO;
  
  NSData *hash = calcHash(password, salt);
  return [hash isEqualToData:currentHash];
}

// To be used in blocks that ask for a new password. firstPassword must be a __block variable initialized to nil
static void registerNewPassword(NSString **pFirstPassword, NSString *password, MasterPasswordViewController *vc, void (^callback)(BOOL ok))
{
  if(!*pFirstPassword)
  {
    *pFirstPassword = [password copy];
    [vc promptAgain:@"Confirm" accessibilityLabel:@"Confirm password"];
    return;
  }
  if(![password isEqualToString:*pFirstPassword])
  {
    *pFirstPassword = nil;
    [vc promptAgain:@"Passwords don't match, try again" accessibilityLabel:@"New password"];
    return;
  }
  
  // Save 'password'
  // DebugLog(@"Will save password %@", password);
  [vc dismiss];
  
  NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
  NSData *salt = newSalt();
  NSData *hash = calcHash(password, salt);
  [prefs setValue:hash forKey:kMasterPasswordHashKey];
  [prefs setValue:salt forKey:kMasterPasswordSaltKey];
  [prefs setBool:YES forKey:kMasterPasswordEnabledKey];    
  [prefs synchronize];
  
  if(callback)
  {
    callback(YES);
  }
}

+ (void)enableInViewController:(UIViewController *)viewController callback:(void (^)(BOOL ok))callback
{
  if(!viewController)
    viewController = rootViewController();

  __block NSString *firstPassword = nil;
  void (^callbackCopy)(BOOL ok) = callback ? [callback copy] : nil;
  
  MasterPasswordViewController * mvc = [[MasterPasswordViewController alloc]initWithTitle:@"Master Password" instructions:@"Enter new master password" accessibilityLabel:@"New password" allowCancel:YES callback:^(MasterPasswordViewController *vc, NSString *password) {
    if(!password.length)
    {
      [vc dismiss];
      if(callbackCopy)
      {
        callbackCopy(NO);
      }
      return;
    }
    registerNewPassword(&firstPassword, password, vc, ^(BOOL ok)
    {
      if(!ok)
        callbackCopy(NO);
      
      NSAssert(!encryptionKey, @"There should be no key now");
      encryptionKey = [NSMutableData dataWithLength:kCCKeySizeAES128];
      if(!encryptionKey)
      {
        callbackCopy(NO);
        return;
      }
      int ret = SecRandomCopyBytes(kSecRandomDefault, kCCKeySizeAES128, encryptionKey.mutableBytes);
      if(ret)
      {
        encryptionKey = nil;
        callbackCopy(NO);
        return;
      }
      ok = saveEncryptionKey(password);
      if(!ok)
      {
        encryptionKey = nil;
      }
      A77Log(@"Master Password enabled");      
      callbackCopy(ok);
    });
  } ];
  
  [mvc presentInViewController:viewController];
}

+ (void)disableInViewController:(UIViewController *)viewController callback:(void (^)(BOOL ok))callback
{
  if(!viewController)
    viewController = rootViewController();
  
  void (^callbackCopy)(BOOL ok) = callback ? [callback copy] : nil;
  
  MasterPasswordViewController *mvc = [[MasterPasswordViewController alloc]initWithTitle:@"Master Password" instructions:@"Enter master password" accessibilityLabel:@"Password" allowCancel:YES callback:^(MasterPasswordViewController *vc, NSString *password) {
    if(!password.length)
    {
      [vc dismiss];
      if(callbackCopy)
      {
        callbackCopy(NO);
      }      
      return;
    }
    if(!verifyPassword(password))
    {
      [vc badPassword];
      return;
    }
    // Ok, password correct, disable
    A77Log(@"Master Password disabled");

    NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
    [prefs setBool:NO forKey:kMasterPasswordEnabledKey];
    [prefs removeObjectForKey:kMasterPasswordHashKey];
    [prefs removeObjectForKey:kMasterPasswordSaltKey];
    [prefs removeObjectForKey:kMasterPasswordEncryptionKey];
    if(encryptionKey)
    {
      memset(encryptionKey.mutableBytes, 0, kCCKeySizeAES128);
      encryptionKey = nil;
    }
    
    [vc dismiss];
    
    if(callbackCopy)
    {
      callbackCopy(YES);
    }       
  }];

  [mvc presentInViewController:viewController];  
}

static BOOL isVerifying = NO;

+ (void)verifyInViewController:(UIViewController *)viewController allowCancel:(BOOL)allowCancel callback:(void (^)(BOOL ok))callback
{
  if(isVerifying)
    return;
  
  if(![self isEnabled])
    return;

  A77Log(@"Verifying Master Password");
  
  if(!viewController)
    viewController = rootViewController();
  
  isVerifying = YES;
  
  void (^callbackCopy)(BOOL ok) = callback ? [callback copy] : nil;
  
  MasterPasswordViewController *mvc = [[MasterPasswordViewController alloc]initWithTitle:@"Master Password" instructions:@"Enter master password" accessibilityLabel:@"Master Password" allowCancel:allowCancel callback:^(MasterPasswordViewController *vc, NSString *password) {
    if(!password.length)
    {
      A77Log(@"Master Password verification canceled");
      NSAssert(allowCancel, @"Cancel not allowed");
      [vc dismiss];
      isVerifying = NO;
      if(callbackCopy)
      {
        callbackCopy(NO);
      }
      return;
    }
    if(!verifyPassword(password))
    {
      A77Log(@"Master Password verification failed (bad password)");
      [vc badPassword];
      return;
    }
    A77Log(@"Master Password verification succeeded");
    [vc dismiss];
    isVerifying = NO;
    loadEncryptionKey(password);
    if(callbackCopy)
    {
      callbackCopy(YES);
    }
  }];

  [mvc presentInViewController:viewController];  
}

+ (void)changePasswordInViewController:(UIViewController *)viewController
{
  if(!viewController)
    viewController = rootViewController();
  
  NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
  if(![prefs boolForKey:kMasterPasswordEnabledKey])
    return;

  __block BOOL verified = NO;
  __block NSString *firstPassword = nil;
  
  MasterPasswordViewController *mvc = [[MasterPasswordViewController alloc]initWithTitle:@"Master Password" instructions:@"Enter current password" accessibilityLabel:@"Current Password" allowCancel:YES callback:^(MasterPasswordViewController *vc, NSString *password) {
    if(!password.length)
    {
      [vc dismiss];
      return;
    }
    if(!verified)
    {
      if(!verifyPassword(password))
      {
        [vc badPassword];
        return;
      }
      verified = YES;
      [vc promptAgain:@"Enter new password" accessibilityLabel:@"New password"];
      return;
    }
    registerNewPassword(&firstPassword, password, vc, ^(BOOL ok) { 
      if(ok)
        saveEncryptionKey(password);
    });
  }];
  
  [mvc presentInViewController:viewController];
}

+ (NSData *)encryptString:(NSString *)string
{
  if(!encryptionKey)
    return nil;
  
  u_int8_t iv[kCCBlockSizeAES128];
  int ret = SecRandomCopyBytes(kSecRandomDefault, kCCBlockSizeAES128, iv);
  if(ret)
    return nil;  
  
  CCCryptorRef cctx = NULL;
  CCCryptorStatus cret = CCCryptorCreate(kCCEncrypt, kCCAlgorithmAES128, kCCOptionPKCS7Padding, encryptionKey.bytes, kCCKeySizeAES128, iv, &cctx);
  if(cret != kCCSuccess)
    return nil;
  
  NSData *dataToEncrypt = [string dataUsingEncoding:NSUTF8StringEncoding];
  size_t encryptedLen = CCCryptorGetOutputLength(cctx, dataToEncrypt.length, YES);
  NSMutableData *encryptedData = [NSMutableData dataWithLength:encryptedLen + kCCBlockSizeAES128];
  
  u_int8_t *p = (u_int8_t *)encryptedData.mutableBytes;
  memcpy(p, iv, kCCBlockSizeAES128);
  p += kCCBlockSizeAES128;
  size_t pLen = encryptedLen;
  
  size_t written = 0;
  cret = CCCryptorUpdate(cctx, dataToEncrypt.bytes, dataToEncrypt.length, p, pLen, &written);
  if(cret != kCCSuccess)
  {
    CCCryptorRelease(cctx);  
    return nil;
  }
  
  p += written;
  pLen -= written;
  
  written = 0;
  cret = CCCryptorFinal(cctx, p, pLen, &written);
  CCCryptorRelease(cctx);  
  if(cret != kCCSuccess)
    return nil;
  
  NSAssert(pLen == written, @"Size mismatch");

  return encryptedData;
}

+ (NSString *)decryptString:(NSData *)encrypted
{
  if(!encrypted || encrypted.length < 2 * kCCBlockSizeAES128)
    return nil;
  
  CCCryptorRef cctx = NULL;
  CCCryptorStatus cret = CCCryptorCreate(kCCDecrypt, kCCAlgorithmAES128, kCCOptionPKCS7Padding, encryptionKey.bytes, kCCKeySizeAES128, encrypted.bytes, &cctx);
  if(cret != kCCSuccess)
    return nil;
  
  size_t encodedStrLen = CCCryptorGetOutputLength(cctx, encrypted.length - kCCBlockSizeAES128, YES);
  NSMutableData *encodedStr = [NSMutableData dataWithLength:encodedStrLen];
  u_int8_t *out = (u_int8_t *)encodedStr.mutableBytes;
  size_t outLen = encodedStr.length;
  
  u_int8_t *in = ((u_int8_t *)encrypted.bytes) + kCCBlockSizeAES128;
  size_t inLen = encrypted.length - kCCBlockSizeAES128;
  
  size_t written = 0;
  cret = CCCryptorUpdate(cctx, in, inLen, out, outLen, &written);
  if(cret != kCCSuccess)
  {
    CCCryptorRelease(cctx);  
    return nil;
  }
  
  out += written;
  outLen -= written;
  size_t totalLen = written;

  written = 0;
  cret = CCCryptorFinal(cctx, out, outLen, &written);
  CCCryptorRelease(cctx);  
  if(cret != kCCSuccess)
    return nil;
  totalLen += written;
  [encodedStr setLength:totalLen];
  
  return [[NSString alloc]initWithData:encodedStr encoding:NSUTF8StringEncoding];
}


@end
