//
//  MobiusAdVc.m
//  pwSafe
//
//  Created by Jorge P Vasquez on 2/19/13.
//
//

#import "MobiusAdVc.h"
#import "GradientView.h"
#import "MobiusSDK.h"
#import "pwSafeTheme.h"

NSString * const kMobiusAddFirstShownDateKey = @"MobiusAdFirstShownDate";
NSString * const kMobiusAlreadyViewedKey = @"MobiusAddAlreadyViewed";
NSString * const kMobiusNSkipped = @"MobiusAddNSkipped";
NSString * const kMobiusSafesRegOffer = @"MobiusSafesOfferedRegister";

@interface MobiusAdVc()
@property (weak, nonatomic) IBOutlet UIButton *dismissBtn;
@property (weak, nonatomic) IBOutlet UIButton *getNowBtn;

@end

@implementation MobiusAdVc

+ (BOOL)mobiusAdAlreadyViewed
{
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
  BOOL already = [defaults boolForKey:kMobiusAlreadyViewedKey];
  
  if([NSUbiquitousKeyValueStore class])
  {
    NSUbiquitousKeyValueStore *icDefaults = [NSUbiquitousKeyValueStore defaultStore];
    BOOL icAlready = [icDefaults boolForKey:kMobiusAlreadyViewedKey];
    if(!already)
    {
      already = icAlready;
      if(icAlready)
        [defaults setBool:YES forKey:kMobiusAlreadyViewedKey];
    }
    else
    {
      if(!icAlready)
        [icDefaults setBool:YES forKey:kMobiusAlreadyViewedKey];
    }
  }

  return already;
}

+ (NSDate *)firstDateMobiusAdViewed
{
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
  NSDate *firstDate = [defaults objectForKey:kMobiusAddFirstShownDateKey];

  if(![firstDate isKindOfClass:[NSDate class]])
  {
    firstDate = nil;
    [defaults removeObjectForKey:kMobiusAddFirstShownDateKey];
  }

  if([NSUbiquitousKeyValueStore class])
  {
    NSUbiquitousKeyValueStore *icDefaults = [NSUbiquitousKeyValueStore defaultStore];
    NSDate *iCloudFirstDate = [icDefaults objectForKey:kMobiusAddFirstShownDateKey];
  
    if(!firstDate)
    {
      firstDate = iCloudFirstDate;
      if(iCloudFirstDate)
        [defaults setObject:firstDate forKey:kMobiusAddFirstShownDateKey];
    }
    else
    {
      if(!iCloudFirstDate)
      {
        [icDefaults setObject:firstDate forKey:kMobiusAddFirstShownDateKey];
      }
      else
      {
        NSComparisonResult comp = [firstDate compare:iCloudFirstDate];
        if(comp == NSOrderedAscending)
        {
          [icDefaults setObject:firstDate forKey:kMobiusAddFirstShownDateKey];
        }
        else if(comp == NSOrderedDescending)
        {
          firstDate = iCloudFirstDate;
          [defaults setObject:iCloudFirstDate forKey:kMobiusAddFirstShownDateKey];
        }
      }
    }
  }
  
  return  firstDate;
}

+ (void)setMobiusAdViewed
{
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
  NSUbiquitousKeyValueStore *icDefaults;
  
  if([NSUbiquitousKeyValueStore class])
    icDefaults = [NSUbiquitousKeyValueStore defaultStore];

  if(![self mobiusAdAlreadyViewed])
  {
    [defaults setBool:YES forKey:kMobiusAlreadyViewedKey];
    if(icDefaults)
      [icDefaults setBool:YES forKey:kMobiusAlreadyViewedKey];
  }
  
  NSDate *firstDate = [self firstDateMobiusAdViewed];
  if(!firstDate)
  {
    NSDate *now = [NSDate date];
    [defaults setObject:now forKey:kMobiusAddFirstShownDateKey];
    if(icDefaults)
      [icDefaults setObject:now forKey:kMobiusAddFirstShownDateKey];
  }
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  GradientView *view = (GradientView *)self.view;
  view.colors = @[(id)[UIColor lightGrayColor].CGColor,
                  (id)[UIColor darkGrayColor].CGColor];

  if(!iOS6())
  {
    for(UIButton *btn in @[self.dismissBtn, self.getNowBtn])
    {
      [btn setTitleShadowColor:[UIColor darkTextColor] forState:UIControlStateNormal];
      [btn setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
      btn.titleLabel.shadowOffset = CGSizeMake(0, 1 );
    }
  }
}

- (void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];

  [MobiusAdVc setMobiusAdViewed];
}

- (IBAction)dismiss:(id)sender
{
  [self dismissModalViewControllerAnimated:YES];
}

- (IBAction)getItNow:(id)sender
{
  [[MobiusSDK sharedMobius]showMobiusEngineAppStoreOverViewController:self completionBlock:^(BOOL result, NSError *error) {
    if(result)
      [self dismiss:sender];
  }];
}
@end
