/*
 *  NSStringUtils.h
 *  iPasswordSafe
 *
 *  Created by Jorge Peixoto Vasquez on 3/7/11.
 *  Copyright 2011 app77. All rights reserved.
 *
 */

#include "Core/StringX.h"

typedef std::basic_string<wchar_t,
std::char_traits<wchar_t>,
S_Alloc::SecureAlloc<wchar_t> > WString;

@interface NSString (wstring_additions)
@property (readonly) WString wstring;
+(NSString*) stringWithwstring:(const WString &)string;
+(NSString*) stringWithWstring:(const std::wstring &)ws;
@end

@implementation NSString (wstring_additions)

#if TARGET_RT_BIG_ENDIAN
const NSStringEncoding kEncoding_wchar_t = NSUTF32BigEndianStringEncoding;
#else
const NSStringEncoding kEncoding_wchar_t = NSUTF32LittleEndianStringEncoding;
#endif

+(NSString*) stringWithwstring:(const WString &)ws
{
  size_t len = ws.size();
  auto data = ws.data();
  while(len > 0 && data[len - 1] == 0)
    len--;
  if(len == 0)
    return [NSString string];
  size_t size = len * sizeof(wchar_t);
  NSString* result = [[NSString alloc] initWithBytes:data length:size encoding:kEncoding_wchar_t ];
#if !__has_feature(objc_arc)
  [result autorelease];
#endif
  NSAssert(result || !size, @"Conversion failed");
  return result;  
}

+(NSString*) stringWithWstring:(const std::wstring &)ws
{
  size_t len = ws.size();
  auto data = ws.data();
  while(len > 0 && data[len - 1] == 0)
    len--;
  if(len == 0)
    return [NSString string];
  size_t size = len * sizeof(wchar_t);
  NSString* result = [[NSString alloc] initWithBytes:data length:size encoding:kEncoding_wchar_t ];
#if !__has_feature(objc_arc)
  [result autorelease];
#endif
  NSAssert(result || !size, @"Conversion failed");
  return result;
}

-(WString) wstring
{
  NSData* asData = [self dataUsingEncoding:kEncoding_wchar_t];
  const wchar_t *wData = (const wchar_t *)[asData bytes];
  size_t len = asData.length / sizeof(wchar_t);
  
  while(len > 0 && wData[len - 1] == 0)
    len--;
  
  if(len == 0)
    return WString();
  
  return WString(wData, len);
}

@end
