//
//  NewEditSafeViewController.m
//  iPasswordSafe
//
//  Created by Jorge Peixoto Vasquez on 3/16/11.
//  Copyright 2011 app77. All rights reserved.
//

#import "NewEditSafeViewController.h"
#include "core/PWScore.h"
#import "NSStringUtils.h"
#import "SafeCtx.h"
#import "SafeCtxStorage.h"
#import "MBProgressHUD.h"
#import "AppTheme.h"
#import "SafePwdInputAccessoryView.h"

@implementation NewEditSafeViewController
{
  UITableViewCell *cellName;
  UITableViewCell *cellPassword1;
  UITableViewCell *cellPassword2;
  UITableViewCell *cellError;  
  
  UITextField *txtName;
  UITextField *txtPassword1;
  UITextField *txtPassword2;
  
  __weak id target;
  SEL targetSelector;
  
  BOOL result;
  BOOL createNewSafe;
  
  NSString *dir;
  NSString *dropboxDir;
  NSString *curName;
  NSArray *forbiddenNames;
#ifdef PWSAFE_COZI
  NSString *m_safeId;
#endif
}

@synthesize result, forbiddenNames, dropboxDir;

#ifdef PWSAFE_COZI

- (id)initWithTarget:(id)_target selector:(SEL)_selector directory:(NSString *)newSafeDir
{
  if(!(self = [super initWithStyle:UITableViewStyleGrouped]))
    return nil;

  createNewSafe = YES;
  targetSelector = _selector;
  target = _target;
  dir = newSafeDir;
  
  // Create new safe Id
  CFUUIDRef cfId = CFUUIDCreate(NULL);
  if(!cfId)
  {
    NSLog(@"Could not create a new UUID");
    return nil;
  }
  m_safeId = (__bridge_transfer NSString *)CFUUIDCreateString(NULL, cfId);
  CFRelease(cfId);
  if(!m_safeId)
  {
    NSLog(@"Could not turn new UUID into string");
    return nil;
  }
  
#ifdef PWSAFE_VAULT
  self.navigationItem.title = @"New vault";
#else
  self.navigationItem.title = @"New safe"];
#endif

  return self;
}

#else

- (id) initWithTarget:(id)_target selector:(SEL)_selector directory:(NSString *)newSafeDir locationName:(NSString *)locName
{
  if(!(self = [super initWithStyle:UITableViewStyleGrouped]))
    return nil;

  createNewSafe = YES;  
  targetSelector = _selector;
  target = _target;
  dir = [newSafeDir copy];

#ifdef PWSAFE_VAULT
  self.navigationItem.title = [NSString stringWithFormat:@"New %@ vault", locName];
#else
  self.navigationItem.title = [NSString stringWithFormat:@"New %@ safe", locName];
#endif
  
  return self;
}

#endif

- (id) initWithTarget:(id)_target selector:(SEL)_selector name:(NSString *)currentName
{
  if(!(self = [super initWithStyle:UITableViewStyleGrouped]))
    return nil;

  createNewSafe = NO;  
  targetSelector = _selector;
  target = _target;
  curName = currentName;

#ifdef PWSAFE_VAULT
  self.navigationItem.title = [NSString stringWithFormat:@"Edit vault"];
#else
  self.navigationItem.title = [NSString stringWithFormat:@"Edit safe"];
#endif
  
  return self;
}

- (void)dealloc
{
  if(self.isViewLoaded)
  {
    self.tableView.delegate = nil;
    self.tableView.dataSource = nil;
  }
}

- (BOOL)askForName
{
  return createNewSafe;
}

- (void) releaseSubViews 
{
  cellName = nil;
  cellPassword1 = nil;
  cellPassword2 = nil;
  cellError = nil;
  
  txtName = nil;
  txtPassword1 = nil;
  txtPassword2 = nil;
}

- (void)viewDidUnload
{
  [super viewDidUnload];
  [self releaseSubViews];
}

- (void)onCancel:(id)sender
{
  result = FALSE;
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
  [target performSelector:targetSelector withObject:self];
#pragma clang diagnostic pop
}

- (NSString *)safePath
{
#ifdef PWSAFE_COZI
  return [dir stringByAppendingPathComponent:[m_safeId stringByAppendingPathExtension:@"psafe3"]];
#else
  return [dir stringByAppendingPathComponent:[self.safeName stringByAppendingPathExtension:@"psafe3"]];
#endif
}

- (NSString *)safeName
{
  return txtName.text;
}

- (NSString *)safePassword
{
  return txtPassword1.text;
}

- (void)hideHud:(id)iHud
{
  MBProgressHUD *hud = (MBProgressHUD *)iHud;
  [hud hide:YES];
}

- (void)onDone:(id)sender
{
  NSString *password = txtPassword1.text;
#ifdef DEBUG_MODE
  NSString *confirm = txtPassword2.text;
  NSAssert([password isEqualToString:confirm], @"Passwords should match");
#endif
  
  if(createNewSafe)
  {
    PWScore core;
    core.NewFile(password.wstring);
    
    NSString *safePath = self.safePath;
    core.SetCurFile(safePath.wstring);
    int ret = core.WriteCurFile();
    if(ret != PWSfile::SUCCESS)
    {
      MBProgressHUD *hud = [[MBProgressHUD alloc]initWithView:self.view];
      hud.customView = [[UIImageView alloc]initWithImage:[UIImage imageNamed:@"Bad.png"]];
      hud.mode = MBProgressHUDModeCustomView;
#ifdef PWSAFE_VAULT
      hud.labelText = @"Can't create vault";
#else
      hud.labelText = @"Can't create safe";
#endif
      hud.animationType = MBProgressHUDAnimationZoom;
      hud.yOffset = -80;
      hud.opacity = .8;
      hud.removeFromSuperViewOnHide = YES;
      [self.view addSubview:hud];
      [hud show:YES];
      [self performSelector:@selector(hideHud:) withObject:hud afterDelay:1];
      return;
    }
    
    SafeCtx *safeCtx = [[SafeCtxStorage sharedStorage]safeCtxForPath:safePath];
    safeCtx.password = password;
    safeCtx.keepAlivePeriod = 300;
    safeCtx.opened = YES;
    [safeCtx sleep];
    safeCtx.opened = NO;
#ifdef PWSAFE_COZI
    safeCtx.coziDisplayName = self.safeName;
    safeCtx.coziModified = YES;
    safeCtx.coziSafeId = m_safeId;
#endif
    
    NSAssert(!safeCtx.expired, @"Should be ready");
  }
  
  result = TRUE;
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
  [target performSelector:targetSelector withObject:self];
#pragma clang diagnostic pop
}

- (BOOL)showPasswordMismatch
{
  NSString *pw1 = txtPassword1.text;
  NSString *pw2 = txtPassword2.text;
  return !self.passwordsMatch && pw1.length && pw2.length && [pw1 rangeOfString:pw2].location == NSNotFound;
}


#pragma mark - View lifecycle

- (void)viewWillAppear:(BOOL)animated
{
  [self.navigationController setToolbarHidden:YES animated:animated];
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  [self.tableView themeViewDidLoadInIpadPos:TVP_FORMSHEET];

  UIBarButtonItem *btn  = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(onCancel:)];
  self.navigationItem.leftBarButtonItem = btn;

  btn = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(onDone:)];
  self.navigationItem.rightBarButtonItem = btn;
  btn.enabled = NO;
}

// Don't implement view did unload, as we need the controls after the view has been hidden, during the callback

- (void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  [(self.askForName ? txtName : txtPassword1) becomeFirstResponder];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)errorCellRow
{
  return self.askForName ? 3 : 2;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  NSInteger n = (![self showPasswordMismatch] && self.safenameAllowed) ? 3 : 4;
  if(!self.askForName)
    n--;
  return n;
}

- (void)makeTxtCell:(__strong UITableViewCell **)pCell textField:(__strong UITextField **)pTextField forPassword:(BOOL)forPassword withLabel:(NSString *)label last:(BOOL)last valueForEdition:(NSString *)valueForEdition
{
  if(*pCell)
    return;

  *pCell = [[UITableViewCell alloc]initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:label];
  UITableViewCell *cell = *pCell;
  cell.textLabel.text = label;

  cell.selectionStyle = UITableViewCellSelectionStyleNone;
  
  NSUInteger w = UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ? 350 : 170;
  
  UITextField *textField = [[UITextField alloc]initWithFrame:CGRectMake(120, 12, w, 21)];
  textField.clearButtonMode = UITextFieldViewModeWhileEditing;
  textField.borderStyle = UITextBorderStyleNone;
  textField.adjustsFontSizeToFitWidth = YES;
  textField.secureTextEntry = forPassword;
  textField.textColor = cell.detailTextLabel.textColor;
  textField.autocapitalizationType = UITextAutocapitalizationTypeNone;
  textField.autocorrectionType = UITextAutocorrectionTypeNo;
  textField.enablesReturnKeyAutomatically = NO;
  textField.returnKeyType = last ? UIReturnKeyDone : UIReturnKeyNext;
  textField.clearsOnBeginEditing = NO;
  textField.delegate = self;
  textField.accessibilityLabel = label;
  if(!createNewSafe)
    textField.text = valueForEdition;
  
  [textField addTarget:self action:@selector(onTextChanged:) forControlEvents:UIControlEventEditingChanged];
  
  if(forPassword && UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone)
  {
    SafePwdInputAccessoryView *accessoryView = [[SafePwdInputAccessoryView alloc]initWithFrame:CGRectMake(0, 0, self.view.frame.size.width, 44.0f)];;
    accessoryView.allowedKeyboardTypes = @[@(UIKeyboardTypeDefault), @(UIKeyboardTypeNumberPad)];
    textField.inputAccessoryView = accessoryView;
    textField.keyboardType = (UIKeyboardType)[accessoryView.allowedKeyboardTypes[0] integerValue];
    accessoryView.textField = textField;
  }
  
  *pTextField = textField;
  
  [cell.contentView addSubview:textField];
}

- (NSString *)errorTxt
{
#ifdef PWSAFE_VAULT
  if(!self.safenameAllowed)
    return @"Vault name already in use";
#else
  if(!self.safenameAllowed)
    return @"Safe name already in use";
#endif
  if([self showPasswordMismatch])
    return @"Passwords don't match";
  return nil;
}

- (void)onTextChanged:(id)sender
{
  UIBarButtonItem *doneBtn = self.navigationItem.rightBarButtonItem;
  
  BOOL safenameAllowed = self.safenameAllowed;
  BOOL passwordsMatch = self.passwordsMatch;
  BOOL showError = !safenameAllowed || [self showPasswordMismatch];
  
  doneBtn.enabled = safenameAllowed && passwordsMatch && txtPassword1.text.length > 0 && (!self.askForName || txtName.text.length > 0);

  NSIndexPath *rowBefore = [NSIndexPath indexPathForRow:[self errorCellRow] - 1 inSection:0];
  UITableViewCell *cellBeforeError = [self.tableView cellForRowAtIndexPath:rowBefore];
  NSArray *idxs = @[[NSIndexPath indexPathForRow:[self errorCellRow] inSection:0]];          
  
  if(showError)
  {
    if(!cellError)
    {
      [self.tableView insertRowsAtIndexPaths:idxs withRowAnimation:UITableViewRowAnimationTop];
      if(iOS6())
        [self.tableView addBackgroundsToGroupedCell:cellBeforeError atIndexPath:rowBefore];
    }
    else
    {
      cellError.textLabel.text = [self errorTxt];
    }
  }
  else
  {
    if(cellError)
    {
      cellError = nil;
      [self.tableView deleteRowsAtIndexPaths:idxs withRowAnimation:UITableViewRowAnimationTop];
      if(iOS6())
        [self.tableView addBackgroundsToGroupedCell:cellBeforeError atIndexPath:rowBefore];
    }
  }
}

- (BOOL)safenameAllowed
{
  if(!self.askForName || !self.forbiddenNames)
    return YES;
  
  NSString *typedFilename = [[txtName.text lowercaseString]precomposedStringWithCanonicalMapping];
  for(NSString *fileName in forbiddenNames)
  {
    if([fileName isEqualToString:typedFilename])
      return NO;
  }
  
  return YES;
}

- (BOOL)passwordsMatch
{
  NSString *pass1 = txtPassword1.text;
  NSString *pass2 = txtPassword2.text;
  
  BOOL match = (!pass1 && !pass2) || [pass1 isEqualToString:pass2];
  
  return match;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  NSInteger row = indexPath.row;
  NSInteger section = indexPath.section;
  
  if(section == 0)
  {
    if(row == [self errorCellRow])
    {
      NSAssert(!self.passwordsMatch || !self.safenameAllowed, @"Must have an error");
      if(cellError)
        return cellError;

      cellError = [[UITableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"Bad password"];
      cellError.textLabel.textColor = [UIColor whiteColor];
      cellError.backgroundColor = [UIColor redColor];
      cellError.selectionStyle = UITableViewCellSelectionStyleNone;
      cellError.textLabel.text = [self errorTxt];
      [tableView addBackgroundsToGroupedCell:cellError atIndexPath:indexPath];
      cellError.backgroundView = nil; // Remove custom, keep the rest
      return cellError;
    }

    if(!self.askForName)
      row++;

    UITableViewCell *cell = nil;
    switch(row)
    {
    case 0:
      [self makeTxtCell:&cellName textField:&txtName forPassword:NO withLabel:@"Name" last:NO valueForEdition:curName];
      cell = cellName;
      break;
    case 1:
      [self makeTxtCell:&cellPassword1 textField:&txtPassword1 forPassword:YES withLabel:@"Password" last:NO valueForEdition:nil];
      cell = cellPassword1;
      break;
    case 2:
      [self makeTxtCell:&cellPassword2 textField:&txtPassword2 forPassword:YES withLabel:@"Confirm" last:YES valueForEdition:nil];
      cell = cellPassword2;
      break;        
    }
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];
    
    return cell;
  }

  return nil;
}

- (NSString *)tableView:(UITableView *)tableView titleForFooterInSection:(NSInteger)section
{
  if(!createNewSafe)
    return @"";
#ifdef PWSAFE_VAULT
  return @"This is the password that you will need to open your vault. Please make note of it – you will use it each time you come back to this app.\nShare this password only with your family members you wish to give access to this new vault.";
#else
  return @"This is the password that you will need to open your safe. Please make note of it – you will use it each time you come back to this app.";
#endif
}

#pragma mark - TextFieldDelegate

-(BOOL)textFieldShouldReturn:(UITextField *)textField
{
  if(textField.returnKeyType == UIReturnKeyDone)
  {
    if(self.navigationItem.rightBarButtonItem.enabled)
      [self onDone:textField];
  }
  else
  {
    UITextField *nextTextField;
    if(textField == txtName)
      nextTextField = txtPassword1;
    else
      nextTextField = txtPassword2;
    [nextTextField becomeFirstResponder];
  }
  return NO;
}

- (void)textFieldDidBeginEditing:(UITextField *)textField
{
  if(!iOS6())
    textField.textColor = kThemeTintColor;
}

- (void)textFieldDidEndEditing:(UITextField *)textField
{
  if(!iOS6())
    textField.textColor = [UIColor blackColor];
}

@end
