//
//  NewPDFPasswordVC.m
//  pwSafe
//
//  Created by Jorge P. Vasquez on 28/09/12.
//
//

#import "NewPDFPasswordVC.h"
#import "AppTheme.h"

@interface NewPDFPasswordVC ()
{
  __weak UITableViewCell *cellPassword1;
  __weak UITableViewCell *cellPassword2;
  __weak UITableViewCell *cellError;
  
  __weak UITextField *txtPassword1;
  __weak UITextField *txtPassword2;
  
  NewPDFPasswordHandler handler;
}

@property (nonatomic, readonly) BOOL passwordsMatch;

@end

@implementation NewPDFPasswordVC

- (id) initWithTitle:(NSString *)title handler:(NewPDFPasswordHandler)newHandler
{
  if(!(self = [super initWithStyle:UITableViewStyleGrouped]))
    return nil;
  
  handler = [newHandler copy];
  
  self.navigationItem.title = title;
  return self;
}

- (void)dealloc
{
  if(self.isViewLoaded)
  {
    self.tableView.delegate = nil;
    self.tableView.dataSource = nil;
  }
}

- (void)onCancel:(id)sender
{
  _result = FALSE;
  handler(self);
}

- (NSString *)password
{
  return txtPassword1.text;
}

- (void)onDone:(id)sender
{
#ifdef DEBUG_MODE
  NSString *password = txtPassword1.text;
  NSString *confirm = txtPassword2.text;
  NSAssert([password isEqualToString:confirm], @"Passwords should match");
#endif
  
  _result = TRUE;
  handler(self);
}

- (BOOL)showPasswordMismatch
{
  NSString *pw1 = txtPassword1.text;
  NSString *pw2 = txtPassword2.text;
  return !self.passwordsMatch && pw1.length && pw2.length && [pw1 rangeOfString:pw2].location == NSNotFound;
}


#pragma mark - View lifecycle

- (void)viewWillAppear:(BOOL)animated
{
  [self.navigationController setToolbarHidden:YES animated:animated];
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  [self.tableView themeViewDidLoadInIpadPos:TVP_FORMSHEET];
  
  UIBarButtonItem *btn  = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(onCancel:)];
  self.navigationItem.leftBarButtonItem = btn;
  
  btn = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(onDone:)];
  self.navigationItem.rightBarButtonItem = btn;
  btn.enabled = NO;
}

// Don't implement view did unload, as we need the controls after the view has been hidden, during the callback

- (void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  [txtPassword1 becomeFirstResponder];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)errorCellRow
{
  return 2;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  NSInteger n = ![self showPasswordMismatch] ? 2 : 3;
  return n;
}

- (void)makeTxtCell:(__strong UITableViewCell **)pCell textField:(__strong UITextField **)pTextField forPassword:(BOOL)forPassword withLabel:(NSString *)label last:(BOOL)last
{
  if(*pCell)
    return;
  
  UITableViewCell *cell = [[UITableViewCell alloc]initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:label];
  *pCell = cell;
  cell.textLabel.text = label;
  
  cell.selectionStyle = UITableViewCellSelectionStyleNone;
  
  NSUInteger w = UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ? 350 : 170;
  
  UITextField *textField = [[UITextField alloc]initWithFrame:CGRectMake(120, 12, w, 21)];
  textField.clearButtonMode = UITextFieldViewModeWhileEditing;
  textField.borderStyle = UITextBorderStyleNone;
  textField.adjustsFontSizeToFitWidth = YES;
  textField.secureTextEntry = forPassword;
  textField.textColor = cell.detailTextLabel.textColor;
  textField.autocapitalizationType = UITextAutocapitalizationTypeNone;
  textField.autocorrectionType = UITextAutocorrectionTypeNo;
  textField.enablesReturnKeyAutomatically = NO;
  textField.returnKeyType = last ? UIReturnKeyDone : UIReturnKeyNext;
  textField.clearsOnBeginEditing = NO;
  textField.delegate = self;
  textField.accessibilityLabel = label;
  
  [textField addTarget:self action:@selector(onTextChanged:) forControlEvents:UIControlEventEditingChanged];
  
  *pTextField = textField;
  
  [cell.contentView addSubview:textField];
}

- (NSString *)errorTxt
{
  if([self showPasswordMismatch])
    return @"Passwords don't match";
  return nil;
}

- (void)onTextChanged:(id)sender
{
  UIBarButtonItem *doneBtn = self.navigationItem.rightBarButtonItem;
  
  BOOL passwordsMatch = self.passwordsMatch;
  BOOL showError = [self showPasswordMismatch];
  
  doneBtn.enabled = passwordsMatch && txtPassword1.text.length > 0;
  
  NSArray *idxs = [NSArray arrayWithObject:[NSIndexPath indexPathForRow:[self errorCellRow] inSection:0]];
  
  if(showError)
  {
    if(!cellError)
    {
      [self.tableView insertRowsAtIndexPaths:idxs withRowAnimation:UITableViewRowAnimationTop];
    }
    else
    {
      cellError.textLabel.text = [self errorTxt];
    }
  }
  else
  {
    if(cellError)
    {
      cellError = nil;
      [self.tableView deleteRowsAtIndexPaths:idxs withRowAnimation:UITableViewRowAnimationTop];
    }
  }
}

- (BOOL)passwordsMatch
{
  NSString *pass1 = txtPassword1.text;
  NSString *pass2 = txtPassword2.text;
  
  BOOL match = (!pass1 && !pass2) || [pass1 isEqualToString:pass2];
  
  return match;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  NSInteger row = indexPath.row;
  NSInteger section = indexPath.section;
  
  
  if(section == 0)
  {
    if(row == [self errorCellRow])
    {
      NSAssert(!self.passwordsMatch, @"Must have an error");
      if(cellError)
        return cellError;
      
      
      UITableViewCell *cell = [[UITableViewCell alloc]initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"Bad password"];
      cellError = cell;
      cellError.textLabel.textColor = [UIColor whiteColor];
      cellError.backgroundColor = [UIColor redColor];
      cellError.selectionStyle = UITableViewCellSelectionStyleNone;
      cellError.textLabel.text = [self errorTxt];
      cellError.backgroundView = nil; // Remove custom, keep the rest
      return cellError;
    }
    
    UITableViewCell *cell = nil;
    UITextField *txtField = nil;
    switch(row)
    {
      case 0:
        [self makeTxtCell:&cell textField:&txtField forPassword:YES withLabel:@"Password" last:NO];
        cellPassword1 = cell;
        txtPassword1 = txtField;
        break;
      case 1:
        [self makeTxtCell:&cell textField:&txtField forPassword:YES withLabel:@"Confirm" last:YES];
        cellPassword2 = cell;
        txtPassword2 = txtField;
        break;
    }
    
    NSAssert(cell, @"Must return a cell");
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];
    return cell;
  }
  
  return nil;
}

- (NSString *)tableView:(UITableView *)tableView titleForFooterInSection:(NSInteger)section
{
  return @"This is the password which protects the PDF you are creating, encrypting it.";
}

#pragma mark - TextFieldDelegate

-(BOOL)textFieldShouldReturn:(UITextField *)textField
{
  if(textField.returnKeyType == UIReturnKeyDone)
  {
    if(self.navigationItem.rightBarButtonItem.enabled)
      [self onDone:textField];
  }
  else
  {
    [txtPassword2 becomeFirstResponder];
  }
  return NO;
}

@end
