//
//  NewPwPolicyVC.m
//  pwSafe
//
//  Created by Jorge P. Vasquez on 03/10/12.
//
//

#import "NewPwPolicyVC.h"
#import "AppTheme.h"

@interface NewPwPolicyVC () <UITextFieldDelegate>
{
  void (^_handler)(NewPwPolicyVC *);
}
@end

@implementation NewPwPolicyVC

#define NEW_PW_POLICY_NAME_TAG 23141

- (id)initWithHandler:(void (^)(NewPwPolicyVC *))handler
{
  self = [super initWithStyle:UITableViewStyleGrouped];
  if (!self)
    return nil;
  
  _handler = [handler copy];
  self.navigationItem.title = @"New Policy";
  self.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(onCancel:)];
  self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(onDone:)];
  // Disable Done button
  self.navigationItem.rightBarButtonItem.enabled = NO;

  return self;
}

- (void)dealloc
{
  if(self.isViewLoaded)
  {
    self.tableView.delegate = nil;
    self.tableView.dataSource = nil;
  }
}

- (void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
  UITextField *nameField = (UITextField *)[cell.contentView viewWithTag:NEW_PW_POLICY_NAME_TAG];
  [nameField becomeFirstResponder];
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  [self.tableView themeViewDidLoadInIpadPos:TVP_FORMSHEET];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
}


#pragma mark - Table view data source

#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  // Return the number of sections.
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  // Return the number of rows in the section.
  return 1;
}



// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  static NSString *CellIdentifier = @"Password";
  
  NSAssert2(indexPath.section == 0, @"Internal error @ %s:%d", __FILE__, __LINE__);
  
  UITableViewCell *cell;
  NSAssert(indexPath.row == 0, @"Only one row");
  {
    cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    UITextField *nameField;
    if (cell == nil)
    {
      cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:CellIdentifier];
      
      NSUInteger w = UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ? 370 : 190;
      nameField = [[UITextField alloc]initWithFrame:CGRectMake(100, 12, w, 21)];
      nameField.borderStyle = UITextBorderStyleNone;
      nameField.adjustsFontSizeToFitWidth = YES;
      nameField.textColor = cell.detailTextLabel.textColor;
      nameField.autocapitalizationType = UITextAutocapitalizationTypeNone;
      nameField.autocorrectionType = UITextAutocorrectionTypeNo;
      nameField.enablesReturnKeyAutomatically = YES;
      nameField.returnKeyType = UIReturnKeyDone;
      nameField.clearsOnBeginEditing = NO;
      nameField.clearButtonMode = UITextFieldViewModeWhileEditing;
      nameField.delegate = self;
      nameField.tag = NEW_PW_POLICY_NAME_TAG;
      [cell.contentView addSubview:nameField];
      [nameField addTarget:self action:@selector(onNameChanged:) forControlEvents:UIControlEventEditingChanged];
      nameField.backgroundColor = nil;
      nameField.opaque = NO;
      nameField.accessibilityLabel = @"Name";
    }
    
    cell.textLabel.text = @"Name";
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
  }

  [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];
  
  return cell;
}

- (void)onDone:(id)sender
{
  NSAssert(_name && _name.length, @"Policy name cannot be empty");
  _ok = YES;
  _handler(self);
}

- (void)onCancel:(id)sender
{
  _ok = NO;
  _handler(self);
}

- (BOOL) textFieldShouldReturn:(UITextField *)textField
{
  NSString *name = textField.text;
  if(!name || ![name length])
    return NO;
  NSAssert([name isEqualToString:_name], @"Should be the same");
  [self onDone:textField];
  return NO;
}

- (void) onNameChanged:(UITextField *)textField
{
  NSString *name = textField.text;
  self.navigationItem.rightBarButtonItem.enabled = name && [name length];
  _name = name;
}
@end
