//
//  PasteboardPasswordCleaner.m
//  pwSafe
//
//  Created by Jorge P. Vasquez on 05/10/12.
//
//

#import "PasteboardPasswordCleaner.h"

@implementation PasteboardPasswordCleaner

static NSString * const kAutoClearPasswordFromPasteboardKey = @"clearPasswordFromPasteboard";
static NSString * const kAutoClearPasswordTimeoutMinutesKey = @"clearPasswordTimeoutMinutes";
static NSString * const kAutoClearPasswordNotifyKey = @"clearPasswordNotify";

static BOOL autoClearPasswordFromPasteboard()
{
  NSNumber *obj = [[NSUserDefaults standardUserDefaults]objectForKey:kAutoClearPasswordFromPasteboardKey];
  if(!obj)
    return YES;
  
  return obj.boolValue;
}

static BOOL notifyOnPasswordClearedFromPasteboard()
{
  NSNumber *obj = [[NSUserDefaults standardUserDefaults]objectForKey:kAutoClearPasswordNotifyKey];
  if(!obj)
    return YES;
  
  return obj.boolValue;
}

+ (void)notify
{
  if(!notifyOnPasswordClearedFromPasteboard())
    return;
  
  UIApplication *app = [UIApplication sharedApplication];
  UILocalNotification *notif = [[UILocalNotification alloc]init];
  notif.alertBody = @"Pasteboard cleared to protect your password.";
  notif.hasAction = YES;
  notif.soundName = UILocalNotificationDefaultSoundName;
  
  [app presentLocalNotificationNow:notif];
}

static UIBackgroundTaskIdentifier *kLastBgIdentifierPtr = NULL;

+ (void)passwordCopied:(NSString *)password
{
  if(!autoClearPasswordFromPasteboard())
    return;
  
  UIApplication *app = [UIApplication sharedApplication];
  NSDate *whenStarted = [NSDate date];
  
  if(kLastBgIdentifierPtr)
  {
    [app endBackgroundTask:*kLastBgIdentifierPtr];
    *kLastBgIdentifierPtr = UIBackgroundTaskInvalid;
  }
  
  __block UIBackgroundTaskIdentifier bgIdentifier = [app beginBackgroundTaskWithExpirationHandler:^(void) {
    NSString *currentPasteboardStr = [UIPasteboard generalPasteboard].string;
    NSTimeInterval elapsed = [[NSDate date]timeIntervalSinceDate:whenStarted];
    if(elapsed >= 60 && currentPasteboardStr && [currentPasteboardStr isEqualToString:password])
    {
      [UIPasteboard generalPasteboard].string = @"";
      [self notify];
    }

    NSAssert(bgIdentifier != UIBackgroundTaskInvalid, @"Must not run this block if the timer elapses");
    
    if(&bgIdentifier == kLastBgIdentifierPtr)
      kLastBgIdentifierPtr = NULL;
    
    UIBackgroundTaskIdentifier identifier = bgIdentifier;
    bgIdentifier = UIBackgroundTaskInvalid;
    
    // This must be the last one on the block
    [app endBackgroundTask:identifier];
  }];

  kLastBgIdentifierPtr = &bgIdentifier;
  
  NSNumber *delayInMinutes = [[NSUserDefaults standardUserDefaults]objectForKey:kAutoClearPasswordTimeoutMinutesKey];
  double delayInSeconds = delayInMinutes ? delayInMinutes.integerValue * 60 : 120;

  dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
  dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
    if(&bgIdentifier == kLastBgIdentifierPtr)
      kLastBgIdentifierPtr = NULL;

    if(bgIdentifier == UIBackgroundTaskInvalid)
      return;
    
    NSString *currentPasteboardStr = [UIPasteboard generalPasteboard].string;
    if(currentPasteboardStr && [currentPasteboardStr isEqualToString:password])
    {
      [UIPasteboard generalPasteboard].string = @"";
      [self notify];
    }
    
    UIBackgroundTaskIdentifier identifier = bgIdentifier;
    bgIdentifier = UIBackgroundTaskInvalid;
    
    // This must be the last one on the block
    [app endBackgroundTask:identifier];
  });
}

+ (void)passwordRemovedFromPasteboard
{
  if(!autoClearPasswordFromPasteboard())
    return;
  
  UIApplication *app = [UIApplication sharedApplication];
  
  if(kLastBgIdentifierPtr)
  {
    [app endBackgroundTask:*kLastBgIdentifierPtr];
    *kLastBgIdentifierPtr = UIBackgroundTaskInvalid;
    kLastBgIdentifierPtr = NULL;
  }
}

@end
