//
//  RestoreBackupsVC.m
//  pwSafe
//
//  Created by Jorge Peixoto Vasquez on 04/06/12.
//  Copyright (c) 2012 app77. All rights reserved.
//

#import "RestoreBackupsVC.h"
#import "SyncEnabled.h"
#import "FileSystemFolders.h"
#import "MBProgressHUD.h"
#import "SyncEnabled.h"
#import "SafeCtx.h"
#import "SafeCtxStorage.h"

#ifdef PWSAFE_COZI
#  import "CoziSafeStatus.h"
#  import "CoziFamilySafes.h"
#  import "CoziSafeStatus.h"
#endif
#import "AppTheme.h"

@interface RestoreBackupsVC ()
{
  NSMutableArray *m_sectionNames;
  NSMutableArray *m_sectionFiles;
  NSMutableSet *m_indexPathsToRecover;
  NSDateFormatter *m_dateFormatter;
}
@end

@implementation RestoreBackupsVC

static NSArray *backupFilesInDir(NSString *dir)
{
  DebugLog(@"Looking for backups in %@", dir);
  NSFileManager *fileMan = [NSFileManager defaultManager];
  NSError *error = nil;
  NSArray *dirContents = [fileMan contentsOfDirectoryAtPath:dir error:&error];
  if(!dirContents)
  {
    A77Log(@"Could not read directory '%@': %@", dir, [error localizedDescription]);
    return [NSMutableArray arrayWithCapacity:2];
  }
  NSMutableArray *newFilesInDocsFolder = [[NSMutableArray alloc]initWithCapacity:dirContents.count];
  for(NSString *entry in dirContents)
  {
    DebugLog(@"Evaluating file %@", entry);
    NSString *stdEntry = [entry precomposedStringWithCanonicalMapping];
    NSString *path = [dir stringByAppendingPathComponent:stdEntry];
    BOOL isDir;
    BOOL ok = [fileMan fileExistsAtPath:path isDirectory:&isDir];
    NSString *ext = [stdEntry.pathExtension lowercaseString];
    NSString *internalExt = stdEntry.stringByDeletingPathExtension.pathExtension.lowercaseString;
    if(ok && !isDir && [ext isEqualToString:@"old"] && ([internalExt isEqualToString:@"psafe3"] || [internalExt isEqualToString:@"dat"]))
    {
      // Backup file
      [newFilesInDocsFolder addObject:[dir stringByAppendingPathComponent:stdEntry]];
    }
  }
  
  return [NSMutableArray arrayWithArray:[newFilesInDocsFolder sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)]];
}

- (id)init
{
  self = [super initWithStyle:UITableViewStyleGrouped];
  if (!self)
    return nil;
  
  m_dateFormatter = [[NSDateFormatter alloc]init];
  m_dateFormatter.timeStyle = NSDateFormatterShortStyle;
  m_dateFormatter.dateStyle = NSDateFormatterMediumStyle;
  
  NSMutableArray *names = [NSMutableArray arrayWithCapacity:3];
  NSMutableArray *files = [NSMutableArray arrayWithCapacity:3];
  
  NSString *safes;
#ifdef PWSAFE_VAULT
  safes = @"vaults";
#else
  safes = @"safes";
#endif
  
  [names addObject:[@"Local " stringByAppendingString:safes]];
  [files addObject:backupFilesInDir(localSafeDir())];

#ifdef PWSAFE_COZI
  [names addObject:[@"Family " stringByAppendingString:safes]];
  [files addObject:backupFilesInDir(coziFamilySafesDir())];
#endif
  
  if(isDropboxReady())
  {
    [names addObject:[@"Dropbox " stringByAppendingString:safes]];
    [files addObject:backupFilesInDir(dropboxDir())];
  }
  
  m_sectionFiles = files;
  m_sectionNames = names;
  m_indexPathsToRecover = [NSMutableSet setWithCapacity:32];
  
  self.navigationItem.title = @"Backups";
  UIBarButtonItem *item = [[UIBarButtonItem alloc]initWithTitle:@"Restore" style:UIBarButtonItemStyleDone target:self action:@selector(onDone:)];
  item.enabled = NO;
  self.navigationItem.rightBarButtonItem = item;
                                      
  return self;
}

- (void)dealloc
{
  if(self.isViewLoaded)
  {
    self.tableView.delegate = nil;
    self.tableView.dataSource = nil;
  }
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  BOOL res = (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
  return res;
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  [self.tableView themeViewDidLoadInIpadPos:TVP_FORMSHEET];  
}

- (void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  iCloudURL(^(NSURL *url) {
    NSString *dir;
    if(!(dir = url.path).length)
      return;
#ifdef PWSAFE_VAULT
    static NSString * const safes = @"vaults";
#else
    static NSString * const safes = @"safes";
#endif
    [m_sectionNames addObject:[@"iCloud " stringByAppendingString:safes]];
    [m_sectionFiles addObject:backupFilesInDir(dir)];
    [self.tableView insertSections:[NSIndexSet indexSetWithIndex:m_sectionNames.count -1] withRowAnimation:UITableViewRowAnimationAutomatic];
  });
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return m_sectionNames.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  return [m_sectionFiles[section]count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  static NSString *CellIdentifier = @"Cell";
  UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
  if(!cell)
     cell = [[UITableViewCell alloc]initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
  NSString *path = m_sectionFiles[indexPath.section][indexPath.row];
#ifdef PWSAFE_COZI
  NSString *cellTitle;
  
  if([path rangeOfString:coziFamilySafesDir()].location == 0)
  {
    NSString *originalPath = path.stringByDeletingPathExtension;
    SafeCtx *originalCtx = [SafeCtxStorage sharedStorage][originalPath];
    cellTitle = originalCtx.coziDisplayName;
    if(!cellTitle.length)
    {
      NSString *baseFileName = originalPath.lastPathComponent.stringByDeletingPathExtension;
      if(baseFileName.length >= 4)
        cellTitle = [@"Unknown safe - " stringByAppendingString:[baseFileName substringToIndex:4]];
      else
        cellTitle = [@"Unknown safe - " stringByAppendingString:baseFileName];
    }
  }

  if(!cellTitle.length)
    cellTitle = path.lastPathComponent.stringByDeletingPathExtension.stringByDeletingPathExtension;
  
  cell.textLabel.text = cellTitle;
#else
  cell.textLabel.text = path.lastPathComponent.stringByDeletingPathExtension.stringByDeletingPathExtension;
#endif
  NSError *error = nil;
  NSDictionary *attribs = [[NSFileManager defaultManager]attributesOfItemAtPath:path error:&error];
  if(attribs)
  {
    NSDate *fileModifDate = attribs.fileModificationDate;
    cell.detailTextLabel.text = [m_dateFormatter stringFromDate:fileModifDate];
  }
  else 
  {
    cell.detailTextLabel.text = error.localizedDescription;
  }
  BOOL isMarked;
  isMarked = [m_indexPathsToRecover containsObject:indexPath];
    
  cell.accessoryType = isMarked ? UITableViewCellAccessoryCheckmark : UITableViewCellAccessoryNone;
  cell.accessibilityTraits = isMarked ? UIAccessibilityTraitSelected : UIAccessibilityTraitNone;
  [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];

  return cell;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
  return m_sectionNames[section];
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  [tableView deselectRowAtIndexPath:indexPath animated:YES];
  UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];

  if([m_indexPathsToRecover containsObject:indexPath])
  {
    [m_indexPathsToRecover removeObject:indexPath];
    cell.accessoryType = UITableViewCellAccessoryNone;
    cell.accessibilityTraits = UIAccessibilityTraitNone;
  }
  else
  {
    cell.accessoryType = UITableViewCellAccessoryCheckmark;
    cell.accessibilityTraits = UIAccessibilityTraitSelected;
    [m_indexPathsToRecover addObject:indexPath];
  }
  self.navigationItem.rightBarButtonItem.enabled = m_indexPathsToRecover.count > 0;
}

#pragma mark - Restore

- (void)onDone:(id)sender
{
  NSSet *indexPathsToRecover = [m_indexPathsToRecover copy];
  self.tableView.userInteractionEnabled = NO;
  self.navigationController.navigationBar.userInteractionEnabled = NO;  
  
  A77Log(@"Restoring safes");
  
  dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
    NSFileManager *fm = [NSFileManager defaultManager];
    BOOL allOK = YES;

    for(NSIndexPath *idxPath in indexPathsToRecover)
    {
      NSString *path = m_sectionFiles[idxPath.section][idxPath.row];
      NSString *ext = path.lastPathComponent.stringByDeletingPathExtension.pathExtension;
      BOOL __block ok;
      NSError *error = nil;
      NSURL *srcURL = [NSURL fileURLWithPath:path];
      BOOL isICloud = [fm isUbiquitousItemAtURL:srcURL];

      A77Log(@"Restoring safe %@", path);
#ifdef PWSAFE_COZI
      CFUUIDRef cfId = CFUUIDCreate(NULL);
      if(!cfId)
      {
        A77Log(@"Could not create a new UUID");
        continue;
      }
      NSString *safeId = (__bridge_transfer NSString *)CFUUIDCreateString(NULL, cfId);
      CFRelease(cfId);
      if(!safeId)
      {
        A77Log(@"Could not turn new UUID into string");
        continue;
      }
      
      NSString *recoveredFilePath = [[coziFamilySafesDir() stringByAppendingPathComponent:safeId]stringByAppendingPathExtension:ext];
#else
      NSString *fileName = path.lastPathComponent.stringByDeletingPathExtension.stringByDeletingPathExtension;

      NSString *recoveredFilePathBase = [localSafeDir() stringByAppendingPathComponent:[@"Restored-" stringByAppendingString:fileName]];
      NSString *recoveredFilePath = [recoveredFilePathBase stringByAppendingPathExtension:ext];
      NSUInteger idx =  1U;
      while([fm fileExistsAtPath:recoveredFilePath])
        recoveredFilePath = [NSString stringWithFormat:@"%@-%lu.%@", recoveredFilePathBase, (unsigned long)idx++, ext];
#endif
      
      if(isICloud)
        ok = [fm setUbiquitous:NO itemAtURL:srcURL destinationURL:[NSURL fileURLWithPath:recoveredFilePath] error:&error];
      else
        ok = [fm moveItemAtPath:path toPath:recoveredFilePath error:&error];

      A77Log(@"Restore result = %d, error = %@", ok, error);
      
      if(!ok)
      {
        A77Log(@"Restore failed");
        
        allOK = NO;
        dispatch_async(dispatch_get_main_queue(), ^{
          self.tableView.userInteractionEnabled = YES;
          self.navigationController.navigationBar.userInteractionEnabled = YES;  
        
          MBProgressHUD *hud = [[MBProgressHUD alloc]initWithView:self.view];
          hud.customView = [[UIImageView alloc]initWithImage:[UIImage imageNamed:@"Bad.png"]];
          hud.mode = MBProgressHUDModeCustomView;
          hud.labelText = @"Could not restore";
          hud.detailsLabelText = error.localizedDescription;
          hud.animationType = MBProgressHUDAnimationZoom;
          hud.yOffset = -50;
          hud.opacity = .8;
          hud.removeFromSuperViewOnHide = YES;
          [self.navigationController.view addSubview:hud];
          [hud show:YES];
          [self performSelector:@selector(hideHud:) withObject:hud afterDelay:2]; // hideHUD will call [self release]
        });
        continue;
      }
      
#ifdef PWSAFE_COZI
      dispatch_sync(dispatch_get_main_queue(), ^{
        SafeCtx *safeCtx = [SafeCtxStorage sharedStorage][recoveredFilePath];
        
        NSString *originalDisplayName;
        if([path rangeOfString:coziFamilySafesDir()].location == 0)
        {
          NSString *originalPath = path.stringByDeletingPathExtension;
          SafeCtx *originalCtx = [SafeCtxStorage sharedStorage][originalPath];
          originalDisplayName = originalCtx.coziDisplayName;
        }
        else
        {
          originalDisplayName = path.lastPathComponent.stringByDeletingPathExtension.stringByDeletingPathExtension;
        }
        
        if(!originalDisplayName.length)
          safeCtx.coziDisplayName = [@"Restored vault - " stringByAppendingString:[safeId substringToIndex:4]];
        else
          safeCtx.coziDisplayName = [NSString stringWithFormat:@"Restored - %@ - %@", originalDisplayName, [safeId substringToIndex:4]];
        
        safeCtx.coziModified = YES;
        safeCtx.coziSafeId = safeId;
        CoziFamilySafes *cfs = [CoziFamilySafes sharedInstance];
        [cfs safeModifiedLocally:recoveredFilePath];
        if([cfs.delegate respondsToSelector:@selector(safeUpdated:)])
          [cfs.delegate safeUpdated:recoveredFilePath];
      });
#endif
    }
    if(allOK)
    {
      dispatch_async(dispatch_get_main_queue(), ^{
        self.tableView.userInteractionEnabled = YES;
        self.navigationController.navigationBar.userInteractionEnabled = YES;
        [self.navigationController popViewControllerAnimated:YES];
      });
    }
  });
}

- (void)hideHud:(MBProgressHUD *)hud
{
  [hud hide:YES];
}

@end
