//
//  RootViewController.m
//  iPasswordSafe
//
//  Created by Jorge Peixoto Vasquez on 3/4/11.
//  Copyright 2011 app77. All rights reserved.
//

#import "RootViewController.h"
#if defined(PWSAFE_WITH_DROPBOX) || defined (PWSAFE_WITH_ICLOUD)
# define RVC_HAS_SYNC
# import "FileViewControllerSync.h"
# import "SyncEnabled.h"
# define FileViewController FileViewControllerSync
#else
# import "FileViewController.h"
#endif
#import "OpenSafeViewController.h"
#import "SafeCtx.h"
#import "SafeCtxStorage.h"
#import "NewEditSafeViewController.h"
#import "NetworkActivityBroker.h"
#import "MBProgressHUD.h"
#import "DetailViewController.h"
#import "GroupViewController.h"
#import "Welcome.h"
#import "MacAppAdd.h"
#import "MobiusAdVc.h"
#import "AddHelpTextUITableViewController.h"
#import "MasterPasswordViewController.h"
#import "IASKSettingsReader.h"
#import "IASKPSToggleSwitchSpecifierViewCell.h"
#import "IASKSpecifier.h"
#import "IASKSwitch.h"
#import "SafeFile.h"
#ifdef PWSAFE_WITH_ICLOUD
#import "SafeFile_iCloud.h"
#define SafeFileClass SafeFile_iCloud
#else
#define SafeFileClass SafeFile
#endif

#import "RootViewController.h"
#import "EditableSimpleTableViewCell.h"
#import "iPasswordSafeAppDelegate.h"
#import "FileSystemFolders.h"
#import "UIViewController+OpenSafe.h"
#import "AppTheme.h"
#import "Appirater.h"
#import "InactiveScreenCover.h"
#import "DismissableNavigationController.h"
#import "BackendMessage.h"
#import "OpenInChromeController.h"

#import <Twitter/TWTweetComposeViewController.h>
#import <Social/Social.h>

#include <sys/types.h>
#include <sys/sysctl.h>
#include <fcntl.h>
#import <objc/runtime.h>

const void * const kAlertBlockKey = &kAlertBlockKey;
const void * const kAlertTextViewAlertKey = &kAlertTextViewAlertKey;

#ifndef RVC_HAS_SYNC
static NSInteger iCloudSection() { return -1; }
static NSInteger dummySection() { return -1; }
static NSUInteger numberOfSections() { return 1; }
static BOOL isICloudConfigured() { return NO; }
static BOOL isDropboxReady() { return NO; }
static BOOL isICloudSupported() { return NO; }
#endif

#define RVC_HELP_TEXT_TAG_BASE 2200

@interface RootViewController () <UIAlertViewDelegate, IASKSettingsDelegate, UINavigationControllerDelegate>
{
  NSMutableDictionary *m_state;
  NSMutableDictionary *m_closeSafeTimers;
  BOOL m_showWelcomeAfterSettings;
  BOOL m_showMacAddAfterSettings;
  BOOL m_showMobiusAddAfterSettings;
  IASKAppSettingsViewController *m_settingsViewController;
  int m_docDirFd;
  CFFileDescriptorRef m_docDirFileDescriptor;
  BOOL m_appeared;
  void (^m_onNavigationControllerShowedMe)();
}

@end

@implementation RootViewController

- (NSString *)dirOfSection:(NSUInteger)section
{
  if(section == RVC_SECTION_LOCAL)
    return localSafeDir();
  if(section == RVC_SECTION_DROPBOX)
    return dropboxDir();
  if(section == RVC_SECTION_ICLOUD)
    return m_iCloudURL.path;
  return nil;
}

- (void) setupNavBar 
{
  // Navigation bar
  self.navigationItem.rightBarButtonItem = self.editButtonItem;
  UIBarButtonItem *settingsBtn = [[UIBarButtonItem alloc]initWithImage:[UIImage imageNamed:imageName(@"Settings")] style:UIBarButtonItemStylePlain target:self action:@selector(onSettings:)];
  settingsBtn.accessibilityLabel = @"Settings";
  self.navigationItem.leftBarButtonItem = settingsBtn;
}

- (id) initWithState:(NSMutableDictionary *)state
{
  self = [super initWithStyle:UITableViewStyleGrouped];
  if(!self)
    return nil;

  dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
    recoverSafesInDir(localSafeDir());
  });
  
  m_docDirFd = -1;
  m_state = state;
  
  m_closeSafeTimers = [[NSMutableDictionary alloc]init];

  [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(on_settingsValueChanged:) name:kIASKAppSettingChanged object:nil];

  
//#warning LAUNCH SCREEN
//  if(!iOS6())
//  {
//    [[UIApplication sharedApplication]setStatusBarHidden:YES];
//    double delayInSeconds = 2.0;
//    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
//    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
//      CGRect frame = self.navigationController.navigationBar.frame;
//      frame.size.height = 64;
//      self.navigationController.navigationBar.frame = frame;
//      
//      if([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad && !iOS6())
//      {
//        GroupViewController *g = [DetailViewController sharedDetailViewControllerWithState:m_state].groupViewController;
//        frame = g.navigationController.navigationBar.frame;
//        frame.size.height = 64;
//        g.navigationController.navigationBar.frame = frame;
//        
//        UISplitViewController *split = (UISplitViewController *)[UIApplication sharedApplication].keyWindow.rootViewController;
//        
//        frame = split.view.frame;
//        frame.origin.y += 20;
//        frame.size.height -= 20;
//        split.view.frame = frame;
//        
//        frame.origin.y -= 20;
//        frame.size.height += 20;
//        split.view.frame = frame;
//        
//      }
//    });
//  }
//  self.tableView.dataSource = nil;
//  self.tableView.delegate = nil;
//  self.navigationItem.leftBarButtonItem = nil;
//  self.navigationItem.rightBarButtonItem = nil;
//  if([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad)
//  {
//    GroupViewController *g = [DetailViewController sharedDetailViewControllerWithState:m_state].groupViewController;
//    g.navigationItem.leftBarButtonItem = nil;
//    g.navigationItem.rightBarButtonItem = nil;
//    g.toolbarItems = nil;
//  }
//  return self;
  
  // Toolbar
  UIBarButtonItem *btn = [UIBarButtonItem buttonWithImageNamed:imageName(@"Close") target:self action:@selector(onCloseSafes:)];
  btn.accessibilityLabel = @"Close";
  UIBarButtonItem *spacer = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
  UIBarButtonItem *addBtn = [UIBarButtonItem buttonWithImageNamed:imageName(@"Plus") target:self action:@selector(onAddNewItem:)];
  addBtn.accessibilityLabel = @"Add";
  NSMutableArray *buttons = [NSMutableArray arrayWithObjects:btn, spacer, nil];
#ifdef PWSAFE_WITH_DROPBOX  
  UIBarButtonItem *dropboxBtn = [UIBarButtonItem buttonWithImageNamed:imageName(@"Dropbox") target:self action:@selector(onDropboxBtn:)];
  dropboxBtn.accessibilityLabel = @"Dropbox";
  [buttons addObject:dropboxBtn];
  [buttons addObject:spacer];
#endif
  [buttons addObject:addBtn];
  
  self.toolbarItems = buttons;
  
  [self setupNavBar];
  
  return self;
}

- (void)loadSavedState
{
  [self loadSavedState_OpenSafe:m_state];
}

-(void)refresh
{
  m_filesInDocsFolder = nil;
  m_filesInDropboxFolder = nil;
  m_filesInICloudFolder = nil;
  
  [self.tableView reloadData];
}

- (void)insertSafe:(NSString *)newSafeFileName inTable:(NSMutableArray *)table section:(NSUInteger)section
{
  NSUInteger idx = 0;
  NSUInteger max = table.count;
  while(idx < max && [table[idx]caseInsensitiveCompare:newSafeFileName] == NSOrderedAscending)
    idx++;
  [table insertObject:newSafeFileName atIndex:idx];
  UITableView *tv = self.tableView;
  [tv insertRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:idx inSection:section]] withRowAnimation:UITableViewRowAnimationTop];
  if(iOS6() && max > 0)
  {
    double delayInSeconds = 0.3;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
      if(idx == 0)
        [tv reloadRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:1 inSection:section]] withRowAnimation:UITableViewRowAnimationNone];
      else if(idx == max)
        [tv reloadRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:idx - 1 inSection:section]] withRowAnimation:UITableViewRowAnimationNone];
    });
  }

  [Crashlytics setIntValue:(int)table.count forKey:section == RVC_SECTION_LOCAL ? @"Local safes" : (section == RVC_SECTION_DROPBOX ? @"Dropbox safes" : @"iCloud safes")];
}

void recoverSafesInDir(NSString *dir)
{
  NSFileManager *fileMan = [NSFileManager defaultManager];
  NSError *error = nil;
  NSArray *dirContents = [fileMan contentsOfDirectoryAtPath:dir error:&error];
  if(!dirContents)
  {
    A77Log(@"Could not read directory '%@': %@", dir, [error localizedDescription]);
    return;
  }
  for(NSString *entry in dirContents)
  {
    NSString *path = [dir stringByAppendingPathComponent:entry];
    BOOL isDir;
    BOOL ok = [fileMan fileExistsAtPath:path isDirectory:&isDir];
    if(!ok || isDir)
      continue;
    NSString *ext = [entry.pathExtension lowercaseString];
    if([ext isEqualToString:@"tmp"])
    {
      NSString *newFileName = [entry stringByDeletingPathExtension];
      ext = [newFileName.pathExtension lowercaseString];
      if(![ext isEqualToString:@"psafe3"] && ![ext isEqualToString:@"dat"])
        continue;
      
      // Delete temporary file
      ok = [fileMan removeItemAtPath:path error:nil];
      NSCAssert(ok, @"Delete must succeed");
    }
  }
}

- (NSMutableArray *) safesInDirectory: (NSString *) dir  
{
  NSFileManager *fileMan = [NSFileManager defaultManager];
  NSError *error = nil;
  NSArray *dirContents = [fileMan contentsOfDirectoryAtPath:dir error:&error];
  if(!dirContents)
  {
    A77Log(@"Could not read directory '%@': %@", dir, [error localizedDescription]);
    return [NSMutableArray arrayWithCapacity:2];
  }
  NSMutableArray *newFilesInDocsFolder = [[NSMutableArray alloc]initWithCapacity:dirContents.count];
  for(NSString *entry in dirContents)
  {
    NSString *stdEntry = [entry precomposedStringWithCanonicalMapping];
    NSString *path = [dir stringByAppendingPathComponent:stdEntry];
    BOOL isDir;
    BOOL ok = [fileMan fileExistsAtPath:path isDirectory:&isDir];
    NSString *ext = [stdEntry.pathExtension lowercaseString];
    if(ok && !isDir && ([ext isEqualToString:@"psafe3"] || [ext isEqualToString:@"dat"]))
    {
      // Normal file
      [newFilesInDocsFolder addObject:stdEntry];
    }
  }
  
  return [NSMutableArray arrayWithArray:[newFilesInDocsFolder sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)]];
}

-(void)createNewLocalSafe
{
  if(self.editing)
    [self setEditing:NO animated:YES];
  
  NewEditSafeViewController *newSafeVC = [[NewEditSafeViewController alloc]initWithTarget:self selector:@selector(onAddComplete:) directory:localSafeDir() locationName:@"local"];
  NSMutableArray *filenames = [[NSMutableArray alloc]initWithCapacity:m_filesInDocsFolder.count];
  for(NSUInteger idx = 0; idx < m_filesInDocsFolder.count; idx++)
    [filenames addObject:[m_filesInDocsFolder[idx]lowercaseString].precomposedStringWithCanonicalMapping.stringByDeletingPathExtension];
  newSafeVC.forbiddenNames = filenames;
  UINavigationController *nav = [[UINavigationController alloc]initWithRootViewController:newSafeVC];
  nav.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    nav.modalPresentationStyle = UIModalPresentationFormSheet;
  [self presentModalViewController:nav animated:YES];
}

-(void)onAddComplete:(id)sender
{
  NewEditSafeViewController *newSafeVC = (NewEditSafeViewController *)sender;
  if(newSafeVC.result)
    [self refresh];

  [self dismissModalViewControllerAnimated:YES];
}

- (void)onAddNewItem:(id)sender
{
  [self createNewLocalSafe]; 
}

- (void) onSettings:(id)sender
{
  m_showWelcomeAfterSettings = NO;
  m_showMacAddAfterSettings = NO;
  m_showMobiusAddAfterSettings = NO;
  m_settingsViewController = [[IASKAppSettingsViewController alloc] initWithNibName:@"IASKAppSettingsView" bundle:nil];
  m_settingsViewController.showCreditsFooter = NO;
  m_settingsViewController.delegate = self;
  m_settingsViewController.showAppVersion = YES;
  
  DismissableNavigationController *nav = [[DismissableNavigationController alloc]initWithRootViewController:m_settingsViewController];
  nav.modalPresentationStyle = UIModalPresentationFormSheet;
  nav.modalTransitionStyle = (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || !iOS6()) ? UIModalTransitionStyleCoverVertical : UIModalTransitionStyleFlipHorizontal;
  
  [self presentModalViewController:nav animated:YES];
}

- (NSString *) filePath: (NSIndexPath *) indexPath  
{
  NSUInteger section = indexPath.section;
  if(section == RVC_SECTION_LOCAL)
  {
    return [localSafeDir() stringByAppendingPathComponent:m_filesInDocsFolder[indexPath.row]];
  }
  else if(section == RVC_SECTION_DROPBOX)
  {
    return [dropboxDir() stringByAppendingPathComponent:m_filesInDropboxFolder[indexPath.row]];
  }
  else if(section == RVC_SECTION_ICLOUD)
  {
    return [m_iCloudURL.path stringByAppendingPathComponent:m_filesInICloudFolder[indexPath.row]];
  }
  else
  {
    NSAssert(0, @"don't know the section type");
    return nil;
  }
}

- (void)onReloadTimerFired:(NSTimer *)timer
{
  NSString *safePath = [timer userInfo];
  [m_closeSafeTimers removeObjectForKey:safePath];
  // No files changed, just closed / open state
  [self.tableView reloadData];
}

static void closeFile(NSString *file, NSString *dir) 
{
  NSString *safePath = [dir stringByAppendingPathComponent:file];
  SafeCtx *ctx = [[SafeCtxStorage sharedStorage]safeCtxForPath:safePath];
  [ctx expireNow];    
};

- (void)onCloseSafes:(id)sender
{
  NSString *localDir = localSafeDir();
  NSString *dbDir = dropboxDir();
  NSString *icDir = m_iCloudURL.path;
  
  for(NSString *file in m_filesInDocsFolder)
    closeFile(file, localDir);
  
  for(NSString *file in m_filesInDropboxFolder)
    closeFile(file, dbDir);

  if(icDir && m_filesInICloudFolder)
  {
    for(NSString *file in m_filesInICloudFolder)
      closeFile(file, icDir);
  }
    
  [self refresh];
  
  [UIPasteboard generalPasteboard].string = @"";
  
  UIView *rootView = [MBProgressHUD rootView];
  MBProgressHUD *hud = [[MBProgressHUD alloc]initWithView:rootView];
  hud.customView = [[UIImageView alloc]initWithImage:[UIImage imageNamed:@"Check.png"]];
  hud.mode = MBProgressHUDModeCustomView;
  hud.labelText = @"Clipboard cleared";
  hud.animationType = MBProgressHUDAnimationZoom;
  hud.yOffset = -50;
  hud.opacity = .8;
  hud.removeFromSuperViewOnHide = YES;
  [rootView addSubview:hud];
  [hud show:YES];
  [self performSelector:@selector(hideHud:) withObject:hud afterDelay:1];
}

- (void)showWelcome
{
  Welcome *welcome = [[Welcome alloc]init];
  welcome.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    welcome.modalPresentationStyle = UIModalPresentationFormSheet;
  [self presentModalViewController:welcome animated:YES];
}

//! return whether welcome screen has been shown or not
- (BOOL)showWelcomeIfNeeded
{
  // Leave for next time if there is another modal window
  if(self.modalViewController)
    return NO;
  
  NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
  BOOL hideWelcomeScreen = [prefs boolForKey:@"hideWelcomeScreen"];
  if(hideWelcomeScreen)
    return NO;
  
  [self showWelcome];
  return YES;
}

- (void)showMobiusAd
{
  MobiusAdVc *ad = [[MobiusAdVc alloc]init];
  ad.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    ad.modalPresentationStyle = UIModalPresentationFormSheet;
  [self presentModalViewController:ad animated:YES];
}

static NSString * const kMacAdd_nSkipped = @"MacAdd_nSkipped";
static NSString * const kMacAdd_alreadyViewed = @"MacAdd_alreadyViewed";

- (void)showMacAdd
{
  MacAppAdd *add = [[MacAppAdd alloc]init];
  add.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    add.modalPresentationStyle = UIModalPresentationFormSheet;
  [self presentModalViewController:add animated:YES];
}

- (void)showMacAddIfNeeded
{
  // Leave for next time if there is another modal window
  if(self.modalViewController)
  {
    A77Log(@"MacAdd: another modal VC, skipping");
    return;
  }

  NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];

  BOOL alreadyViewed = [prefs boolForKey:kMacAdd_alreadyViewed];
  
  if(alreadyViewed)
  {
    A77Log(@"MacAdd: already shown, not showing again");
    return;
  }
  
  NSInteger nSkipped = [prefs integerForKey:kMacAdd_nSkipped];
  if(nSkipped < 8)
  {
    nSkipped++;
    [prefs setInteger:nSkipped forKey:kMacAdd_nSkipped];
    A77Log(@"MacAdd: skipped %ld time, skipping again", (long)nSkipped - 1);
    return;
  }
  
  A77Log(@"MacAdd: reached time to see it");
  [self showMacAdd];
  
  [prefs setBool:YES forKey:kMacAdd_alreadyViewed];
}

#pragma mark - docs monitoring

- (void)moveRowsReflectingTableChangeFrom:(NSArray *)oldTable to:(NSArray *)newTable inSection:(NSInteger)section
{
  NSMutableArray *rowsToDelete = nil;
  NSMutableArray *rowsToInsert = nil;
  NSMutableArray *rowsToReload = nil;
  
  // Calc rows to delete
  for(NSUInteger i = 0; i < oldTable.count; i++)
  {
    NSString *fileName = oldTable[i];
    if([newTable indexOfObject:fileName] != NSNotFound)
      continue;
    DebugLog(@"Row to delete: %lu", (unsigned long)i);
    if(!rowsToDelete)
      rowsToDelete = [NSMutableArray arrayWithCapacity:oldTable.count];
    [rowsToDelete addObject:[NSIndexPath indexPathForRow:i inSection:section]];
  }
  
  // Calc rows to insert
  for(NSUInteger i = 0; i < newTable.count; i++)
  {
    NSString *fileName = newTable[i];
    if([oldTable indexOfObject:fileName] != NSNotFound)
      continue;
    DebugLog(@"Row to insert: %lu", (unsigned long)i);
    if(!rowsToInsert)
      rowsToInsert = [NSMutableArray arrayWithCapacity:oldTable.count];
    [rowsToInsert addObject:[NSIndexPath indexPathForRow:i inSection:section]];
  }
  
  // Calc rows to reload
  if(rowsToDelete && rowsToInsert)
  {
    for(NSIndexPath *idx in rowsToDelete)
    {
      if([rowsToInsert indexOfObject:idx] == NSNotFound)
        continue;
      if(!rowsToReload)
        rowsToReload = [NSMutableArray arrayWithCapacity:oldTable.count];
      
      DebugLog(@"Row to reload: %ld", (long)idx.row);
      [rowsToReload addObject:idx];
    }
  }
  
  // this has to be done after the loops above, because it modifies the iterated colections rowsToInsert and rowsToDelete
  if(rowsToReload)
  {
    for(NSIndexPath *idx in rowsToReload)
    {
      [rowsToInsert removeObject:idx];
      [rowsToDelete removeObject:idx];
    }
  }

  if(!rowsToInsert && !rowsToDelete && !rowsToReload)
    return;
  
  // Failsafe
  if(oldTable.count + (rowsToInsert ? rowsToInsert.count : 0) - (rowsToDelete ? rowsToDelete.count : 0) != newTable.count)
  {
    A77Log(@"Failsafe triggered - rowsToInsert: %@, rowsToDelete: %@, rowsToReload: %@, m_filesInIcloud: %@, oldTable = %@", rowsToInsert, rowsToDelete, rowsToReload, newTable, oldTable);
    [self.tableView reloadSections:[NSIndexSet indexSetWithIndex:section] withRowAnimation:UITableViewRowAnimationLeft];
    return;
  }
  
  A77Log(@"Updating table view on section %ld change - rowsToInsert: %@, rowsToDelete: %@, rowsToReload: %@, newTable.count: %lu, oldTable.count = %lu", (long)section, rowsToInsert, rowsToDelete, rowsToReload, (u_long)newTable.count, (u_long)oldTable.count);
  
  [self.tableView beginUpdates];   
  
  if(rowsToReload && rowsToReload.count)
    [self.tableView reloadRowsAtIndexPaths:rowsToReload withRowAnimation:UITableViewRowAnimationLeft];
  if(rowsToDelete && rowsToDelete.count)
    [self.tableView deleteRowsAtIndexPaths:rowsToDelete withRowAnimation:UITableViewRowAnimationLeft];
  if(rowsToInsert && rowsToInsert.count)
    [self.tableView insertRowsAtIndexPaths:rowsToInsert withRowAnimation:UITableViewRowAnimationLeft];

  [self.tableView endUpdates];
  
  NSUInteger newCount = newTable.count;
  if(iOS6() && newCount > 0)
  {
    NSIndexPath *first = [NSIndexPath indexPathForRow:0 inSection:section];

    NSMutableArray *lateReload = [NSMutableArray arrayWithCapacity:4];
    [lateReload addObject:first];
  
    if(newCount > 1)
    {
      NSIndexPath *last = [NSIndexPath indexPathForRow:newCount - 1 inSection:section];
      [lateReload addObject:last];
      
      if(newCount > 2)
      {
        NSIndexPath *second = [NSIndexPath indexPathForRow:1 inSection:section];
        [lateReload addObject:second];
        
        if(newCount > 3)
        {
          NSIndexPath *beforeLast = [NSIndexPath indexPathForRow:newCount - 2 inSection:section];
          [lateReload addObject:beforeLast];
        }
      }
    }
  
    double delayInSeconds = .3;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
      [self.tableView reloadRowsAtIndexPaths:lateReload withRowAnimation:UITableViewRowAnimationNone];
    });
  }
}

- (void)onDocDirKQueue
{
  if(!m_docDirFileDescriptor)
    return;
  
  // Pull the native FD around which the CFFileDescriptor was wrapped
  int kq = CFFileDescriptorGetNativeDescriptor(m_docDirFileDescriptor);
  if (kq < 0) 
    return;

  // If we pull a single available event out of the queue, assume the directory was updated
  struct kevent event;
  struct timespec timeout = {0, 0};
  int n = kevent(kq, NULL, 0, &event, 1, &timeout);
  // (Re-)Enable a one-shot (the only kind) callback
  CFFileDescriptorEnableCallBacks(m_docDirFileDescriptor, kCFFileDescriptorReadCallBack);
  if(n != 1)
    return;

  NSMutableArray *newFiles = [self safesInDirectory:localSafeDir()];
  
  if(!m_filesInDocsFolder)
  {
    m_filesInDocsFolder = newFiles;
    [self.tableView reloadSections:[NSIndexSet indexSetWithIndex:RVC_SECTION_LOCAL] withRowAnimation:UITableViewRowAnimationNone];
    return;
  }

  NSArray *oldFiles = m_filesInDocsFolder;  
  m_filesInDocsFolder = newFiles;

  [self moveRowsReflectingTableChangeFrom:oldFiles to:newFiles inSection:RVC_SECTION_LOCAL];

  [Crashlytics setIntValue:(int)m_filesInDocsFolder.count forKey:@"Local safes"];
}

static void onDocDirKQueue(CFFileDescriptorRef fdref, CFOptionFlags callBackTypes, void *info)
{
  RootViewController *rootVC = (__bridge RootViewController *)info;
  
#ifdef DEBUG_MODE
  double delayInSeconds = 5.0;
  dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
  dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
      [rootVC onDocDirKQueue];
  });
#else
  [rootVC onDocDirKQueue];
#endif
}

- (void)stopDocsMonitor
{
  DebugLog(@"Stopping docs monitor (desc = %p, fd = %d)", m_docDirFileDescriptor, m_docDirFd);
  if(m_docDirFileDescriptor)
  {
    CFFileDescriptorInvalidate(m_docDirFileDescriptor);
    CFRelease(m_docDirFileDescriptor);
    m_docDirFileDescriptor = NULL;
  }
  if(m_docDirFd >= 0)
  {
    close(m_docDirFd);
    m_docDirFd = -1;
  }
}

- (void)startDocsMonitor
{
  DebugLog(@"Starting docs monitor (desc = %p, fd = %d)", m_docDirFileDescriptor, m_docDirFd);
  if(m_docDirFileDescriptor || m_docDirFd >= 0)
    [self stopDocsMonitor];
  
  // Monitors the documents folder
  int kq = kqueue();
  if(kq < 0)
    return;
  
  m_docDirFd = open([localSafeDir() cStringUsingEncoding:NSUTF8StringEncoding], O_EVTONLY);
  if(m_docDirFd < 0)
  {
    close(kq);
    return;
  }
  struct kevent ke;
  EV_SET(&ke, m_docDirFd, EVFILT_VNODE, EV_ADD|EV_ENABLE|EV_CLEAR, NOTE_WRITE, 0, 0);
  struct timespec pollOnly = { 0, 0 };
  int n = kevent(kq, &ke, 1, NULL, 0, &pollOnly);
  if(n < 0)
  {
    close(m_docDirFd);
    m_docDirFd = -1;
    close(kq);
    return;
  }
  
  CFFileDescriptorContext ctx = { 0, (__bridge void *)(self), NULL, NULL, NULL };
  m_docDirFileDescriptor = CFFileDescriptorCreate(kCFAllocatorDefault, kq, true, onDocDirKQueue, &ctx);
  if(!m_docDirFileDescriptor)
  {
    close(m_docDirFd);
    m_docDirFd = -1;
    close(kq);
    return;
  }
  CFRunLoopSourceRef source = CFFileDescriptorCreateRunLoopSource(kCFAllocatorDefault, m_docDirFileDescriptor, 0);
  if(!source)
  {
    CFFileDescriptorInvalidate(m_docDirFileDescriptor);
    CFRelease(m_docDirFileDescriptor);
    m_docDirFileDescriptor = NULL;
    close(m_docDirFd);
    m_docDirFd = -1;    
    return;
  }
  CFRunLoopAddSource(CFRunLoopGetMain(), source, kCFRunLoopDefaultMode);
  CFRelease(source);
  CFFileDescriptorEnableCallBacks(m_docDirFileDescriptor, kCFFileDescriptorReadCallBack);
}

- (void)appDidBecomeActiveInternal
{
  double delayInSeconds = 3;
  dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
  dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
    [self startDocsMonitor];
  });
}

- (void)appWillResignActive
{
  [self stopDocsMonitor];
}

- (void)appDidBecomeActive
{
  double delayInSeconds = .6;
  dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
  dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
    [self appDidBecomeActiveInternal];
  });
}

#pragma mark - Settings

static NSString * const kChangeMasterPasswordKey = @"changeMasterPassword";
static NSString * const kAutoClearPasswordFromPasteboardKey = @"clearPasswordFromPasteboard";
static NSString * const kAutoClearPasswordTimeoutMinutesKey = @"clearPasswordTimeoutMinutes";
static NSString * const kAutoClearPasswordNotifyKey = @"clearPasswordNotify";
NSString * const kOpenInChromeKey = @"openInChrome";

static BOOL autoClearPasswordFromPasteboard()
{
  NSNumber *obj = [[NSUserDefaults standardUserDefaults]objectForKey:kAutoClearPasswordFromPasteboardKey];
  if(!obj)
    return YES;
  return obj.boolValue;
}


- (void)on_settingsValueChanged:(NSNotification *)notification
{
  NSString *key = notification.object;
  NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
  IASKSettingsReader *reader = m_settingsViewController.settingsReader;
  NSMutableArray *indexes = [NSMutableArray arrayWithCapacity:2];

  if([key isEqualToString:kAutoClearPasswordFromPasteboardKey])
  {
    for(NSInteger s = 0; s < [reader numberOfSections]; s++)
    {
      NSUInteger nRows = [reader numberOfRowsForSection:s];
      if(nRows == 0)
        continue;
      NSString *k = [reader specifierForIndexPath:[NSIndexPath indexPathForRow:0 inSection:s]].key;
      if(![k isEqualToString:kAutoClearPasswordFromPasteboardKey])
        continue;
      
      for(NSInteger r = 1; r < nRows; r++)
      {
        NSIndexPath *indexPath = [NSIndexPath indexPathForRow:r inSection:s];
        [indexes addObject:indexPath];
      }
      break;
    }
    
    [m_settingsViewController.tableView reloadRowsAtIndexPaths:indexes withRowAnimation:UITableViewRowAnimationFade];
  }
  else if([key isEqualToString:kMasterPasswordEnabledKey])
  {
    BOOL enabled = ![prefs boolForKey:kMasterPasswordEnabledKey]; // Value is already toggled
    
    for(NSInteger s = 0; s < [reader numberOfSections]; s++)
    {
      for(NSInteger r = 0; r < [reader numberOfRowsForSection:s]; r++)
      {
        NSIndexPath *indexPath = [NSIndexPath indexPathForRow:r inSection:s];
        NSString *k = [[reader specifierForIndexPath:indexPath]key];
        if(![k isEqualToString:kChangeMasterPasswordKey] && ![k isEqualToString:kMasterPasswordExpireMinutesKey])
          continue;
        
        // Found
        [indexes addObject:indexPath];
        if(indexes.count < 2)
          continue;
        
        void (^callback)(BOOL ok) = ^(BOOL ok)
        {
          if(ok)
          {
            [m_settingsViewController.tableView reloadRowsAtIndexPaths:indexes withRowAnimation:UITableViewRowAnimationFade];
            return;
          }
          
          [prefs setBool:enabled forKey:kMasterPasswordEnabledKey];
          IASKPSToggleSwitchSpecifierViewCell *cell = (IASKPSToggleSwitchSpecifierViewCell *)[m_settingsViewController.tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:r - 2 inSection:s]];
          [cell.toggle setOn:enabled animated:YES];
        };
        
        if(!enabled)
          [MasterPasswordViewController enableInViewController:self.modalViewController callback:callback];
        else
          [MasterPasswordViewController disableInViewController:self.modalViewController callback:callback];
        
      }
    }
  }
}

- (BOOL)isEnabledForSpecifier:(IASKSpecifier*)specifier
{
  NSString *key = specifier.key;
  
  if([key isEqualToString:kChangeMasterPasswordKey] || [key isEqualToString:kMasterPasswordExpireMinutesKey])
    return [MasterPasswordViewController isEnabled];
  
  if([key isEqualToString:kAutoClearPasswordNotifyKey] || [key isEqualToString:kAutoClearPasswordTimeoutMinutesKey])
    return autoClearPasswordFromPasteboard();

  if([key isEqualToString:kOpenInChromeKey])
    return [[OpenInChromeController sharedInstance]isChromeInstalled];
    
  return YES;
}

- (void)settingsViewControllerDidEnd:(IASKAppSettingsViewController*)sender
{
  m_settingsViewController = nil;
  [self dismissModalViewControllerAnimated:YES]; 
}

static NSString *sysPlatform()
{
  size_t size;
  sysctlbyname("hw.machine", NULL, &size, NULL, 0);
  char *machine = malloc(size);
  sysctlbyname("hw.machine", machine, &size, NULL, 0);
  NSString *platform = [NSString stringWithCString:machine encoding:NSISOLatin1StringEncoding];
  free(machine);
  return platform;
}

- (NSString*)mailComposeBodyForSubject:(NSString *)subject
{
  NSMutableString *body = [[NSMutableString alloc]initWithCapacity:1024];
  
  if([subject rangeOfString:@"support"].location != NSNotFound)
  {
    [body appendFormat:@"pwSafe version: %@\r\n", [[NSBundle mainBundle]objectForInfoDictionaryKey:@"CFBundleShortVersionString"]];
    [body appendFormat:@"iCloud: %@\r\nDropbox: %@\r\n", m_iCloudURL != nil ? @"yes" : @"no", isDropboxReady() ? @"yes" : @"no"];
    [body appendFormat:@"Safes: %lu / %lu / %lu\r\n", (u_long)m_filesInDocsFolder.count, (u_long)m_filesInDropboxFolder.count, (u_long)m_filesInICloudFolder.count];
    [body appendFormat:@"Country: %@\r\nLanguage: %@\r\n", [[NSLocale currentLocale]objectForKey:NSLocaleCountryCode], [NSLocale preferredLanguages][0]];
    [body appendFormat:@"Device type code: %@\r\niOS version: %@\r\n", sysPlatform(), [[UIDevice currentDevice] systemVersion]];
  }
  else if([subject rangeOfString:@"App development"].location != NSNotFound)
  {
    [body appendString:@"Please tell us about the app you would like us to develop."];
  }

  return body;
}

- (void)settingsViewController:(IASKAppSettingsViewController*)sender buttonTappedForKey:(NSString*)key
{
  if([key isEqualToString:@"showWelcomeNow"])
  {
    m_showWelcomeAfterSettings = YES;
    [sender dismiss:self];
  }
  else if([key isEqualToString:@"showMacAddNow"])
  {
    m_showMacAddAfterSettings = YES;
    [sender dismiss:self];
  }
  else if([key isEqualToString:@"showMobiusAdNow"])
  {
    m_showMobiusAddAfterSettings = YES;
    [sender dismiss:self];
  }
  else if([key isEqualToString:@"changeMasterPassword"])
  {
    [MasterPasswordViewController changePasswordInViewController:self.modalViewController];
  }
  else if([key isEqualToString:@"shareTweet"] || [key isEqualToString:@"shareFacebook"])
  {
    BOOL tweet = [key isEqualToString:@"shareTweet"];
    NSString *deviceName = UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad ? @"iPad" : @"iPhone";
    NSString *text = [NSString stringWithFormat:@"I use pwSafe on my %@ to manage my passwords. It's simple, secure and synced. Check it out!", deviceName];
    NSURL *url = [NSURL URLWithString:@"http://itunes.apple.com/us/app/pwsafe-password-safe-for-ios/id440783112?mt=8"];
    
    if([SLComposeViewController self])
    {
      NSString *socialNetwork = tweet ? SLServiceTypeTwitter : SLServiceTypeFacebook;
      NSString *socialNetworkName = tweet ? @"Twitter" : @"Facebook";
      
      if(![SLComposeViewController isAvailableForServiceType:socialNetwork])
      {
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:[NSString stringWithFormat:@"%@ not configured", socialNetworkName]
                                                       message:[NSString stringWithFormat:@"You need to configure %@ in the %@ Settings app.", socialNetworkName, deviceName]
                                                      delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        A77Log(@"%@NotConfigured", socialNetworkName);
        return;
      }
      
      SLComposeViewController *slComposeVC = [SLComposeViewController composeViewControllerForServiceType:socialNetwork];
      __weak SLComposeViewController *slComposeVCWeak = slComposeVC;
      [slComposeVC setInitialText:text];
      [slComposeVC addURL:url];
      
      [slComposeVC setCompletionHandler:^(SLComposeViewControllerResult result) {
        switch (result)
        {
          case SLComposeViewControllerResultCancelled:
            A77Log(@"%@ Canceled", socialNetworkName);
            break;
          case SLComposeViewControllerResultDone:
            A77Log(@"%@ Sent", socialNetworkName);
            break;
          default:
            break;
        }
        // Dismiss the tweet composition view controller.
        [slComposeVCWeak dismissModalViewControllerAnimated:YES];
        [InactiveScreenCover sharedCover].disableHideScreenOnInactive = NO;
      }];
      
      // Present the tweet composition view controller modally.
      [InactiveScreenCover sharedCover].disableHideScreenOnInactive = YES;
      [self.modalViewController presentModalViewController:slComposeVC animated:YES];
      
      return;
    }
    
    if(!tweet)
    {
      UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"iOS 6 Required" message:@"iOS 6 is needed to post in Facebook." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
      [alert show];
      A77Log(@"TwitterNotSupported");
      return;
    }
    
    if(![TWTweetComposeViewController canSendTweet])
    {
      UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"No Twitter Acount" message:[NSString stringWithFormat:@"You need to configure your Twitter account in the %@ Settings app.", deviceName] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
      [alert show];
      A77Log(@"TwitterNotConfigured");
      return;
    }
    
    // Set up the built-in twitter composition view controller.
    TWTweetComposeViewController *tweetViewController = [[TWTweetComposeViewController alloc] init];
    __weak TWTweetComposeViewController *tweetViewControllerWeak = tweetViewController;
    
    // Set the initial tweet text. 
    [tweetViewController setInitialText:text];
    [tweetViewController addURL:url];

    // Create the completion handler block.
    [tweetViewController setCompletionHandler:^(TWTweetComposeViewControllerResult result) 
    {
      switch (result) 
      {
        case TWTweetComposeViewControllerResultCancelled:
          A77Log(@"TweetCanceled");
          break;
        case TWTweetComposeViewControllerResultDone:
          A77Log(@"TweetSent");
          break;
        default:
          break;
      }
      // Dismiss the tweet composition view controller.
      [tweetViewControllerWeak dismissModalViewControllerAnimated:YES];
      [InactiveScreenCover sharedCover].disableHideScreenOnInactive = NO;
    }];
    
    // Present the tweet composition view controller modally.
    [InactiveScreenCover sharedCover].disableHideScreenOnInactive = YES;
    [self.modalViewController presentModalViewController:tweetViewController animated:YES];
  }
}

- (NSURL *)settingsViewController:(IASKAppSettingsViewController *)sender urlToOpenForKey:(NSString *)key
{
  if(![key isEqualToString:@"RateApp"])
    return nil;

  return [Appirater ratingURL];
}

- (void)settingsViewControllerDeallocing:(IASKAppSettingsViewController *)sender
{
  if(m_showWelcomeAfterSettings)
    [self showWelcome];
  else if(m_showMacAddAfterSettings)
    [self showMacAdd];
  else if(m_showMobiusAddAfterSettings)
    [self showMobiusAd];
}


#pragma mark -
#pragma mark View lifecycle

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
  [super didRotateFromInterfaceOrientation:fromInterfaceOrientation];
  [self adjustHelpFramesViewTagBase:RVC_HELP_TEXT_TAG_BASE];
  [self setupNavBar];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)viewDidLoad
{
  DebugLog(@"RootViewController -> viewDidLoad");
  [super viewDidLoad];
  
  UITableView *tableView = self.tableView;
  [tableView themeViewDidLoadInIpadPos:TVP_LEFT];
  if([tableView respondsToSelector:@selector(setSeparatorInset:)])
    [tableView setSeparatorInset:UIEdgeInsetsMake(0, 15, 0, 0)];
  
  
  self.navigationItem.title = @"pwSafe";
  
#ifdef PWSAFE_VAULT
  UIBarButtonItem *backButton = [[UIBarButtonItem alloc] initWithTitle:@"Vaults" style:UIBarButtonItemStyleBordered target:nil action:nil];
#else
  UIBarButtonItem *backButton = [[UIBarButtonItem alloc] initWithTitle:@"Safes" style:UIBarButtonItemStyleBordered target:nil action:nil];
#endif
  self.navigationItem.backBarButtonItem = backButton;

  DebugLog(@"RootViewController -> viewDidLoad / done");
}

- (void)viewWillAppear:(BOOL)animated 
{
  [self.navigationController setToolbarHidden:NO animated:animated];
  
  [super viewWillAppear:animated];

  [self clearState:m_state];

  if(self.currentSafeCtx)
  {
    NSAssert(self.currentOpenedFile, @"Must a have a file too");
    
    [self closeSafeFile:self.currentOpenedFile];
    self.currentOpenedFile = nil;
    
    SafeCtx *currentSafeCtx = self.currentSafeCtx;
    
    if(currentSafeCtx.opened)
    {
      NSTimeInterval keepAlivePeriod =currentSafeCtx.keepAlivePeriod;
      NSString *safePath = currentSafeCtx.safePath;
      if(keepAlivePeriod)
      {
        NSTimer *newTimer = [NSTimer scheduledTimerWithTimeInterval:keepAlivePeriod * 60 
                                                          target:self
                                                        selector:@selector(onReloadTimerFired:) 
                                                        userInfo:safePath 
                                                         repeats:NO];
        NSTimer *oldTimer = m_closeSafeTimers[safePath];
        [oldTimer invalidate];
        m_closeSafeTimers[safePath] = newTimer;
      }
      else
      {
        [m_closeSafeTimers removeObjectForKey:currentSafeCtx.safePath];
      }
      [currentSafeCtx sleep];      
      currentSafeCtx.opened = NO;
    }

    self.currentSafeCtx = nil;
    
    [self refresh];
  }
  
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    [[DetailViewController sharedDetailViewControllerWithState:m_state]clear];
}

- (void)viewDidAppear:(BOOL)animated
{
  [super viewDidAppear:animated];
  if(!m_appeared)
  {
    m_appeared = YES;
    iPasswordSafeAppDelegate *dg = [UIApplication sharedApplication].delegate;
    [dg runNowOrAfterMasterPassword:^{
      if(![self showWelcomeIfNeeded])
        [self showMacAddIfNeeded];
    }];
  }
  
  showBackendMessage(@"MOTD", nil);
}

#pragma mark -
#pragma mark Table view data source

// Customize the number of sections in the table view.
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView 
{
  return RVC_SECTION_COUNT;
}

// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section 
{
  if(!m_filesInDocsFolder)
  {
    m_filesInDocsFolder = [self safesInDirectory:localSafeDir()];
    [Crashlytics setIntValue:(int)m_filesInDocsFolder.count forKey:@"Local safes"];
  }
  if(!m_filesInDropboxFolder)
  {
    m_filesInDropboxFolder = [self safesInDirectory:dropboxDir()];
    [Crashlytics setIntValue:(int)m_filesInDropboxFolder.count forKey:@"Dropbox safes"];
  }
  if(!m_filesInICloudFolder && m_iCloudURL)
  {
    m_filesInICloudFolder = [self safesInDirectory:m_iCloudURL.path];
    [Crashlytics setIntValue:(int)m_filesInICloudFolder.count forKey:@"iCloud safes"];
  }

  // This method gets called in every insertion / deletion
  BOOL empty = 0 == m_filesInDocsFolder.count + m_filesInDropboxFolder.count + m_filesInICloudFolder.count;
  self.navigationItem.rightBarButtonItem.enabled = !empty;
#ifdef PWSAFE_VAULT
  NSString *helpText = @"Tap here to create\na new vault";
#else
  NSString *helpText = @"Tap here to create\na new safe";
#endif
  
  [self adjustAddHelpText:helpText viewTagBase:RVC_HELP_TEXT_TAG_BASE show:empty];
  
  if(section == RVC_SECTION_DUMMY)
    return 1; // Dummy row

  if(section == RVC_SECTION_LOCAL)
    return m_filesInDocsFolder.count;
  if(section == RVC_SECTION_DROPBOX)
    return m_filesInDropboxFolder.count;
  if(section == RVC_SECTION_ICLOUD)
    return m_filesInICloudFolder.count;

  return 0;    
}

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath 
{
  NSInteger section = indexPath.section;
  NSInteger row = indexPath.row;
  NSArray *files = section == RVC_SECTION_DUMMY ? nil : *[self filesForSection:section];
  
  if(section == RVC_SECTION_DUMMY)
  {
    // Dummy
    static NSString *CellIdentifier = @"Dummy";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) 
    {
      cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
      cell.backgroundView = [[UIView alloc]initWithFrame:CGRectMake(0, 0, tableView.bounds.size.width, tableView.rowHeight)];
      cell.opaque = NO;
      cell.backgroundColor = [UIColor clearColor];
      cell.backgroundColor = [UIColor clearColor];
    }

    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    return cell;
  }
  
  static NSString *CellIdentifier = @"CellID";
  
  EditableSimpleTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
  if (cell == nil) 
  {
    cell = [[EditableSimpleTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
#ifdef THEME_USE_CUSTOM_TABLE_VIEW_CELL_SELECTION_BG
    cell.selectionStyle = themeGroupedCellSelectionStyle;
#endif
  }
  
  NSString *path = files[row];
  NSString *extension = path.pathExtension;
  cell.textField.text = [path stringByDeletingPathExtension];
  
  if(tableView != self.tableView)
    return cell;
  
  cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
  
  SafeCtx *ctx = [[SafeCtxStorage sharedStorage]safeCtxForPath:[self filePath: indexPath]];
  
  if(ctx.expired || !ctx.password || !ctx.password.length)
    cell.imageView.image = [UIImage imageNamed:imageName(@"safeClosed")];
  else
    cell.imageView.image = [UIImage imageNamed:imageName(@"safeOpen")];
  
  [cell setTextValidator:^BOOL(NSString *text) {
    NSString *newFileName = [text stringByAppendingPathExtension:extension];
    
    newFileName = [[newFileName precomposedStringWithCanonicalMapping]lowercaseString];
    
    for(__strong NSString *file in *[self filesForSection:section])
    {
      file = [[file precomposedStringWithCanonicalMapping]lowercaseString];
      if([file isEqualToString:newFileName])
        return NO;
    }
    return YES;
  }];
  
  __weak EditableSimpleTableViewCell *wcell = cell;
  [cell setTextChanged:^(NSString *text, NSString *originalText) {
    if([text isEqualToString:originalText])
      return;
    NSString *originalFileName = [originalText stringByAppendingPathExtension:extension];
    NSString *newFileName = [text stringByAppendingPathExtension:extension];
    [self safeRenamedFrom:originalFileName to:newFileName inSection:section];
    wcell.accessibilityLabel = text;
  }];
  
  [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];
  cell.isAccessibilityElement = YES;
  cell.accessibilityTraits = UIAccessibilityTraitButton;
  cell.accessibilityHint = @"Opens safe.";
  cell.accessibilityLabel = cell.textField.text;
  
  return cell;
}

- (void)completeDeletionForIndexPath:(NSIndexPath *)indexPath
{
  BOOL dropBoxJustOne = m_filesInDropboxFolder.count == 1;
  BOOL dropBoxEmpty = m_filesInDropboxFolder.count == 0;
  BOOL iCloudJustOne = m_filesInICloudFolder.count == 1;
  BOOL iCloudEmpty = m_filesInICloudFolder.count == 0;
  BOOL localJustOne = m_filesInDocsFolder.count == 1;
  BOOL localEmpty = m_filesInDocsFolder.count == 0;
  BOOL haveSync = [self.tableView numberOfSections] > RVC_SECTION_LOCAL;
  
  BOOL reloadDB;
  BOOL reloadIC;
  NSMutableArray * __strong *pFiles;
  
  NSUInteger section = indexPath.section;
  NSString *filePath = [self filePath:indexPath];
  
  if(section == RVC_SECTION_DROPBOX)
  {
    reloadDB = dropBoxJustOne && (!iCloudEmpty || !localEmpty);
    reloadIC = iCloudEmpty && localEmpty && dropBoxJustOne;
    pFiles = &m_filesInDropboxFolder;
  }
  else if(section == RVC_SECTION_ICLOUD)
  {
    reloadIC = iCloudJustOne && (!dropBoxEmpty || !localEmpty);
    reloadDB = dropBoxEmpty && localEmpty && iCloudJustOne;
    pFiles = &m_filesInICloudFolder;
  }
  else
  {
    NSAssert(section == RVC_SECTION_LOCAL, @"Don't knwow about other sections");
    // Remove last entry in local and there is none in dropbox
    reloadDB = dropBoxEmpty && iCloudEmpty && localJustOne;
    reloadIC = iCloudEmpty && dropBoxEmpty && localJustOne;
    pFiles = &m_filesInDocsFolder;
  }
  
  reloadDB = reloadDB && haveSync;
  reloadIC = reloadIC && haveSync;
  
  UITableView *tableView = self.tableView;
  
  if(section == RVC_SECTION_LOCAL || (section == RVC_SECTION_DROPBOX && !reloadDB) || (section == RVC_SECTION_ICLOUD && !reloadIC))
  {
    NSUInteger row = indexPath.row;
    [*pFiles removeObjectAtIndex:row];
    
    [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationLeft];
    NSUInteger nRemaining = [*pFiles count];
    if(iOS6() && nRemaining > 0)
    {
      double delayInSeconds = 0.3;
      dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
      dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
        if(row == 0)
          [tableView reloadRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:0 inSection:section]] withRowAnimation:UITableViewRowAnimationNone];
        else if(row == nRemaining)
          [tableView reloadRowsAtIndexPaths:@[[NSIndexPath indexPathForRow:row - 1 inSection:section]] withRowAnimation:UITableViewRowAnimationNone];
      });
    }
  }
  else
  {
    *pFiles = [self safesInDirectory:[self dirOfSection:section]];
    if(reloadDB && isDropboxEnabled())
      [tableView reloadSections:[NSIndexSet indexSetWithIndex:RVC_SECTION_DROPBOX] withRowAnimation:UITableViewRowAnimationFade];
    if(reloadIC)
      [tableView reloadSections:[NSIndexSet indexSetWithIndex:RVC_SECTION_ICLOUD] withRowAnimation:UITableViewRowAnimationFade];
  }
  
  [Crashlytics setIntValue:(int)(*pFiles).count forKey:section == RVC_SECTION_LOCAL ? @"Local safes" : (section == RVC_SECTION_DROPBOX ? @"Dropbox safes" : @"iCloud safes")];
  
  NSTimer *timer = m_closeSafeTimers[filePath];
  [timer invalidate];
  [m_closeSafeTimers removeObjectForKey:filePath];
  [[SafeCtxStorage sharedStorage]deleteCtxForPath:filePath];
}

- (void)commitDeleteForIndexPath:(NSIndexPath *)indexPath
{
  NSUInteger section = indexPath.section;
  NSFileManager *fm = [NSFileManager defaultManager];
  NSString *filePath = [self filePath:indexPath];

  if(section != RVC_SECTION_ICLOUD)
  {
    NSError *error = nil;
    BOOL ok = [fm removeItemAtPath:filePath error:&error];
    if(!ok)
    {
#ifdef DEBUG_MODE
      BOOL isDir, exists = [fm fileExistsAtPath:filePath isDirectory:&isDir];
      DebugLog(@"Error deleting file %@: %@ / %@ / isDir = %d / exists = %d", filePath, [error localizedDescription], [error localizedFailureReason], isDir, exists);
#endif
      return;
    }
    [self completeDeletionForIndexPath:indexPath];
    return;
  }
  
  dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
    __block BOOL ok;
    NSError *error = nil;
    
    NSAssert([NSFileCoordinator self], @"Must have iCloud at this point");
    NSFileCoordinator *coordinator = [[NSFileCoordinator alloc]initWithFilePresenter:nil];
    NSError __block *internalError = nil;
    [coordinator coordinateWritingItemAtURL:[NSURL fileURLWithPath:filePath] options:NSFileCoordinatorWritingForDeleting error:&error byAccessor:^(NSURL *newURL) {
      ok = [fm removeItemAtURL:newURL error:&internalError];
    }];
    if(!ok && !error)
      error = internalError;
    if(!ok)
    {
      BOOL isDir, exists = [fm fileExistsAtPath:filePath isDirectory:&isDir];
      A77Log(@"Error deleting file %@: %@ / %@ / isDir = %d / exists = %d", filePath, [error localizedDescription], [error localizedFailureReason], isDir, exists);
      return;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
      [self completeDeletionForIndexPath:indexPath];
    });
  });
}
 

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath 
{
  NSAssert(indexPath.section < RVC_SECTION_DUMMY, @"No dummies here");
  
  if (editingStyle != UITableViewCellEditingStyleDelete) 
    return;
  
  if([MasterPasswordViewController isEnabled])
  {
    [MasterPasswordViewController verifyInViewController:nil allowCancel:YES callback:^(BOOL ok) {
      if(ok)
        [self commitDeleteForIndexPath:indexPath];
    }];
    return;
  }
  else
  {
    SafeCtx *ctx = [[SafeCtxStorage sharedStorage]safeCtxForPath:[self filePath: indexPath]];
    BOOL opened = !ctx.expired && ctx.password.length > 0;
    if(opened)
    {
      A77Log(@"RootVC delete opened safe");
      [self commitDeleteForIndexPath:indexPath];
      return;
    }

    UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"Safe Password"
                                                   message:@"Password required to delete safe."
                                                  delegate:self
                                         cancelButtonTitle:@"Cancel"
                                         otherButtonTitles:@"OK", nil];
    alert.tag = ALERTVIEW_TAG_EXECUTE_BLOCK;
    alert.alertViewStyle = UIAlertViewStyleSecureTextInput;
    UITextField *tf = [alert textFieldAtIndex:0];
    objc_setAssociatedObject(tf, kAlertTextViewAlertKey, alert, OBJC_ASSOCIATION_ASSIGN);
    tf.delegate = self;
    tf.returnKeyType = UIReturnKeyDone;
    
    objc_setAssociatedObject(alert, kAlertBlockKey, ^(UIAlertView *alertView, NSInteger btnClicked) {
      if(btnClicked == alert.cancelButtonIndex)
        return;
      
      NSString *password = [[alertView textFieldAtIndex:0]text];
      
      [SafeFileClass verifyPassword:password ofFilePath:ctx.safePath withCallback:^(BOOL passwordOk, NSError *error) {
        if(!passwordOk && [error.domain isEqualToString:kPasswordSafeErrorDomain] && [error.userInfo[kPasswordSafeStatusCodeKey] isEqualToNumber:kWrongPasswordStatusCode]  )
        {
          UIView *view = self.view;
          MBProgressHUD *hud = [[MBProgressHUD alloc]initWithView:view];
          hud.customView = [[UIImageView alloc]initWithImage:[UIImage imageNamed:@"Bad.png"]];
          hud.mode = MBProgressHUDModeCustomView;
          hud.labelText = [error localizedDescription];
          hud.animationType = MBProgressHUDAnimationZoom;
          hud.opacity = .8;
          hud.removeFromSuperViewOnHide = YES;
          [view addSubview:hud];
          [hud show:YES];
          [self performSelector:@selector(hideHud:) withObject:hud afterDelay:2];
          
          A77Log(@"RootVC delete bad password");
          return;
        }
        A77Log(@"RootVC delete confirmed with password");
        [self commitDeleteForIndexPath:indexPath];
      }];
      
    }, OBJC_ASSOCIATION_COPY_NONATOMIC);
    
    [alert show];

  }
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
  return section == 0 ? @"Local safes" : nil;
}

// May be called by subclasses in another thread
- (void)safeRenamedFrom:(NSString *)fileName to:(NSString *)newFileName inSection:(NSInteger)section
{
  // Always reload to keep sorted
  NSFileManager *fm = [NSFileManager defaultManager];
  NSString *dir = [self dirOfSection:section];

  NSString *oldPath = [dir stringByAppendingPathComponent:fileName];
  NSString *newPath = [dir stringByAppendingPathComponent:newFileName];
  
  BOOL ok = [fm moveItemAtPath:oldPath toPath:newPath error:nil];
  
  dispatch_async(dispatch_get_main_queue(), ^{
    
    if(ok)
    {
      SafeCtxStorage *ctxStorage = [SafeCtxStorage sharedStorage];
      SafeCtx *oldCtx = [ctxStorage safeCtxForPath:oldPath];
      SafeCtx *newCtx = [ctxStorage safeCtxForPath:newPath];
      
      newCtx.safeUUID = oldCtx.safeUUID;
      if(oldCtx.password && oldCtx.password.length && !oldCtx.expired)
        newCtx.password = oldCtx.password;
      
      [ctxStorage deleteCtxForPath:oldPath];
    }
    
    NSMutableArray *__strong *pFiles = [self filesForSection:section];
    *pFiles = nil;
    [self.tableView reloadSections:[NSIndexSet indexSetWithIndex:section] withRowAnimation:UITableViewRowAnimationNone];
  });
}

#pragma mark -
#pragma mark Table view delegate

- (void)showError:(NSString *)errorMsg inView:(UIView *)view
{
  if(!view)
    view = self.navigationController.view;
  
  MBProgressHUD *hud = [[MBProgressHUD alloc]initWithView:view];
  hud.customView = [[UIImageView alloc]initWithImage:[UIImage imageNamed:@"Bad.png"]];
  hud.mode = MBProgressHUDModeCustomView;
  hud.labelText = errorMsg;
  hud.animationType = MBProgressHUDAnimationZoom;
  hud.yOffset = -50;
  hud.opacity = .8;
  hud.removeFromSuperViewOnHide = YES;
  [self.modalViewController.view addSubview:hud];
  [hud show:YES];
  [self performSelector:@selector(hideHud:) withObject:hud afterDelay:1];
}

- (void)onOpenSafeComplete:(id)openSafeViewController
{
  OpenSafeViewController *openSafeVC = (OpenSafeViewController *)openSafeViewController;
  
  SafeCtx *safeCtx = openSafeVC.safeCtx;
  if(!safeCtx)
  {
    NSString *safePath = openSafeVC.safePath;
    NSUInteger row, section;
    NSArray *files;
    
    DebugLog(@"Comparing %@ vs %@", [safePath stringByDeletingLastPathComponent], dropboxDir());
    
    if([[safePath stringByDeletingLastPathComponent] isEqualToString:dropboxDir()])
    {
      section = RVC_SECTION_DROPBOX;
      files = m_filesInDropboxFolder;
    }
    else if(m_iCloudURL && [[safePath stringByDeletingLastPathComponent] isEqualToString:m_iCloudURL.path])
    {
      section = RVC_SECTION_ICLOUD;
      files = m_filesInICloudFolder;
    }
    else
    {
      section = RVC_SECTION_LOCAL;
      files = m_filesInDocsFolder;
    }
    
    NSString *keyFile = [safePath lastPathComponent];

    for(row = 0; row < files.count; row++)
      if([keyFile isEqualToString:files[row]])
        break;
    NSAssert(row < files.count, @"Unknown file");
    
    [self.tableView deselectRowAtIndexPath:[NSIndexPath indexPathForRow:row inSection:section] animated:YES];

    [self dismissModalViewControllerAnimated:YES];
    
    return;
  }
  
  [self openSafeForCtx:safeCtx state:m_state callback:^(SafeFile *safeFile, NSError *error) 
  {
    if(!safeFile)
    {
      A77Log(@"Error opening safe: %ld / %@", (long)error.code, error.domain);
      UIView *view = openSafeVC.view;
      [self showError:error.localizedDescription inView:view];
      [openSafeVC clearPassword];
      return;
    }
    
    A77Log(@"Safe opened");
    
    FileViewController *fileViewController = [[FileViewController alloc] initWithSafeCtx:safeCtx safeFile:safeFile state:m_state];
    
    fileViewController.initialGroup = self.initialGroup;
    fileViewController.initialSearchTerm = self.initialSearchTerm;

    NSAssert(!self.currentSafeCtx, @"No safe should be opened");
    self.currentSafeCtx = safeCtx;
    self.currentOpenedFile = safeFile;
    
    [self.navigationController pushViewController:fileViewController animated:YES];
    
    [self dismissModalViewControllerAnimated:YES];
  }];
}

- (void)openSafeWithUUID:(NSString *)safeUUID password:(NSString *)password animated:(BOOL)animated
{
  SafeCtx *safeCtx = [[SafeCtxStorage sharedStorage]safeCtxForUUID:safeUUID];
  if(!safeCtx)
  {
    A77Log(@"Safe with UUID not found");
    dispatch_async(dispatch_get_main_queue(), ^{
#ifdef PWSAFE_VAULT
      [self showError:@"Vault not found" inView:nil];
#else
      [self showError:@"Safe not found" inView:nil];
#endif
    });
    return;
  }
  
  safeCtx.password = password;
  safeCtx.opened = YES;
  [safeCtx sleep];
  safeCtx.opened = NO;
  
  UINavigationController *nav = (UINavigationController *)self.modalViewController;
  OpenSafeViewController *openSafeVC;
  
  if(nav && [nav isKindOfClass:[UINavigationController class]] &&
     (openSafeVC = (OpenSafeViewController *)nav.visibleViewController) &&
     [openSafeVC isKindOfClass:[OpenSafeViewController class]])
  {
    NSAssert([openSafeVC.safeCtx.safeUUID isEqualToString:safeUUID], @"Must be opening the correct safe");
    [self dismissModalViewControllerAnimated:animated];
  }
  
  [self openSafeForCtx:safeCtx state:m_state callback:^(SafeFile *safeFile, NSError *error) {
    if(!safeFile)
    {
      A77Log(@"Error opening safe: %ld / %@", (long)error.code, error.domain);
      [self showError:error.localizedDescription inView:nil];
      return;
    }
    A77Log(@"Safe opened");

    
    UINavigationController *navController = self.navigationController;
    FileViewController *fileViewController = [[FileViewController alloc] initWithSafeCtx:safeCtx safeFile:safeFile state:m_state];
    
    if(navController.visibleViewController == self)
    {
      NSAssert(!self.currentSafeCtx, @"No safe should be opened");
      self.currentSafeCtx = safeCtx;
      self.currentOpenedFile = safeFile;
      
      [navController pushViewController:fileViewController animated:YES];
    }
    else if(![self.currentSafeCtx.safeUUID isEqualToString:safeUUID])
    {
      navController.delegate = self;
      __weak RootViewController* weakSelf = self;
      m_onNavigationControllerShowedMe = [ ^{
        RootViewController *rvc = weakSelf;
        if(!rvc)
          return;
        NSCAssert(!rvc.currentSafeCtx, @"No safe should be opened");
        weakSelf.currentSafeCtx = safeCtx;
        weakSelf.currentOpenedFile = safeFile;
        [navController pushViewController:fileViewController animated:YES];
        navController.delegate = nil;
        rvc->m_onNavigationControllerShowedMe = nil;
      } copy];
      
      UIViewController *currentVC = navController.viewControllers.lastObject;
      if(currentVC != self)
      {
        [currentVC dismissModalViewControllerAnimated:animated];
        [navController popToRootViewControllerAnimated:animated];
      }
      if(!animated)
        [navController pushViewController:fileViewController animated:YES];
    }
  }];
}

- (void)openSafeWithName:(NSString *)safeName groupName:(NSString *)groupName searching:(NSString *)searchStr animated:(BOOL)animated
{
  NSString *safePath = nil;
  NSFileManager *fm = [NSFileManager defaultManager];
  BOOL isDir;
  
  NSString *safeName2 = [safeName stringByAppendingPathExtension:@"dat"];
  safeName = [safeName stringByAppendingPathExtension:@"psafe3"];
  
  if(m_iCloudURL)
  {
    safePath = [m_iCloudURL.path stringByAppendingPathComponent:safeName];
    if(![fm fileExistsAtPath:safePath isDirectory:&isDir] || isDir)
    {
      safePath = [m_iCloudURL.path stringByAppendingPathComponent:safeName2];
      if(![fm fileExistsAtPath:safePath isDirectory:&isDir] || isDir)
        safePath = nil;
    }
  }
  
  if(!safePath && isDropboxReady())
  {
    safePath = [dropboxDir() stringByAppendingPathComponent:safeName];
    if(![fm fileExistsAtPath:safePath isDirectory:&isDir] || isDir)
    {
      safePath = [dropboxDir() stringByAppendingPathComponent:safeName2];
      if(![fm fileExistsAtPath:safePath isDirectory:&isDir] || isDir)
        safePath = nil;
    }
  }
  
  if(!safePath)
  {
    safePath = [localSafeDir() stringByAppendingPathComponent:safeName];
    if(![fm fileExistsAtPath:safePath isDirectory:&isDir] || isDir)
    {
      safePath = [localSafeDir() stringByAppendingPathComponent:safeName2];
      if(![fm fileExistsAtPath:safePath isDirectory:&isDir] || isDir)
        safePath = nil;
    }
  }
  
  if(safePath)
  {
    UINavigationController *navController = self.navigationController;
    
    if(navController.visibleViewController != self)
    {
      if(animated)
      {
        navController.delegate = self;
        __weak RootViewController* weakSelf = self;
        
        m_onNavigationControllerShowedMe =  [^{
          RootViewController *rvc = weakSelf;
          if(!rvc)
            return;
          [rvc openSafeAndAskForPasswordIfNeeded:safePath groupName:groupName searchTerm:searchStr state:rvc->m_state];
          navController.delegate = nil;
          rvc->m_onNavigationControllerShowedMe = nil;
        } copy];
      }
      
      UIViewController *currentVC = navController.viewControllers.lastObject;
      if(currentVC != self)
      {
        [currentVC dismissModalViewControllerAnimated:animated];
        [navController popToRootViewControllerAnimated:animated];
      }
      if(!animated)
        [self openSafeAndAskForPasswordIfNeeded:safePath groupName:groupName searchTerm:searchStr state:m_state];
    }
    else
    {
      [self openSafeAndAskForPasswordIfNeeded:safePath groupName:groupName searchTerm:searchStr state:m_state];
    }
  }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  if(indexPath.section >= RVC_SECTION_DUMMY)
    return;
  
  NSString *filePath = [self filePath:indexPath];
  [self openSafeAndAskForPasswordIfNeeded:filePath state:m_state];
}

- (NSMutableArray *__strong*) filesForSection:(NSUInteger)section
{
  if(section == RVC_SECTION_LOCAL)
    return &m_filesInDocsFolder;
  if(section == RVC_SECTION_DROPBOX)
    return &m_filesInDropboxFolder;
  if(section == RVC_SECTION_ICLOUD)
    return &m_filesInICloudFolder;
  return nil;
}

- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath
{
  NSUInteger section = indexPath.section;
  if(section >= RVC_SECTION_DUMMY)
    return UITableViewCellEditingStyleNone;
  return UITableViewCellEditingStyleDelete;
}

#pragma mark - UIAlertViewDelegate

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex
{
  if(alertView.tag == ALERTVIEW_TAG_EXECUTE_BLOCK)
  {
    void (^block)(UIAlertView *alertView, NSInteger btnIndex) = objc_getAssociatedObject(alertView, kAlertBlockKey);
    NSAssert(block, @"Must have a block");
    if(block)
      block(alertView, buttonIndex);
    return;
  }
}

- (BOOL)alertViewShouldEnableFirstOtherButton:(UIAlertView *)alertView
{
  NSString *password = [alertView textFieldAtIndex:0].text;
  return password.length != 0;
}

#pragma mark - UITextFieldDelegate

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
  UIAlertView *alert = objc_getAssociatedObject(textField, kAlertTextViewAlertKey);
  NSAssert([alert isKindOfClass:[UIAlertView class]], @"Must have found it");
  if(![alert isKindOfClass:[UIAlertView class]])
    return NO;
  if(![self alertViewShouldEnableFirstOtherButton:alert])
    return NO;
  [alert dismissWithClickedButtonIndex:alert.firstOtherButtonIndex animated:YES];
  return YES;
}

#pragma mark - Navigation Controller Delegate

- (void)navigationController:(UINavigationController *)navigationController didShowViewController:(UIViewController *)viewController animated:(BOOL)animated
{
  if(viewController == self && m_onNavigationControllerShowedMe)
    m_onNavigationControllerShowedMe();
}

#pragma mark - Memory management

- (void)didReceiveMemoryWarning 
{
  // Releases the view if it doesn't have a superview.
  [super didReceiveMemoryWarning];
  
  // Relinquish ownership any cached data, images, etc that aren't in use.
}


- (void)dealloc 
{
  [[NSNotificationCenter defaultCenter]removeObserver:self];
  if(self.isViewLoaded)
  {
    self.tableView.delegate = nil;
    self.tableView.dataSource = nil;
  }
}



@end

