//
//  SafeCtx.m
//  iPasswordSafe
//
//  Created by Jorge Peixoto Vasquez on 3/10/11.
//  Copyright 2011 app77. All rights reserved.
//

#import "SafeCtx.h"
#ifndef PWSAFE_COZI
#import "MasterPasswordViewController.h"
#endif

@implementation SafeCtx
{
  NSString *m_safePath;
  NSDate *m_sleepTimeStamp;
  NSString *m_password;
  NSData *m_encryptedPassword;
  NSString *m_uuid;
}

@synthesize keepAlivePeriod = m_keepAlivePeriod;
@synthesize opened = m_opened;
@synthesize safeUUID = m_uuid;

#ifdef PWSAFE_WITH_DROPBOX

@synthesize dropboxPath = m_dropboxPath;
@synthesize dropboxModified = m_dropboxModified;
@synthesize dropboxRevision = m_dropboxRevision;
@synthesize dropboxRev = m_dropboxRev;
@synthesize dropboxSyncTimestamp = m_dropboxSyncTimeStamp;

#endif

#ifdef PWSAFE_COZI

@synthesize coziSafeId = m_coziSafeId;
@synthesize coziModified = m_coziModified;
@synthesize coziFileName = m_coziFileName;
@synthesize coziDisplayName = m_coziDisplayName;
@synthesize coziDeleted = m_coziDeleted;
@synthesize coziRenamed = m_coziRenamed;

#endif

#ifdef DEBUG_MODE
- (void)dealloc
{
  NSLog(@"Deallocing CTX");
}
#endif

- (void)sleep
{
  if(!m_opened)
    return;
  m_sleepTimeStamp = [NSDate date];
}

- (void)expireNow
{
  m_sleepTimeStamp = nil;
  m_password = nil;
}

- (NSDate *)sleepTimeStamp
{
  return m_sleepTimeStamp;
}

- (BOOL)isCloud
{
#if TARGET_IPHONE_SIMULATOR
  return NO;
#endif
  NSFileManager *fm = [NSFileManager defaultManager];
  return [fm isUbiquitousItemAtURL:[NSURL fileURLWithPath:m_safePath]];
}

- (NSString *)password
{
  if(!m_password)
    return nil;
  return [m_password copy];
}

- (void)setPassword:(NSString *)password
{
  m_encryptedPassword = nil;
  
  m_password = password ? [password copy] : nil;
}

- (NSString *)safePath
{
  return m_safePath;
}

- (void)updateSafePath:(NSString *)newSafePath
{
  [self willChangeValueForKey:@"safeName"];
  m_safePath = [newSafePath copy];
  [self didChangeValueForKey:@"safeName"];
}

- (BOOL)expired
{
  if(!m_keepAlivePeriod)
  {
    m_password = nil;
    return TRUE;
  }
  if(!m_sleepTimeStamp)
  {
    m_password = nil;
    return TRUE;
  }
  NSTimeInterval timeSinceNow = -[m_sleepTimeStamp timeIntervalSinceNow];
  uint max = m_keepAlivePeriod * 60;
  if(timeSinceNow > max)
  {
    m_password = nil;
    return TRUE;
  }
  return FALSE;
}

- (NSString *)keepAlivePeriodText
{
  NSString *text;
  if(m_keepAlivePeriod == 0)
    text = @"Never";
  else if(m_keepAlivePeriod == 1)
    text = @"1 minute";
  else if(m_keepAlivePeriod < 60)
    text = [NSString stringWithFormat:@"%d minutes", m_keepAlivePeriod];
  else if(m_keepAlivePeriod == 60)
    text = @"1 hour";
  else
    text = [NSString stringWithFormat:@"%d hours", m_keepAlivePeriod / 60];
  return text;
}


- (void)createUUID
{
  CFUUIDRef uuid = CFUUIDCreate(NULL);
  CFStringRef uuidStr = CFUUIDCreateString(kCFAllocatorDefault, uuid);
  CFRelease(uuid);
  m_uuid = (__bridge_transfer NSString *)uuidStr;
}

-(id) initWithPath:(NSString *)path {
  self = [super init];
  if(!self)
    return nil;
  m_safePath = path;
  
  [self createUUID];
  
  m_keepAlivePeriod = 5;
  
  return self;
}

- (NSString *)safeName
{
#ifdef PWSAFE_COZI
  return m_coziDisplayName;
#else
  return [[m_safePath lastPathComponent] stringByDeletingPathExtension];
#endif
}

- (NSString *)safeTempPath
{
  return [m_safePath stringByAppendingPathExtension:@"tmp"];
}


-(id)initWithCoder:(NSCoder *)decoder
{
  self = [super init];
  if(!self)
    return nil;

  self.keepAlivePeriod = [decoder decodeInt32ForKey:@"keepAlivePeriod"];
  m_safePath = [decoder decodeObjectForKey:@"safePath"];

#ifdef PWSAFE_WITH_DROPBOX
  m_dropboxPath = [decoder decodeObjectForKey:@"dropboxPath"];
  m_dropboxModified = [decoder decodeBoolForKey:@"dropboxModified"];
  m_dropboxSyncTimeStamp = [decoder decodeObjectForKey:@"dropboxSyncTimeStamp"];
  m_dropboxRev = [decoder decodeObjectForKey:@"dropboxRev"];
  if(!m_dropboxRev)
    m_dropboxRevision = [decoder decodeInt64ForKey:@"dropboxRevision"];
#endif
  
#ifdef PWSAFE_COZI
  m_coziSafeId = [decoder decodeObjectForKey:@"coziSafeId"];
  m_coziFileName = [decoder decodeObjectForKey:@"coziFileName"];
  m_coziModified = [decoder decodeBoolForKey:@"coziModified"];
  m_coziDisplayName = [decoder decodeObjectForKey:@"coziDisplayName"];
  m_coziDeleted = [decoder decodeBoolForKey:@"coziDeleted"];
  m_coziRenamed = [decoder decodeBoolForKey:@"coziRenamed"];
#endif
  
  m_encryptedPassword = [decoder decodeObjectForKey:@"encryptedPassword"];
  m_sleepTimeStamp = [decoder decodeObjectForKey:@"sleepTimestamp"];
  m_uuid = [decoder decodeObjectForKey:@"uuid"];
  if(!m_uuid)
    [self createUUID];
  
  // DebugLog(@"Decoded a safeCtx %@ with timestamp = %@ and keep_alive_period = %d minutes and encryptedPasswordLen = %d", self.safeName, m_sleepTimeStamp, self.keepAlivePeriod, m_encryptedPassword.length);
  
  return self;
}

-(void)encodeWithCoder:(NSCoder *)coder
{
  // DebugLog(@"Saving ctx with dropboxPath = %@ and safePath = %@", m_dropboxPath, m_safePath);
  
  [coder encodeInt32:self.keepAlivePeriod forKey:@"keepAlivePeriod"];
  [coder encodeObject:m_safePath forKey:@"safePath"];
  
#ifdef PWSAFE_WITH_DROPBOX
  [coder encodeObject:m_dropboxPath forKey:@"dropboxPath"];
  [coder encodeBool:m_dropboxModified forKey:@"dropboxModified"];
  [coder encodeObject:m_dropboxSyncTimeStamp forKey:@"dropboxSyncTimeStamp"];
  
  if(!m_dropboxRev && m_dropboxRevision) 
    [coder encodeInt64:m_dropboxRevision forKey:@"dropboxRevision"];
  
  [coder encodeObject:m_dropboxRev forKey:@"dropboxRev"];
#endif
  
#ifdef PWSAFE_COZI
  [coder encodeObject:m_coziSafeId forKey:@"coziSafeId"];
  [coder encodeObject:m_coziFileName forKey:@"coziFileName"];
  [coder encodeBool:m_coziModified forKey:@"coziModified"];
  [coder encodeObject:m_coziDisplayName forKey:@"coziDisplayName"];
  [coder encodeBool:m_coziDeleted forKey:@"coziDeleted"];
  [coder encodeBool:m_coziRenamed forKey:@"coziRenamed"];
#endif
  
  [coder encodeObject:m_sleepTimeStamp forKey:@"sleepTimestamp"];
  
#ifndef PWSAFE_COZI
  if(m_password && m_password.length && self.keepAlivePeriod > 0 && [MasterPasswordViewController isEnabled])
  {
    NSData *encryptedPw = [MasterPasswordViewController encryptString:m_password]; 
    [coder encodeObject:encryptedPw forKey:@"encryptedPassword"];
  }
#endif
  
  [coder encodeObject:m_uuid forKey:@"uuid"];
}

#ifndef PWSAFE_COZI
- (void)decryptPassword
{
  if(!m_encryptedPassword || self.expired)
    return;
  self.password = [MasterPasswordViewController decryptString:m_encryptedPassword];
}
#endif

#ifdef PWSAFE_WITH_DROPBOX

-(BOOL)dropboxLastSyncExpired
{
  NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
  NSNumber *maxMinutesObj = [prefs objectForKey:@"autoSyncIntervalMinutes"];
  NSInteger maxMinutes = maxMinutesObj ? [maxMinutesObj integerValue] : 30;
  if(maxMinutes < 0) // Auto-sync disabled
    return NO;
  if(!m_dropboxSyncTimeStamp || m_dropboxModified)
    return YES;
  NSTimeInterval elapsed = -[m_dropboxSyncTimeStamp timeIntervalSinceNow];
  return maxMinutes * 60 < elapsed;
}

- (void)syncedWithDropboxRev:(NSString *)rev
{
  m_dropboxSyncTimeStamp = [NSDate date];
  m_dropboxRev = [rev copy];
  m_dropboxModified = NO;  
}

#endif

#ifdef PWSAFE_COZI

- (void)syncedWithCoziFileName:(NSString *)coziFileName
{
  m_coziFileName = [coziFileName copy];
  m_coziModified = NO;
  m_coziDeleted = NO;
  m_coziRenamed = NO;
}

- (void)setCoziDisplayName:(NSString *)coziDisplayName
{
  [self willChangeValueForKey:@"safeName"];
  m_coziDisplayName = coziDisplayName;
  [self didChangeValueForKey:@"safeName"];
}

#endif

@end
