//
//  SafePwdInputAccessoryView.m
//  pwSafe
//
//  Created by Jorge P. Vasquez on 16/01/13.
//
//

#import "SafePwdInputAccessoryView.h"
#import "AppTheme.h"

@implementation SafePwdInputAccessoryView
{
  NSArray *_stdItems;
}

- (id)initWithFrame:(CGRect)frame
{
  self = [super initWithFrame:frame];
  if (!self)
    return nil;
  
  self.barStyle = UIBarStyleBlackTranslucent;
  self.translucent = YES;
  
  _stdItems = @[
    [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:NULL],
    [[UIBarButtonItem alloc]initWithTitle:@"Keyboard type" style:UIBarButtonItemStyleBordered target:self action:@selector(toggleKeyboard:)]
  ];
  self.items = _stdItems;
  
  if([self respondsToSelector:@selector(setTintAdjustmentMode:)])
  {
    self.tintColor = kThemeLightTintColor;
    self.barStyle = kThemeBarStyle;
  }

  return self;
}

- (void)dealloc
{
  self.textField = nil;
}

static NSString * const kRememberKeyboardKey = @"rememberKeyboardType";
static NSString * const kLastKeyboardKey = @"lastKeyboardType";

- (void)toggleKeyboard:(id)sender
{
  UIKeyboardType curType = _textField.keyboardType;
  NSUInteger idx = [_allowedKeyboardTypes indexOfObject:@(curType)];
  NSAssert2(idx != NSNotFound, @"Current keyboard type %ld not allowed (%@)", (long)curType, _allowedKeyboardTypes);
  idx++;
  if(idx == _allowedKeyboardTypes.count)
    idx = 0;
  UIKeyboardType newType = (UIKeyboardType)[_allowedKeyboardTypes[idx] integerValue];
  _textField.keyboardType = newType;
  [_textField reloadInputViews];
  
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
  if([defaults boolForKey:kRememberKeyboardKey])
    [defaults setInteger:newType forKey:kLastKeyboardKey];
}

- (void)setAdditionalButtons:(NSArray *)additionalButtons
{
  _additionalButtons = additionalButtons;
  self.items = additionalButtons ? [additionalButtons arrayByAddingObjectsFromArray:_stdItems] : _stdItems;
}

- (void)setTextField:(UITextField *)textField
{
  _textField = textField;
  if(!textField)
    return;
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
  if([defaults boolForKey:kRememberKeyboardKey])
  {
    NSNumber *lastKeyboard = [defaults objectForKey:kLastKeyboardKey];
    if(lastKeyboard)
    {
      UIKeyboardType lastType = lastKeyboard.intValue;
      if(!_allowedKeyboardTypes || [_allowedKeyboardTypes indexOfObject:lastKeyboard] != NSNotFound)
        textField.keyboardType = lastType;
    }
  }
}

@end
