//
//  SelectGroupViewController.m
//  iPasswordSafe
//
//  Created by Jorge Peixoto Vasquez on 5/30/11.
//  Copyright 2011 app77. All rights reserved.
//

#import "SelectGroupViewController.h"
#import "SafeFile.h"
#import "CreateGroupViewController.h"
#import "AppTheme.h"

@implementation SelectGroupViewController
{
  NSString *m_selectedGroup;
  NSArray *m_groups;
  SafeFile *m_file;
}

@synthesize selectedGroup = m_selectedGroup;

- (id)initWithSafeFile:(SafeFile *)file currentGroup:(NSString *)group
{
  self = [super initWithStyle:UITableViewStyleGrouped];
  if (!self) 
    return nil;

  m_groups = file.sortedGroups;
  m_selectedGroup = group;
  m_file = file;
  self.navigationItem.title = @"Groups";
  
  return self;
}

- (void)dealloc
{
  if(self.isViewLoaded)
  {
    self.tableView.delegate = nil;
    self.tableView.dataSource = nil;
  }
}

#pragma mark - View lifecycle

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  if(self.navigationController.modalPresentationStyle == UIModalPresentationFormSheet)
    [self.tableView themeViewDidLoadInIpadPos:TVP_FORMSHEET];  
  else
    [self.tableView themeViewDidLoadInIpadPos:TVP_RIGHT];
  
  if(_allowCreatingNewGroup)
  {
    UIBarButtonItem *spacer = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
    UIBarButtonItem *addBtn = [UIBarButtonItem buttonWithImageNamed:@"Plus" target:self action:@selector(onAddNewItem:)];
    addBtn.accessibilityLabel = @"Add";
    self.toolbarItems = @[spacer, addBtn];
  }
}

#pragma mark - Add new group

-(void)onAddComplete:(id)sender
{
  CreateGroupViewController *newGroupVC = sender;
  if(newGroupVC.result)
  {
    NSString *newGroupName = newGroupVC.groupName;
    m_selectedGroup = newGroupName;
    [m_file addGroup:newGroupName];
    // No need to reload, [safeWasInternally modified will be called if needed]
  }
  [self dismissModalViewControllerAnimated:YES];
}

- (void)safeWasInternallyModified
{
  m_groups = m_file.sortedGroups;
  [self.tableView reloadData];
}

- (void)groupRenamedFrom:(NSString *)oldGroupName to:(NSString *)newGroupName
{
  if([m_selectedGroup isEqualToString:oldGroupName])
    m_selectedGroup = [newGroupName copy];
}

- (void)onAddNewItem:(id)sender
{
  CreateGroupViewController *newGroupVC = [[CreateGroupViewController alloc] initWithTarget:self selector:@selector(onAddComplete:) file:m_file];
  
  UINavigationController *nav = [[UINavigationController alloc]initWithRootViewController:newGroupVC];
  nav.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
  if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    nav.modalPresentationStyle = UIModalPresentationFormSheet;
  
  [self presentModalViewController:nav animated:YES];
}

- (void)viewWillAppear:(BOOL)animated
{
  [super viewWillAppear:animated];
  [self.navigationController setToolbarHidden:!self.toolbarItems || !self.toolbarItems.count animated:animated];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
  return m_groups.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  static NSString *CellIdentifier = @"Cell";
    
  UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
  if (cell == nil) 
    cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
  
  NSUInteger row = indexPath.row;
  NSString *group = m_groups[row];
  NSString *formattedGroup;
  if(!group.length)
  {
    formattedGroup = @"Ungrouped";
  }
  else
  {
    NSArray *groups = [group componentsSeparatedByString:@"."];
    static const unichar arrowChar = 0x21B3;
    NSString *arrow = [NSString stringWithCharacters:&arrowChar length:1];
    NSString *blank = @"  ";
    
    if(groups.count > 1)
    {
      NSMutableString *prefix = [NSMutableString stringWithString:@""];
      NSUInteger nBlanks = groups.count - 2;
      for(NSUInteger i = 0; i < nBlanks; i++)
        [prefix appendString:blank];
    
      formattedGroup = [NSString stringWithFormat:@"%@%@ %@", prefix, arrow, groups[groups.count - 1]];
    }
    else
      formattedGroup = group;
  }

  cell.textLabel.text = formattedGroup;
  if([group isEqualToString:m_selectedGroup])
  {
    cell.accessoryType = UITableViewCellAccessoryCheckmark;
    cell.accessibilityTraits = UIAccessibilityTraitSelected;
  }
  else
  {
    cell.accessoryType = UITableViewCellAccessoryNone;
    cell.accessibilityTraits = UIAccessibilityTraitNone;
  }
  
  [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];
  
  return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
  NSUInteger row = indexPath.row;
  m_selectedGroup = m_groups[row];
  [self.navigationController popViewControllerAnimated:YES];
}

@end
