//
//  SimpleDetailTableViewCell.m
//  iPasswordSafe
//
//  Created by Jorge Peixoto Vasquez on 4/6/11.
//  Copyright 2011 app77. All rights reserved.
//

#import "SimpleDetailTableViewCell.h"
#import "AppTheme.h"

@implementation SimpleDetailTableViewCell
{
  __weak id m_target;
  SEL m_onTextChanged;
  SEL m_onEnterPressed;
  BOOL m_autoCorrect;
}

@synthesize detailTextField, editButton, hideDetailWhenNotEditing, autoCorrect = m_autoCorrect;

-(id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
  self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
  if (self) 
  {
    detailTextField = [[UITextField alloc]initWithFrame:CGRectZero];
    if(!iOS6())
      detailTextField.font = [UIFont systemFontOfSize:14.f];
    else
      detailTextField.font = [UIFont boldSystemFontOfSize:15.f];
    
    [self.contentView addSubview:detailTextField];
    detailTextField.delegate = self;
    [detailTextField addTarget:self action:@selector(onTextChanged:) forControlEvents:UIControlEventEditingChanged];
    m_autoCorrect = YES;
    detailTextField.clearButtonMode = UITextFieldViewModeWhileEditing;
  }
  
  return self;
}

- (void)setEditButton:(UIButton *)newEditButton
{
  if(editButton)
  {
    [editButton removeFromSuperview];
  }
  editButton = newEditButton;
  if(editButton)
  {
    [self.contentView addSubview:editButton];
  }
}

- (void)setTarget: (id)target textChanged:(SEL)onTextChanged enterPressed:(SEL)onEnterPressed
{
  m_onTextChanged = onTextChanged;
  m_onEnterPressed = onEnterPressed;
  m_target = target;
}

- (void)layoutSubviews
{
  [super layoutSubviews];

  CGRect bounds = self.contentView.bounds;
  
  if(editButton)
  {
    editButton.frame = CGRectMake(bounds.size.width - 61, 6, 55, bounds.size.height - 12);
    editButton.hidden = !self.editing;
  }

  CGRect textLabelFrame = self.textLabel.frame;  
  CGRect frame;
  frame.origin.x = textLabelFrame.origin.x + textLabelFrame.size.width + 6;
  frame.size.width = bounds.size.width - 12 - frame.origin.x;
  if(iOS6())
  {
    frame.origin.y = 11;
    frame.size.height = 19;
  }
  else
  {
    frame.size.height = textLabelFrame.size.height;
    frame.origin.y = textLabelFrame.origin.y;
  }
  self.detailTextField.userInteractionEnabled = self.editing;
  self.selectionStyle = self.editing ? UITableViewCellSelectionStyleNone : UITableViewCellSelectionStyleBlue;
  if(hideDetailWhenNotEditing)
  {
    self.detailTextField.secureTextEntry = !self.editing;
  }
  else
  {
    self.detailTextField.secureTextEntry = NO;
  }
  if(self.editing && editButton)
    frame.size.width -= editButton.frame.size.width; // Leave space for a button
  self.detailTextField.frame = frame;
  self.detailTextField.autocorrectionType = m_autoCorrect ? UITextAutocorrectionTypeDefault : UITextAutocorrectionTypeNo;
}


- (void)onTextChanged:(UITextField *)textField
{
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
  [m_target performSelector:m_onTextChanged withObject:textField.text];
#pragma clang diagnostic pop
}

- (void)textFieldDidBeginEditing:(UITextField *)textField
{
  if(!iOS6())
    textField.textColor = kThemeTintColor;
}

- (void)textFieldDidEndEditing:(UITextField *)textField
{
  if(!iOS6())
    textField.textColor = [UIColor blackColor];
}

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
  return self.editing;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
  [m_target performSelector:m_onEnterPressed];
#pragma clang diagnostic pop
  return YES;
}

- (UIColor *)highlightColor
{
  return self.isEditing ? [UIColor blackColor] : [UIColor whiteColor];
}

- (void)setHighlighted:(BOOL)highlighted animated:(BOOL)animated
{
  UIColor *normalColor = !iOS6() && detailTextField.isFirstResponder ? kThemeTintColor : [UIColor blackColor];
  
  [super setHighlighted:highlighted animated:animated];
  if(highlighted)
    detailTextField.textColor = self.highlightColor;
  else if(!self.isSelected)
  {
    if(!animated)
      detailTextField.textColor = normalColor;
    else
    {
      dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, .2 * NSEC_PER_SEC);
      dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
        UIColor *normalColor = !iOS6() && detailTextField.isFirstResponder ? kThemeTintColor : [UIColor blackColor];
        detailTextField.textColor = normalColor;
      });
    }
  }
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated
{
  UIColor *normalColor = !iOS6() && detailTextField.isFirstResponder ? kThemeTintColor : [UIColor blackColor];
  
  [super setSelected:selected animated:animated];
  
  if(selected)
    detailTextField.textColor = self.highlightColor;
  else if(!self.isHighlighted)
  {
    if(!animated)
      detailTextField.textColor = normalColor;
    else
    {
      dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, .2 * NSEC_PER_SEC);
      dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
        UIColor *normalColor = !iOS6() && detailTextField.isFirstResponder ? kThemeTintColor : [UIColor blackColor];
        detailTextField.textColor = normalColor;
      });
    }
  }
}



@end
