//
//  UIActionSheet+BlockActionSheet.m
//  pwSafe
//
//  Created by Jorge P Vasquez on 12/12/13.
//
//

#import "UIActionSheet+BlockActionSheet.h"

@interface BlockActionSheetDelegate : NSObject <UIActionSheetDelegate>

+ (BlockActionSheetDelegate *)sharedDelegate;

- (void)actionSheet:(UIActionSheet *)actionSheet setWillDismissBlock:(ActionSheetBlock)block;
- (void)actionSheet:(UIActionSheet *)actionSheet setDidDismissBlock:(ActionSheetBlock)block;
- (void)actionSheet:(UIActionSheet *)actionSheet setClickedBlock:(ActionSheetBlock)block;

@end


@implementation UIActionSheet (BlockActionSheet)

+ (BOOL)BlockActionSheetSupported
{
  return [NSMapTable class] != nil;
}

- (void)setWillDismissBlock:(ActionSheetBlock)block
{
  BlockActionSheetDelegate *dg = [BlockActionSheetDelegate sharedDelegate];
  [dg actionSheet:self setWillDismissBlock:block];
  self.delegate = dg;
}

- (void)setDidDismissBlock:(ActionSheetBlock)block
{
  BlockActionSheetDelegate *dg = [BlockActionSheetDelegate sharedDelegate];
  [dg actionSheet:self setDidDismissBlock:block];
  self.delegate = dg;
}

- (void)setClickedBlock:(ActionSheetBlock)block
{
  BlockActionSheetDelegate *dg = [BlockActionSheetDelegate sharedDelegate];
  [dg actionSheet:self setClickedBlock:block];
  self.delegate = dg;
}

@end

@implementation BlockActionSheetDelegate
{
  NSMapTable *_clickedMap;
  NSMapTable *_willDismissMap;
  NSMapTable *_didDismissMap;
}

+ (BlockActionSheetDelegate *)sharedDelegate
{
  static BlockActionSheetDelegate *obj = nil;
  if(!obj)
    obj = [[BlockActionSheetDelegate alloc]init];
  return obj;
}

- (instancetype)init
{
  if(!(self = [super init]))
    return nil;
  
  _clickedMap = [[NSMapTable alloc]initWithKeyOptions:NSMapTableWeakMemory valueOptions:NSMapTableCopyIn capacity:16];
  _willDismissMap = [[NSMapTable alloc]initWithKeyOptions:NSMapTableWeakMemory valueOptions:NSMapTableCopyIn capacity:16];
  _didDismissMap = [[NSMapTable alloc]initWithKeyOptions:NSMapTableWeakMemory valueOptions:NSMapTableCopyIn capacity:16];
  
  return self;
}

- (void)actionSheet:(UIActionSheet *)actionSheet calledMap:(NSMapTable *)map withButtonIndex:(NSInteger)buttonIndex
{
  ActionSheetBlock block = [map objectForKey:actionSheet];
  if(block)
    block(actionSheet, buttonIndex);
}

#pragma mark - Public

- (void)actionSheet:(UIActionSheet *)actionSheet setWillDismissBlock:(ActionSheetBlock)block
{
  [_willDismissMap setObject:block forKey:actionSheet];
}

- (void)actionSheet:(UIActionSheet *)actionSheet setDidDismissBlock:(ActionSheetBlock)block
{
  [_didDismissMap setObject:block forKey:actionSheet];
}

- (void)actionSheet:(UIActionSheet *)actionSheet setClickedBlock:(ActionSheetBlock)block
{
  [_clickedMap setObject:block forKey:actionSheet];
}

#pragma mark - UIActionSheetDelegate

- (void)actionSheet:(UIActionSheet *)actionSheet willDismissWithButtonIndex:(NSInteger)buttonIndex
{
  [self actionSheet:actionSheet calledMap:_willDismissMap withButtonIndex:buttonIndex];
}

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex
{
  [self actionSheet:actionSheet calledMap:_didDismissMap withButtonIndex:buttonIndex];
}

- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
  [self actionSheet:actionSheet calledMap:_clickedMap withButtonIndex:buttonIndex];
}

@end
