//
//  Welcome.m
//  iPasswordSafe
//
//  Created by Jorge Peixoto Vasquez on 6/13/11.
//  Copyright 2011 app77. All rights reserved.
//

#import "Welcome.h"
#import "GradientView.h"
#import "pwSafeTheme.h"

@interface Welcome()

@property (nonatomic, weak) IBOutlet UILabel *instructionsLabel;
@property (nonatomic, weak) IBOutlet UIImageView *instructionsImage;
@property (nonatomic, weak) IBOutlet UIButton *backButton;
@property (nonatomic, weak) IBOutlet UIButton *nextButton;
@property (weak, nonatomic) IBOutlet UILabel *titleLabel;
@property (weak, nonatomic) IBOutlet UIButton *skipButton;

@end

@implementation Welcome
@synthesize instructionsLabel;
@synthesize instructionsImage;
@synthesize backButton;
@synthesize nextButton;
@synthesize titleLabel;
@synthesize skipButton;

#define NUMBER_OF_SCREENS 5
#define LAST_SCREEN_IDX (NUMBER_OF_SCREENS - 1)

- (NSString *)textForScreen:(int)screen
{
#ifdef PWSAFE_VAULT
  NSString *safeStr = @"vault";
#else
  NSString *safeStr = @"safe";
#endif
  
  switch (screen)
  {
    case 0:
      return [NSString stringWithFormat:@"To create a new %@ to store your passwords, tap the '+' button.", safeStr];
    case 1:
      return @"To setup Dropbox sync, tap the Dropbox logo button.";
    case 2:
      return [NSString stringWithFormat:@"The lock button closes the current %@ or all %@s (on the home screen). It also clears the clipboard.", safeStr, safeStr];
    case 3:
      return @"This button copies the password and opens the URL in Safari with a single tap.";
    case 4:
      return @"Thank you for installing pwSafe. You can launch this welcome tour again from the settings screen.";
    default:
      NSAssert(0, @"Unknown screen");
      return nil;
  }
}

- (UIImage *)imageForScreen:(int)screen
{
  switch (screen) 
  {
    case 0:
      return [UIImage imageNamed:imageName(@"Welcome_CreateNew")];
    case 1:
      return [UIImage imageNamed:imageName(@"Welcome_Dropbox")];
    case 2:
      return [UIImage imageNamed:imageName(@"Welcome_Lock")];
    case 3:
      return [UIImage imageNamed:imageName(@"Welcome_OpenURL")];
    case 4:
      return [UIImage imageNamed:imageName(@"Welcome_Settings")];
    default:
      NSAssert(0, @"Unknown screen");
      return nil;
  }
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) 
    {
      screenNumber = 0;
    }
    return self;
}


#pragma mark - View lifecycle

- (void)viewDidLoad
{
  [super viewDidLoad];

  if(!iOS6())
  {
    for(UIButton *btn in @[nextButton, backButton, skipButton])
    {
      [btn setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
      [btn setTitleShadowColor:[UIColor darkGrayColor] forState:UIControlStateNormal];
      btn.titleLabel.shadowOffset = CGSizeMake(0, 1);
    }
  }
  
  backButton.hidden = YES;
  instructionsLabel.text = [self textForScreen:0];
  instructionsImage.image = [self imageForScreen:0];

  self.view.backgroundColor = kThemeDarkBgColor;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  BOOL res = (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
  return res;
}

- (void)advanceBy:(int)step
{
  int nextScreen = screenNumber + step;
  
  if(nextScreen < 0 || nextScreen > LAST_SCREEN_IDX)
    return;
  
  UIViewAnimationOptions effect = step > 0 ? UIViewAnimationOptionTransitionFlipFromRight : UIViewAnimationOptionTransitionFlipFromLeft;

  [UIView transitionWithView:instructionsImage
                    duration:.5
                     options:effect 
                  animations:^(void) {
                    instructionsImage.image = [self imageForScreen:nextScreen];
                  } completion:^(BOOL finished) {
                  }];
  [UIView transitionWithView:instructionsLabel
                    duration:.5
                     options:effect 
                  animations:^(void) {
                    instructionsLabel.text = [self textForScreen:nextScreen];
                  } completion:^(BOOL finished) {
                    [nextButton setTitle:nextScreen == (LAST_SCREEN_IDX) ? @"Done" : @"Next" forState:UIControlStateNormal];
                  }];
  
  
  if(screenNumber == 0 && nextScreen == 1)
  {
    backButton.alpha = 0;
    backButton.hidden = NO;
    [UIView animateWithDuration:.5 animations:^(void){
      backButton.alpha = 1;
    }];
  }
  if(screenNumber == 1 && nextScreen == 0)
  {
    [UIView animateWithDuration:.5
                     animations:^(void) {
                       backButton.alpha = 0;
                     } completion:^(BOOL finished) {
                       backButton.hidden = YES;
                     }];
  }

  screenNumber = nextScreen;
}

- (IBAction)onNext:(id)sender 
{
  if(screenNumber == LAST_SCREEN_IDX)
    [self onSkip:sender];
  else
    [self advanceBy:1];
}

- (IBAction)onBack:(id)sender 
{
  [self advanceBy:-1];
}

- (IBAction)onSkip:(id)sender 
{
  NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
  [prefs setBool:YES forKey:@"hideWelcomeScreen"];
  [prefs synchronize];
  DebugLog(@"ParentViewController = %@", self.parentViewController);

  [self dismissModalViewControllerAnimated:YES];
}

@end
