//
//  pwSafeTheme.m
//  pwSafe
//
//  Created by Jorge Peixoto Vasquez on 10/07/12.
//  Copyright (c) 2012 app77. All rights reserved.
//

#import "pwSafeTheme.h"
#import "UIImage+iPhone5.h"
#import "PageCellBackground.h"
#import "GradientView.h"
#import "NSObject+BlockKVO.h"

#import <objc/runtime.h>

extern "C" BOOL iOS6()
{
  static char ret = -1;
  if(ret < 0)
  {
    NSString *ver = [[UIDevice currentDevice] systemVersion];
    unichar first = [ver characterAtIndex:0];
    unichar second = [ver characterAtIndex:1];
    ret = second == L'.' && (first == L'5' || first == L'6');
  }
  
  return (BOOL)ret;
}

extern "C" NSString *imageName(NSString *name)
{
  return iOS6() ? name : [name stringByAppendingString:@"7"];
}

@interface UIColor (Utils)
//! allows for easy color creation, like UIColor *c = [UIColor colorWithRGBA:0x12345678];
+ (UIColor *)colorWithRGBA:(u_int32_t)rgba;
@end

@implementation UIColor (Utils)

+ (UIColor *)colorWithRGBA:(u_int32_t)rgba
{
  u_int32_t r = (rgba >> 24) & 0xFF;
  u_int32_t g = (rgba >> 16) & 0xFF;
  u_int32_t b = (rgba >> 8) & 0xFF;
  u_int32_t a = rgba & 0xFF;
  return [UIColor colorWithRed:r/255.0f green:g/255.0f blue:b/255.0f alpha:a/255.0f];
}

+ (UIColor *)colorWithR:(u_int32_t)r G:(u_int32_t)g B:(u_int32_t)b
{
  return [UIColor colorWithRed:r/255.0f green:g/255.0f blue:b/255.0f alpha:1.f];
}


@end

UIColor * const kThemeCellBgTopColor = [UIColor colorWithR:26 G:167 B:243];
UIColor * const kThemeCellBgBottomColor = [UIColor colorWithR:5 G:120 B:183];
UIColor * const kThemeCellUnselectedTopColor = [UIColor colorWithR:255 G:255 B:255];
UIColor * const kThemeCellUnselectedBottomColor = [UIColor colorWithR:236 G:238 B:239];

UIColor * const kThemeGroupedCellBgBgColor = [UIColor clearColor];
UIColor * const kThemeGroupedCellBgStrokeColor = [UIColor colorWithWhite:.5f alpha:1.f];
UIColor * const kThemeBackgroundPatternColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"backgroundPattern"]];
UIColor * const kThemeDarkBgColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"backgroundPatternDark"]];

UIColor * const kThemeTintColor = [UIColor colorWithR:8 G:127 B:191];
UIColor * const kThemeLightTintColor = [UIColor colorWithR:136 G:230 B:255];

const CGFloat kThemeGroupedCellBgCornerRadius = 4.0f;

UIColor * const kNavTextColor = [UIColor colorWithRed:0.89f green:0.91f blue:0.92f alpha:1.00f];
UIColor * const kNavUnselectedTextColor = [UIColor colorWithRed:0.69f green:0.71f blue:0.72f alpha:1.00f];
UIColor * const kNavTextShadowColor = [UIColor colorWithRed:0.09f green:0.10f blue:0.11f alpha:1.00f];
const CGSize kNavShadowOffset = CGSizeMake(0.0, 1.0);


UIColor * const kGrouppedTableViewLabelColor = [UIColor colorWithR:57 G:62 B:67];
static UIFont * kGrouppedTableViewLabelFont = nil;


@implementation UIBarButtonItem (pwSafeTheme)

+ (UIBarButtonItem *)buttonWithImageNamed:(NSString *)imageName target:(id)target action:(SEL)action
{
  UIBarButtonItem *btn = [[UIBarButtonItem alloc]initWithImage:[UIImage imageNamed:imageName] style:UIBarButtonItemStylePlain target:target action:action];
  return btn;
}

@end

@protocol UITableViewDelegateTheme <UITableViewDelegate>

@optional

- (UIView *)_replaced_tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section;

@end

@implementation UITableView (pwSafeTheme)

static CGFloat leftMarginForTableView(UITableView *tableView)
{
  CGFloat tableViewWidth = tableView.frame.size.width;
  
  if(tableViewWidth > 20)
  {
    if(tableViewWidth < 400.f)
      return 10.f;
    return MAX(31.f, MIN(45.f, tableViewWidth*0.06f));
  }
  return tableViewWidth - 10.f;
}

static UIView *groupedTableViewViewForHeaderInSection(id self, SEL _cmd, UITableView *tableView, NSInteger section)
{
  if([self respondsToSelector:@selector(_replaced_tableView:viewForHeaderInSection:)])
  {
    id<UITableViewDelegateTheme> me = self;
    UIView *view = [me _replaced_tableView:tableView viewForHeaderInSection:section];
    if(view)
      return view;
  }
  
  if(![self respondsToSelector:@selector(tableView:titleForHeaderInSection:)])
    return nil;
  
  NSString *text = [self tableView:tableView titleForHeaderInSection:section];
  if(!text.length)
    return nil;
  
  UILabel *label = [[UILabel alloc]init];
  label.opaque = NO;
  label.backgroundColor = [UIColor clearColor];
  
  if(!kGrouppedTableViewLabelFont)
    kGrouppedTableViewLabelFont = [UIFont boldSystemFontOfSize:[UIFont labelFontSize]];
  
  if([label respondsToSelector:@selector(setAttributedText:)])
  {
    NSMutableParagraphStyle *paragraph = [[NSMutableParagraphStyle alloc]init];
    [paragraph setFirstLineHeadIndent:leftMarginForTableView(tableView)];
    NSDictionary *attribs;
    attribs = @{
                NSFontAttributeName : kGrouppedTableViewLabelFont,
                NSForegroundColorAttributeName : kGrouppedTableViewLabelColor,
                NSParagraphStyleAttributeName : paragraph,
                };
    NSAttributedString *str = [[NSAttributedString alloc]initWithString:text attributes:attribs];
    label.attributedText = str;
    
    __weak UILabel *w_label = label;
    [tableView addBlockObserverForKeyPath:@"frame" options:0 autoremove:YES handler:^(id object, NSString *keyPath, NSDictionary *change) {
      UILabel *label = w_label;
      paragraph.firstLineHeadIndent = leftMarginForTableView(object);
      NSDictionary *attribs;
      attribs = @{
                  NSFontAttributeName : kGrouppedTableViewLabelFont,
                  NSForegroundColorAttributeName : kGrouppedTableViewLabelColor,
                  NSParagraphStyleAttributeName : paragraph,
                  };
      NSAttributedString *str = [[NSAttributedString alloc]initWithString:text attributes:attribs];
      label.attributedText = str;
    }];
    
    return label;
  }
  else
  {
    label.font = kGrouppedTableViewLabelFont;
    label.textColor = kGrouppedTableViewLabelColor;
    label.text = text;

    CGFloat h = [text sizeWithFont:kGrouppedTableViewLabelFont].height;
    
    CGFloat w = tableView.frame.size.width;
    NSAssert(w > 0, @"Must have width");
    
    UIView *bg = [[UIView alloc]initWithFrame:CGRectMake(0, 0, w, h + 7.f)];
    bg.opaque = NO;
    bg.backgroundColor = [UIColor clearColor];
    [bg addSubview:label];
    CGFloat leftMargin = leftMarginForTableView(tableView);
    label.frame = CGRectMake(leftMargin, 0, w - leftMargin, h);
    label.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleTopMargin;

    __weak UILabel *w_label = label;
    [tableView addBlockObserverForKeyPath:@"frame" options:0 autoremove:YES handler:^(id object, NSString *keyPath, NSDictionary *change) {
      UILabel *label = w_label;
      CGRect frame = label.frame;
      CGFloat newX = leftMarginForTableView(object);
      CGFloat diff = newX - frame.origin.x;
      frame.origin.x += diff;
      frame.size.width -= diff;
      label.frame = frame;
    }];
    
    return bg;
  }
}

#define kTableViewSectionHeaderH 22.0f
#define TABLE_VIEW_HDR_MARGIN 10.0f

static UIView *plainTableViewViewForHeaderInSection(id self, SEL _cmd, UITableView *tableView, NSInteger section)
{
  if([self respondsToSelector:@selector(_replaced_tableView:viewForHeaderInSection:)])
  {
    id<UITableViewDelegateTheme> me = self;
    UIView *view = [me _replaced_tableView:tableView viewForHeaderInSection:section];
    if(view)
      return view;
  }
  
  NSString *text = [tableView.dataSource tableView:tableView titleForHeaderInSection:section];
  
  if(!text.length)
    return nil;
  
  static NSArray *colors = nil;
  static UIFont *boldFont = nil;
  
  if(!boldFont)
  {
    boldFont = [UIFont boldSystemFontOfSize:[UIFont labelFontSize]];
    UIColor *darkColor = [UIColor colorWithR:110 G:121 B:132];
    UIColor *lightColor = [UIColor colorWithR:53 G:58 B:62];
    colors = @[(id)[darkColor colorWithAlphaComponent:.8f].CGColor, (id)[lightColor colorWithAlphaComponent:.8f].CGColor];
  }
  
  CGFloat w = tableView.frame.size.width;
  UIImage *image = [UIImage imageNamed:@"TableViewPlainSectionHeader"];
  CGSize imgSize = image.size;
  NSAssert(imgSize.height == kTableViewSectionHeaderH, @"Height must match");
  image = [image resizableImageWithCapInsets:UIEdgeInsetsMake(imgSize.height / 2, 0, imgSize.height / 2, 0)];
  UIImageView *imgView = [[UIImageView alloc]initWithImage:image];
  imgView.backgroundColor = [UIColor clearColor];
  imgView.opaque = NO;
  
  CGSize size = CGSizeMake(w - 2 * TABLE_VIEW_HDR_MARGIN, [text sizeWithFont:boldFont].height);
  UILabel *label = [[UILabel alloc]initWithFrame:CGRectMake(TABLE_VIEW_HDR_MARGIN, rintf((kTableViewSectionHeaderH - size.height) / 2), size.width, size.height)];
  label.text = text.uppercaseString;
  label.font = boldFont;
  label.backgroundColor = nil;
  label.opaque = NO;
  label.textColor = [UIColor whiteColor];
  [imgView addSubview:label];
  
  return imgView;
}

- (void)themeViewDidLoadInIpadPos:(TableViewPos)pos
{
  UITableViewStyle style = self.style;
  UIView *bg = [[UIView alloc]init];
  bg.opaque = YES;
  bg.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"backgroundPattern"]];
  self.backgroundView = bg;
  
  BOOL grouped = style == UITableViewStyleGrouped;
  
  if(!grouped)
  {
    self.tableFooterView = [[UIImageView alloc]initWithImage:[UIImage imageNamed:@"list-footer-bg"]];
  }
  
  if(iOS6())
  {
    id<UITableViewDelegate> tvc = self.delegate;
    __weak Class tvcc = [tvc class];
    IMP newMethod = grouped ? (IMP)groupedTableViewViewForHeaderInSection : (IMP)plainTableViewViewForHeaderInSection;
    
    IMP oldMethod = class_replaceMethod(tvcc, @selector(tableView:viewForHeaderInSection:), newMethod, "@@:@i");
    BOOL ok = !oldMethod || oldMethod == newMethod;
    if(!ok)
      ok = class_addMethod(tvcc, @selector(_replaced_tableView:viewForHeaderInSection:), oldMethod, "@@:@i");
    
    if(ok)
    {
      self.delegate = nil;
      self.delegate = tvc;
    }
  }
  else
  {
    self.sectionIndexBackgroundColor = [UIColor colorWithWhite:1 alpha:.2];
    self.sectionIndexTrackingBackgroundColor = [UIColor colorWithWhite:1 alpha:.8];
  }
}

static void removeBackgroundsFromView(UIView *view)
{
  view.backgroundColor = nil;
  view.opaque = NO;
  for(UIView *subview in view.subviews)
    removeBackgroundsFromView(subview);
}

#define SELECTED_BG_TAG 0xabcd

- (void) addBackgroundsToGroupedCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath
{
  if(!iOS6())
  {
    if(cell.selectedBackgroundView.tag == SELECTED_BG_TAG)
      return;
    CGRect frame = { .origin = CGPointZero, .size = cell.frame.size };
    UIView *selectedBg = [[UIView alloc]initWithFrame:frame];
    selectedBg.opaque = YES;
    selectedBg.tag = SELECTED_BG_TAG;
    selectedBg.backgroundColor = kThemeTintColor;
    cell.selectedBackgroundView = selectedBg;
    cell.textLabel.highlightedTextColor = [UIColor whiteColor];
    cell.detailTextLabel.highlightedTextColor = [UIColor whiteColor];
    return;
  }
  
  PageCellGroupPosition pos = [PageCellBackground positionForIndexPath:indexPath inTableView:self];
  
  PageCellBackground *bg = (PageCellBackground *)cell.backgroundView;
  if(bg && [bg isKindOfClass:[PageCellBackground class]])
  {
    bg.position = pos;
    
    PageCellBackground *selectedBg = (PageCellBackground *)cell.selectedBackgroundView;
    if(selectedBg && [selectedBg isKindOfClass:[PageCellBackground class]])
      selectedBg.position = pos;
    
   
    return;
  }
  
  bg = [[PageCellBackground alloc]initSelected:NO grouped:YES];
  bg.position = pos;
  cell.backgroundView = bg;
  
  if(cell.selectionStyle != UITableViewCellSelectionStyleNone)
  {
    PageCellBackground *selectedBg = [[PageCellBackground alloc]initSelected:YES grouped:YES];
    selectedBg.position = pos;
    cell.selectedBackgroundView = selectedBg;
  }
  
  dispatch_async(dispatch_get_main_queue(), ^(void){
    removeBackgroundsFromView(cell.contentView);
    
  });
}

- (void)addThemeToPlainCell:(UITableViewCell *)cell
{
  if(!iOS6())
    return;
  GradientView *bg = [[GradientView alloc]init];
  bg.colors = @[(__bridge id)[UIColor colorWithWhite:1.f alpha:1.f].CGColor, (__bridge id)[UIColor colorWithWhite:240.f/255.f alpha:1.f].CGColor];
  bg.startPoint = CGPointMake(.5f, 0.f);
  bg.endPoint = CGPointMake(.5f, 1.f);
  
  cell.backgroundView = bg;
  
  cell.detailTextLabel.backgroundColor = [UIColor clearColor];
  cell.detailTextLabel.opaque = NO;

  cell.textLabel.backgroundColor = [UIColor clearColor];
  cell.textLabel.opaque = NO;
}

@end

@implementation UIViewController (pwSafeTheme)

- (void)themeViewWithNavPromptDidLoad
{
  if(!iOS6())
    return;
  
  UIImage *bg = [UIImage imageNamed:@"navigationBackgroundPrompt"];
  UINavigationBar *bar = self.navigationController.navigationBar;
  [bar setBackgroundImage:bg forBarMetrics:UIBarMetricsDefault];
}

@end

UIView *themeGroupViewDisabledImage(CGRect frame, BOOL landscape)
{
  UIImage *img = [UIImage imageNamed:@"ElefanteTransparente"];
  UIImageView *imgView = [[UIImageView alloc]initWithImage:img];
  imgView.backgroundColor = nil;
  imgView.opaque = NO;
  frame.origin.x = 87;
  frame.size.width -= 87 * 2;
  imgView.frame = frame;
  imgView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
  imgView.contentMode = UIViewContentModeScaleAspectFit;  
  
  return imgView;
}

void pwSafeThemeSetup(UIWindow *window)
{
  UIColor *barTintColor = [UIColor colorWithRed:82.f/255.f green:89.f/255.f blue:97.f/255.f alpha:1.f];

  UINavigationBar *navBar = [UINavigationBar appearance];
  
  BOOL old = iOS6();
  
  if(old)
  {
    UINavigationBar *realNavBar = [[UINavigationBar alloc]init];
    if([realNavBar respondsToSelector:@selector(setShadowImage:)])
    {
      [navBar setShadowImage:[UIImage imageNamed:@"navigationShadow"] ];
      [navBar setBackgroundImage:[UIImage imageNamed:@"navigationBackgroundNoShadow"] forBarMetrics:UIBarMetricsDefault];
    }
    else
    {
      [navBar setBackgroundImage:[UIImage imageNamed:@"navigationBackground"] forBarMetrics:UIBarMetricsDefault];
    }
    [navBar setTintColor:barTintColor];
  
    UIBarButtonItem *barButtonItem = [UIBarButtonItem appearanceWhenContainedIn:[UINavigationController class], nil];
    [barButtonItem setBackgroundImage:[[UIImage imageNamed:@"barButton"]resizableImageWithCapInsets:UIEdgeInsetsMake(0.0, 5.0, 0.0, 5.0)] forState:UIControlStateNormal barMetrics:UIBarMetricsDefault];
    [barButtonItem setBackButtonBackgroundImage:[[UIImage imageNamed:@"backButton"]resizableImageWithCapInsets:UIEdgeInsetsMake(0.0, 15.0, 0.0, 5.0)] forState:UIControlStateNormal barMetrics:UIBarMetricsDefault];
    
    UIToolbar *toolBar = [UIToolbar appearanceWhenContainedIn:[UINavigationController class], nil];
    [toolBar setBackgroundImage:[UIImage imageNamed:@"toolbarBackground"] forToolbarPosition:UIToolbarPositionBottom barMetrics:UIBarMetricsDefault];
    [toolBar setTintColor:barTintColor];
    
    UISearchBar *search = [UISearchBar appearance];
    [search setBackgroundImage:[UIImage imageNamed:@"searchBackground"]];
    [search setSearchFieldBackgroundImage:[[UIImage imageNamed:@"searchField"]resizableImageWithCapInsets:UIEdgeInsetsMake(0.0, 16.0, 0.0, 16.0)] forState:UIControlStateNormal];
    [search setImage:[UIImage imageNamed:@"searchIconSearch"] forSearchBarIcon:UISearchBarIconSearch state:UIControlStateNormal];
    [search setScopeBarBackgroundImage:[UIImage imageNamed:@"searchScopeBackground"]];
    [search setScopeBarButtonBackgroundImage:[[UIImage imageNamed:@"searchScopeButton"]resizableImageWithCapInsets:UIEdgeInsetsMake(6.0, 6.0, 6.0, 6.0)] forState:UIControlStateNormal];
    [search setScopeBarButtonBackgroundImage:[[UIImage imageNamed:@"searchScopeButtonSelected"]resizableImageWithCapInsets:UIEdgeInsetsMake(6.0, 6.0, 6.0, 6.0)] forState:UIControlStateSelected];
    [search setScopeBarButtonDividerImage:[UIImage imageNamed:@"searchScopeButtonDivider"] forLeftSegmentState:UIControlStateNormal rightSegmentState:UIControlStateNormal];

    NSDictionary *titleTextAttributesNav = @{
                                             UITextAttributeTextColor : kNavTextColor,
                                             UITextAttributeTextShadowColor : kNavTextShadowColor,
                                             UITextAttributeTextShadowOffset : [NSValue valueWithCGSize:kNavShadowOffset],
                                             };

    [navBar setTitleTextAttributes:titleTextAttributesNav];
    [barButtonItem setTitleTextAttributes:titleTextAttributesNav forState:UIControlStateNormal];
    [search setScopeBarButtonTitleTextAttributes:titleTextAttributesNav forState:UIControlStateSelected];
    
    NSDictionary *unselectedScopeTextAttributes = @{
                                                    UITextAttributeTextColor : kNavUnselectedTextColor,
                                                    UITextAttributeTextShadowColor : kNavTextShadowColor,
                                                    UITextAttributeTextShadowOffset : [NSValue valueWithCGSize:kNavShadowOffset],
                                                    };
    
    [search setScopeBarButtonTitleTextAttributes:unselectedScopeTextAttributes forState:UIControlStateNormal];
    
    [[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleBlackOpaque];
  }
  else
  {
    [window setTintColor:kThemeTintColor];
    
    [navBar setBarStyle:UIBarStyleBlack];
    
    [navBar setTintColor:kThemeLightTintColor];
    [navBar setTitleTextAttributes:@{UITextAttributeTextColor:[UIColor whiteColor]}];
    
    UIToolbar *toolbar = [UIToolbar appearance];
    [toolbar setBarStyle:UIBarStyleBlack];
    [toolbar setTintColor:kThemeLightTintColor];
    
    [[UIApplication sharedApplication]setStatusBarStyle:UIStatusBarStyleLightContent];
  }
}

