//
//  IASKAppSettingsViewController.m
//  http://www.inappsettingskit.com
//
//  Copyright (c) 2009-2010:
//  Luc Vandal, Edovia Inc., http://www.edovia.com
//  Ortwin Gentz, FutureTap GmbH, http://www.futuretap.com
//  All rights reserved.
// 
//  It is appreciated but not required that you give credit to Luc Vandal and Ortwin Gentz, 
//  as the original authors of this code. You can give credit in a blog post, a tweet or on 
//  a info page of your app. Also, the original authors appreciate letting them know if you use this code.
//
//  This code is licensed under the BSD license that is available at: http://www.opensource.org/licenses/bsd-license.php
//


#import "IASKAppSettingsViewController.h"
#import "IASKSettingsReader.h"
#import "IASKSettingsStoreUserDefaults.h"
#import "IASKPSToggleSwitchSpecifierViewCell.h"
#import "IASKPSSliderSpecifierViewCell.h"
#import "IASKPSTextFieldSpecifierViewCell.h"
#import "IASKPSTitleValueSpecifierViewCell.h"
#import "IASKSwitch.h"
#import "IASKSlider.h"
#import "IASKSpecifier.h"
#import "IASKSpecifierValuesViewController.h"
#import "IASKTextField.h"

#import "AppTheme.h"

static NSString *kIASKCredits = @"Powered by InAppSettingsKit"; // Leave this as-is!!!

#define kIASKSpecifierValuesViewControllerIndex       0
#define kIASKSpecifierChildViewControllerIndex        1

#define kIASKCreditsViewWidth                         285

CGRect IASKCGRectSwap(CGRect rect);

@interface IASKAppSettingsViewController ()
@property (nonatomic, strong) NSMutableArray *viewList;
@property (nonatomic, strong) NSIndexPath *currentIndexPath;
@property (nonatomic, strong) id currentFirstResponder;

- (void)_textChanged:(id)sender;
- (void)synchronizeSettings;
- (void)reload;
@end

@implementation IASKAppSettingsViewController

@synthesize delegate = _delegate;
@synthesize viewList = _viewList;
@synthesize currentIndexPath = _currentIndexPath;
@synthesize settingsReader = _settingsReader;
@synthesize file = _file;
@synthesize currentFirstResponder = _currentFirstResponder;
@synthesize showCreditsFooter = _showCreditsFooter;
@synthesize showDoneButton = _showDoneButton;
@synthesize settingsStore = _settingsStore;
@synthesize showAppVersion = _showAppVersion;

#pragma mark accessors
- (IASKSettingsReader*)settingsReader {
  if (!_settingsReader) {
    _settingsReader = [[IASKSettingsReader alloc] initWithFile:self.file customBundle:nil];
  }
  return _settingsReader;
}

- (id<IASKSettingsStore>)settingsStore {
  if (!_settingsStore) {
    _settingsStore = [[IASKSettingsStoreUserDefaults alloc] init];
  }
  return _settingsStore;
}

- (NSString*)file {
  if (!_file) {
    return @"Root";
  }
  return _file;
}

- (void)setFile:(NSString *)file {
  if (file != _file) {
    _file = [file copy];
  }
  
  self.settingsReader = nil; // automatically initializes itself
}

#pragma mark standard view controller methods
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
  if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
    // If set to YES, will display credits for InAppSettingsKit creators
    _showCreditsFooter = YES;
    
    // If set to YES, will add a DONE button at the right of the navigation bar
    _showDoneButton = YES;
  }
  return self;
}

- (void)awakeFromNib {
  // If set to YES, will display credits for InAppSettingsKit creators
  _showCreditsFooter = YES;
  
  // If set to YES, will add a DONE button at the right of the navigation bar
  // if loaded via NIB, it's likely we sit in a TabBar- or NavigationController
  // and thus don't need the Done button
  _showDoneButton = NO;
}

- (NSMutableArray *)viewList {
  if (!_viewList) {
    _viewList = [[NSMutableArray alloc] init];
    [_viewList addObject:@{@"ViewName": @"IASKSpecifierValuesView"}];
    [_viewList addObject:@{@"ViewName": @"IASKAppSettingsView"}];
  }
  return _viewList;
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  [self.tableView themeViewDidLoadInIpadPos:TVP_FORMSHEET];  
}

- (void)viewDidUnload {
  [super viewDidUnload];
  // Release any retained subviews of the main view.
  // e.g. self.myOutlet = nil;
  self.view = nil;
  self.viewList = nil;
}

- (void)viewWillAppear:(BOOL)animated {
  [self.tableView reloadData];
  
  self.navigationItem.rightBarButtonItem = nil;
  self.navigationController.delegate = self;
  if (_showDoneButton) {
    UIBarButtonItem *buttonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone 
                                                                                target:self 
                                                                                action:@selector(dismiss:)];
    self.navigationItem.rightBarButtonItem = buttonItem;
  } 
  if (!self.title) {
    self.title = NSLocalizedString(@"Settings", @"");
  }
  
  if (self.currentIndexPath) {
    if (animated) {
      // animate deselection of previously selected row
      [self.tableView selectRowAtIndexPath:self.currentIndexPath animated:NO scrollPosition:UITableViewScrollPositionNone];
      [self.tableView deselectRowAtIndexPath:self.currentIndexPath animated:YES];
    }
    self.currentIndexPath = nil;
  }
  
  [super viewWillAppear:animated];
}

- (CGSize)contentSizeForViewInPopover {
  return [[self view] sizeThatFits:CGSizeMake(320, 2000)];
}

- (void)viewDidAppear:(BOOL)animated {
  [super viewDidAppear:animated];
  
  NSNotificationCenter *dc = [NSNotificationCenter defaultCenter];
  IASK_IF_IOS4_OR_GREATER([dc addObserver:self selector:@selector(synchronizeSettings) name:UIApplicationDidEnterBackgroundNotification object:[UIApplication sharedApplication]];);
  IASK_IF_IOS4_OR_GREATER([dc addObserver:self selector:@selector(reload) name:UIApplicationWillEnterForegroundNotification object:[UIApplication sharedApplication]];);
  [dc addObserver:self selector:@selector(synchronizeSettings) name:UIApplicationWillTerminateNotification object:[UIApplication sharedApplication]];
}

- (void)viewWillDisappear:(BOOL)animated {
  [NSObject cancelPreviousPerformRequestsWithTarget:self];
  [super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated {
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  
  if (!self.navigationController.delegate) {
    // hide the keyboard when we're popping from the navigation controller
    [self.currentFirstResponder resignFirstResponder];
  }
  
  [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  return (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad || interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)didReceiveMemoryWarning {
  // Releases the view if it doesn't have a superview.
  [super didReceiveMemoryWarning];
  
  // Release any cached data, images, etc that aren't in use.
}

- (void)navigationController:(UINavigationController *)navigationController willShowViewController:(UIViewController *)viewController animated:(BOOL)animated {
  if (![viewController isKindOfClass:[IASKAppSettingsViewController class]] && ![viewController isKindOfClass:[IASKSpecifierValuesViewController class]]) {
    [self dismiss:nil];
  }
}

- (void)dealloc {
  if([self.delegate respondsToSelector:@selector(settingsViewControllerDeallocing:)])
    [self.delegate settingsViewControllerDeallocing:self];
  
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  
  _viewList = nil;
  _currentIndexPath = nil;
  _file = nil;
  _currentFirstResponder = nil;
  _settingsReader = nil;
  _settingsStore = nil;
  
  _delegate = nil;
  
  if(self.isViewLoaded)
  {
    self.tableView.delegate = nil;
    self.tableView.dataSource = nil;
  }
}

#pragma mark -
#pragma mark Actions

- (IBAction)dismiss:(id)sender {
  [self.settingsStore synchronize];
  self.navigationController.delegate = nil;
  
  if (self.delegate && [self.delegate conformsToProtocol:@protocol(IASKSettingsDelegate)]) {
    [self.delegate settingsViewControllerDidEnd:self];
  }
}

- (void)toggledValue:(id)sender {
  IASKSwitch *toggle    = (IASKSwitch*)sender;
  IASKSpecifier *spec   = [_settingsReader specifierForKey:[toggle key]];
  
  if ([toggle isOn]) {
    if ([spec trueValue] != nil) {
      [self.settingsStore setObject:[spec trueValue] forKey:[toggle key]];
    }
    else {
      [self.settingsStore setBool:YES forKey:[toggle key]]; 
    }
  }
  else {
    if ([spec falseValue] != nil) {
      [self.settingsStore setObject:[spec falseValue] forKey:[toggle key]];
    }
    else {
      [self.settingsStore setBool:NO forKey:[toggle key]]; 
    }
  }
  [[NSNotificationCenter defaultCenter] postNotificationName:kIASKAppSettingChanged
                                                      object:[toggle key]
                                                    userInfo:@{[toggle key]: [self.settingsStore objectForKey:[toggle key]]}];
}

- (void)sliderChangedValue:(id)sender {
  IASKSlider *slider = (IASKSlider*)sender;
  [self.settingsStore setFloat:[slider value] forKey:[slider key]];
  [[NSNotificationCenter defaultCenter] postNotificationName:kIASKAppSettingChanged
                                                      object:[slider key]
                                                    userInfo:@{[slider key]: @([slider value])}];
}


#pragma mark -
#pragma mark UITableView Functions

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
  return [self.settingsReader numberOfSections];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
  NSInteger n = [self.settingsReader numberOfRowsForSection:section];
  return n;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
  IASKSpecifier *specifier  = [self.settingsReader specifierForIndexPath:indexPath];
  if ([[specifier type] isEqualToString:kIASKCustomViewSpecifier]) {
    if ([self.delegate respondsToSelector:@selector(tableView:heightForSpecifier:)]) {
      return [self.delegate tableView:tableView heightForSpecifier:specifier];
    } else {
      return 0;
    }
  }
  return tableView.rowHeight;
}

- (NSString *)tableView:(UITableView*)tableView titleForHeaderInSection:(NSInteger)section {
  NSString *header = [self.settingsReader titleForSection:section];
  if (0 == header.length) {
    return nil;
  }
  return header;
}

- (UIView *)tableView:(UITableView*)tableView viewForHeaderInSection:(NSInteger)section {
  NSString *key  = [self.settingsReader keyForSection:section];
  if ([self.delegate respondsToSelector:@selector(tableView:viewForHeaderForKey:)]) {
    return [self.delegate tableView:tableView viewForHeaderForKey:key];
  } else {
    return nil;
  }
}

- (CGFloat)tableView:(UITableView*)tableView heightForHeaderInSection:(NSInteger)section {
  NSString *key  = [self.settingsReader keyForSection:section];
  if ([self tableView:tableView viewForHeaderInSection:section] && [self.delegate respondsToSelector:@selector(tableView:heightForHeaderForKey:)]) {
    CGFloat result;
    if ((result = [self.delegate tableView:tableView heightForHeaderForKey:key])) {
      return result;
    }
    
  }
  NSString *title;
  if ((title = [self tableView:tableView titleForHeaderInSection:section])) {
    CGSize size = [title sizeWithFont:[UIFont boldSystemFontOfSize:[UIFont labelFontSize]] 
                    constrainedToSize:CGSizeMake(tableView.frame.size.width - 2*kIASKHorizontalPaddingGroupTitles, INFINITY)
                        lineBreakMode:UILineBreakModeWordWrap];
    return size.height+kIASKVerticalPaddingGroupTitles;
  }
  return 0;
}

- (NSString *)tableView:(UITableView *)tableView titleForFooterInSection:(NSInteger)section
{
  NSString *footerText = [self.settingsReader footerTextForSection:section];
  BOOL lastSection = section == [self.settingsReader numberOfSections] - 1;
  if (_showCreditsFooter && lastSection) 
  {
    // show credits since this is the last section
    if ((footerText == nil) || ([footerText length] == 0)) 
      // show the credits on their own
      return kIASKCredits;
    else
      // show the credits below the app's FooterText
      return [NSString stringWithFormat:@"%@\n\n%@", footerText, kIASKCredits];
  }
  else if(_showAppVersion && lastSection)
  {
    NSString *version = [NSString stringWithFormat:@"Version: %@", [[NSBundle mainBundle]objectForInfoDictionaryKey:@"CFBundleShortVersionString"]];
    // show version since this is the last section
    if ((footerText == nil) || ([footerText length] == 0)) 
      // show the version on their own
      return version;
    else
      // show the version below the app's FooterText
      return [NSString stringWithFormat:@"%@\n\n%@", footerText, version];
  }
  else 
  {
    if ([footerText length] == 0) 
      return nil;
    return [self.settingsReader footerTextForSection:section];
  }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
  IASKSpecifier *specifier  = [self.settingsReader specifierForIndexPath:indexPath];
  NSString *key           = [specifier key];
  
  if ([[specifier type] isEqualToString:kIASKPSToggleSwitchSpecifier]) {
    IASKPSToggleSwitchSpecifierViewCell *cell = (IASKPSToggleSwitchSpecifierViewCell*)[tableView dequeueReusableCellWithIdentifier:[specifier type]];
    
    if (!cell) {
      cell = (IASKPSToggleSwitchSpecifierViewCell*) [[NSBundle mainBundle] loadNibNamed:imageName(@"IASKPSToggleSwitchSpecifierViewCell")
                                                                                   owner:self
                                                                                 options:nil][0];
    }
    [[cell label] setText:[specifier title]];
    
    BOOL enabled = YES;
    if(self.delegate && [self.delegate respondsToSelector:@selector(isEnabledForSpecifier:)])
      enabled = [self.delegate isEnabledForSpecifier:specifier];
    
    cell.label.enabled = enabled;
    cell.toggle.enabled = enabled;
    if(enabled)
      cell.accessibilityTraits &=~ UIAccessibilityTraitNotEnabled;
    else
      cell.accessibilityTraits |= UIAccessibilityTraitNotEnabled;
    
    id currentValue = [self.settingsStore objectForKey:key];
    BOOL toggleState;
    if (currentValue) {
      if ([currentValue isEqual:[specifier trueValue]]) {
        toggleState = YES;
      } else if ([currentValue isEqual:[specifier falseValue]]) {
        toggleState = NO;
      } else {
        toggleState = [currentValue boolValue];
      }
    } else {
      toggleState = [specifier defaultBoolValue];
    }
    [[cell toggle] setOn:toggleState];
    
    [[cell toggle] addTarget:self action:@selector(toggledValue:) forControlEvents:UIControlEventValueChanged];
    [[cell toggle] setKey:key];
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];
    return cell;
  }
  else if ([[specifier type] isEqualToString:kIASKPSMultiValueSpecifier]) {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:[specifier type]];
    
    if (!cell) {
      cell = [[IASKPSTitleValueSpecifierViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:[specifier type]];
      cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
      cell.accessibilityTraits = UIAccessibilityTraitButton;
      cell.backgroundColor = [UIColor whiteColor];
    }
    
    BOOL enabled = YES;
    if(self.delegate && [self.delegate respondsToSelector:@selector(isEnabledForSpecifier:)])
      enabled = [self.delegate isEnabledForSpecifier:specifier];
    cell.textLabel.enabled = enabled;
    cell.detailTextLabel.enabled = enabled;
    cell.selectionStyle = enabled ? UITableViewCellSelectionStyleBlue : UITableViewCellSelectionStyleNone;
    if(enabled)
      cell.accessibilityTraits &=~ UIAccessibilityTraitNotEnabled;
    else
      cell.accessibilityTraits |= UIAccessibilityTraitNotEnabled;

    [[cell textLabel] setText:[specifier title]];
    [[cell detailTextLabel] setText:[[specifier titleForCurrentValue:[self.settingsStore objectForKey:key] != nil ? 
                                      [self.settingsStore objectForKey:key] : [specifier defaultValue]] description]];
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];
    return cell;
  }
  else if ([[specifier type] isEqualToString:kIASKPSTitleValueSpecifier]) {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:[specifier type]];
    
    if (!cell) {
      cell = [[IASKPSTitleValueSpecifierViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:[specifier type]];
      cell.accessoryType = UITableViewCellAccessoryNone;
      cell.backgroundColor = [UIColor whiteColor];
    }
    
    cell.textLabel.text = [specifier title];
    id value = [self.settingsStore objectForKey:key] ? : [specifier defaultValue];
    
    NSString *stringValue;
    if ([specifier multipleValues] || [specifier multipleTitles]) {
      stringValue = [specifier titleForCurrentValue:value];
    } else {
      stringValue = [value description];
    }
    
    cell.detailTextLabel.text = stringValue;
    [cell setUserInteractionEnabled:NO];

    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];
    return cell;
  }
  else if ([[specifier type] isEqualToString:kIASKPSTextFieldSpecifier]) {
    IASKPSTextFieldSpecifierViewCell *cell = (IASKPSTextFieldSpecifierViewCell*)[tableView dequeueReusableCellWithIdentifier:[specifier type]];
    
    if (!cell) {
      cell = (IASKPSTextFieldSpecifierViewCell*) [[NSBundle mainBundle] loadNibNamed:imageName(@"IASKPSTextFieldSpecifierViewCell")
                                                                                owner:self 
                                                                              options:nil][0];
      
      cell.textField.textAlignment = UITextAlignmentLeft;
      cell.textField.returnKeyType = UIReturnKeyDone;
      cell.accessoryType = UITableViewCellAccessoryNone;
    }
    
    [[cell label] setText:[specifier title]];      
    
    NSString *textValue = [self.settingsStore objectForKey:key] != nil ? [self.settingsStore objectForKey:key] : [specifier defaultStringValue];
    if (textValue && ![textValue isMemberOfClass:[NSString class]]) {
      textValue = [NSString stringWithFormat:@"%@", textValue];
    }
    [[cell textField] setText:textValue];
    
    [[cell textField] setKey:key];
    [[cell textField] setDelegate:self];
    [[cell textField] addTarget:self action:@selector(_textChanged:) forControlEvents:UIControlEventEditingChanged];
    [[cell textField] setSecureTextEntry:[specifier isSecure]];
    [[cell textField] setKeyboardType:[specifier keyboardType]];
    [[cell textField] setAutocapitalizationType:[specifier autocapitalizationType]];
    if([specifier isSecure]){
      [[cell textField] setAutocorrectionType:UITextAutocorrectionTypeNo];
    } else {
      [[cell textField] setAutocorrectionType:[specifier autoCorrectionType]];
    }
    [cell setNeedsLayout];
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];    
    return cell;
  }
  else if ([[specifier type] isEqualToString:kIASKPSSliderSpecifier]) {
    IASKPSSliderSpecifierViewCell *cell = (IASKPSSliderSpecifierViewCell*)[tableView dequeueReusableCellWithIdentifier:[specifier type]];
    
    if (!cell) {
      cell = (IASKPSSliderSpecifierViewCell*) [[NSBundle mainBundle] loadNibNamed:@"IASKPSSliderSpecifierViewCell" 
                                                                             owner:self 
                                                                           options:nil][0];
    }
    
    if ([[specifier minimumValueImage] length] > 0) {
      [[cell minImage] setImage:[UIImage imageWithContentsOfFile:[_settingsReader pathForImageNamed:[specifier minimumValueImage]]]];
    }
    
    if ([[specifier maximumValueImage] length] > 0) {
      [[cell maxImage] setImage:[UIImage imageWithContentsOfFile:[_settingsReader pathForImageNamed:[specifier maximumValueImage]]]];
    }
    
    [[cell slider] setMinimumValue:[specifier minimumValue]];
    [[cell slider] setMaximumValue:[specifier maximumValue]];
    [[cell slider] setValue:[self.settingsStore objectForKey:key] != nil ? 
     [[self.settingsStore objectForKey:key] floatValue] : [[specifier defaultValue] floatValue]];
    [[cell slider] addTarget:self action:@selector(sliderChangedValue:) forControlEvents:UIControlEventValueChanged];
    [[cell slider] setKey:key];
    [cell setNeedsLayout];
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];    
    return cell;
  }
  else if ([[specifier type] isEqualToString:kIASKPSChildPaneSpecifier]) {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:[specifier type]];
    
    if (!cell) {
      cell = [[IASKPSTitleValueSpecifierViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:[specifier type]];
      [cell setAccessoryType:UITableViewCellAccessoryDisclosureIndicator];
      cell.accessibilityTraits = UIAccessibilityTraitButton;
      cell.backgroundColor = [UIColor whiteColor];
    }
    
    [[cell textLabel] setText:[specifier title]];
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];    
    return cell;
  } else if ([[specifier type] isEqualToString:kIASKOpenURLSpecifier]) {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:[specifier type]];
    
    if (!cell) {
      cell = [[IASKPSTitleValueSpecifierViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:[specifier type]];
      [cell setAccessoryType:UITableViewCellAccessoryDisclosureIndicator];
      cell.accessibilityTraits = UIAccessibilityTraitButton;
      cell.backgroundColor = [UIColor whiteColor];
    }
    
    cell.textLabel.text = [specifier title];
    cell.detailTextLabel.text = [[specifier defaultValue] description];
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];    
    return cell;        
  } else if ([[specifier type] isEqualToString:kIASKButtonSpecifier]) {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:[specifier type]];
    
    if (!cell) {
      cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:[specifier type]];
      cell.backgroundColor = [UIColor whiteColor];
    }
    cell.textLabel.text = [specifier title];
    BOOL enabled = YES;
    if(self.delegate && [self.delegate respondsToSelector:@selector(isEnabledForSpecifier:)])
      enabled = [self.delegate isEnabledForSpecifier:specifier];
    cell.textLabel.enabled = enabled;
    cell.selectionStyle = enabled ? UITableViewCellSelectionStyleBlue : UITableViewCellSelectionStyleNone;
    cell.textLabel.textAlignment = UITextAlignmentCenter;
    cell.accessibilityTraits = UIAccessibilityTraitButton;
    if(enabled)
      cell.accessibilityTraits &=~ UIAccessibilityTraitNotEnabled;
    else
      cell.accessibilityTraits |= UIAccessibilityTraitNotEnabled;
    
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];    
    return cell;
  } else if ([[specifier type] isEqualToString:kIASKMailComposeSpecifier]) {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:[specifier type]];
    
    if (!cell) {
      cell = [[IASKPSTitleValueSpecifierViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:[specifier type]];
      [cell setAccessoryType:UITableViewCellAccessoryDisclosureIndicator];
      cell.accessibilityTraits = UIAccessibilityTraitButton;
      cell.backgroundColor = [UIColor whiteColor];
    }
    
    cell.textLabel.text = [specifier title];
    cell.detailTextLabel.text = [[specifier defaultValue] description];
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];    
    return cell;
  } else if ([[specifier type] isEqualToString:kIASKCustomViewSpecifier] && [self.delegate respondsToSelector:@selector(tableView:cellForSpecifier:)]) {
    UITableViewCell *cell = [self.delegate tableView:tableView cellForSpecifier:specifier];
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];
    return cell;
  } else {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:[specifier type]];
    
    if (!cell) {
      cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:[specifier type]];
      cell.backgroundColor = [UIColor whiteColor];
    }
    [[cell textLabel] setText:[specifier title]];
    [tableView addBackgroundsToGroupedCell:cell atIndexPath:indexPath];    
    return cell;
  }
}

- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath {
  IASKSpecifier *specifier  = [self.settingsReader specifierForIndexPath:indexPath];
  
  if ([[specifier type] isEqualToString:kIASKPSToggleSwitchSpecifier]) {
    return nil;
  }
  
  BOOL enabled = YES;
  if(self.delegate && [self.delegate respondsToSelector:@selector(isEnabledForSpecifier:)])
    enabled = [self.delegate isEnabledForSpecifier:specifier];
  
  if(!enabled)
    return nil;
  
  return indexPath;
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
  IASKSpecifier *specifier  = [self.settingsReader specifierForIndexPath:indexPath];
  
  if ([[specifier type] isEqualToString:kIASKPSToggleSwitchSpecifier]) {
    [tableView deselectRowAtIndexPath:indexPath animated:NO];
  }
  else if ([[specifier type] isEqualToString:kIASKPSMultiValueSpecifier]) {
    BOOL enabled = YES;
    if(self.delegate && [self.delegate respondsToSelector:@selector(isEnabledForSpecifier:)])
      enabled = [self.delegate isEnabledForSpecifier:specifier];
    
    if(!enabled)
      return;
    
    IASKSpecifierValuesViewController *targetViewController = (self.viewList)[kIASKSpecifierValuesViewControllerIndex][@"viewController"];
    
    if (targetViewController == nil) {
      // the view controller has not been created yet, create it and set it to our viewList array
      // create a new dictionary with the new view controller
      NSMutableDictionary *newItemDict = [NSMutableDictionary dictionaryWithCapacity:3];
      [newItemDict addEntriesFromDictionary: (self.viewList)[kIASKSpecifierValuesViewControllerIndex]];	// copy the title and explain strings
      
      targetViewController = [[IASKSpecifierValuesViewController alloc] initWithNibName:@"IASKSpecifierValuesView" bundle:nil];
      // add the new view controller to the dictionary and then to the 'viewList' array
      newItemDict[@"viewController"] = targetViewController;
      (self.viewList)[kIASKSpecifierValuesViewControllerIndex] = newItemDict;
      
      // load the view controll back in to push it
      targetViewController = (self.viewList)[kIASKSpecifierValuesViewControllerIndex][@"viewController"];
    }
    self.currentIndexPath = indexPath;
    [targetViewController setCurrentSpecifier:specifier];
    targetViewController.settingsReader = self.settingsReader;
    targetViewController.settingsStore = self.settingsStore;
    [[self navigationController] pushViewController:targetViewController animated:YES];
  }
  else if ([[specifier type] isEqualToString:kIASKPSSliderSpecifier]) {
    [tableView deselectRowAtIndexPath:indexPath animated:NO];
  }
  else if ([[specifier type] isEqualToString:kIASKPSTextFieldSpecifier]) {
    IASKPSTextFieldSpecifierViewCell *textFieldCell = (id)[tableView cellForRowAtIndexPath:indexPath];
    [textFieldCell.textField becomeFirstResponder];
  }
  else if ([[specifier type] isEqualToString:kIASKPSChildPaneSpecifier]) {
    
    
    Class vcClass = [specifier viewControllerClass];
    if (vcClass) {
      SEL initSelector = [specifier viewControllerSelector];
      UIViewController * vc = [vcClass alloc];

      if (!initSelector)
      {
        vc = [vc init];
      }
      else
      {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
        [vc performSelector:initSelector withObject:[specifier file] withObject:[specifier key]];
#pragma clang diagnostic pop
      }
      
      if ([vc respondsToSelector:@selector(setDelegate:)]) {
        [vc performSelector:@selector(setDelegate:) withObject:self.delegate];
      }
      if ([vc respondsToSelector:@selector(setSettingsStore:)]) {
        [vc performSelector:@selector(setSettingsStore:) withObject:self.settingsStore];
      }
      self.navigationController.delegate = nil;
      [self.navigationController pushViewController:vc animated:YES];
      return;
    }
    
    if (nil == [specifier file]) {
      [tableView deselectRowAtIndexPath:indexPath animated:YES];
      return;
    }        
    
    IASKAppSettingsViewController *targetViewController = (self.viewList)[kIASKSpecifierChildViewControllerIndex][@"viewController"];
    
    if (targetViewController == nil) {
      // the view controller has not been created yet, create it and set it to our viewList array
      // create a new dictionary with the new view controller
      NSMutableDictionary *newItemDict = [NSMutableDictionary dictionaryWithCapacity:3];
      [newItemDict addEntriesFromDictionary: (self.viewList)[kIASKSpecifierChildViewControllerIndex]];	// copy the title and explain strings
      
      targetViewController = [[[self class] alloc] initWithNibName:@"IASKAppSettingsView" bundle:nil];
      targetViewController.showDoneButton = NO;
      targetViewController.settingsStore = self.settingsStore; 
      targetViewController.delegate = self.delegate;
      
      // add the new view controller to the dictionary and then to the 'viewList' array
      newItemDict[@"viewController"] = targetViewController;
      (self.viewList)[kIASKSpecifierChildViewControllerIndex] = newItemDict;
      
      // load the view controll back in to push it
      targetViewController = (self.viewList)[kIASKSpecifierChildViewControllerIndex][@"viewController"];
    }
    self.currentIndexPath = indexPath;
    targetViewController.settingsReader = [[IASKSettingsReader alloc]initWithFile:specifier.file customBundle:self.settingsReader.bundlePath.lastPathComponent]; 
//    targetViewController.file = specifier.file;
    targetViewController.title = specifier.title;
    targetViewController.showCreditsFooter = NO;
    [[self navigationController] pushViewController:targetViewController animated:YES];
  } else if ([[specifier type] isEqualToString:kIASKOpenURLSpecifier]) {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    NSURL *url;
    if([self.delegate respondsToSelector:@selector(settingsViewController:urlToOpenForKey:)])
      url = [self.delegate settingsViewController:self urlToOpenForKey:specifier.key];
    else
      url = [NSURL URLWithString:specifier.file];
    
    [[UIApplication sharedApplication] openURL:url];
  } else if ([[specifier type] isEqualToString:kIASKButtonSpecifier]) {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    if ([self.delegate respondsToSelector:@selector(settingsViewController:buttonTappedForKey:)]) {
      [self.delegate settingsViewController:self buttonTappedForKey:[specifier key]];
    } else {
      // legacy code, provided for backward compatibility
      // the delegate mechanism above is much cleaner and doesn't leak
      Class buttonClass = [specifier buttonClass];
      SEL buttonAction = [specifier buttonAction];
      if ([buttonClass respondsToSelector:buttonAction]) {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
        [buttonClass performSelector:buttonAction withObject:self withObject:[specifier key]];
#pragma clang diagnostic pop

        NSLog(@"InAppSettingsKit Warning: Using IASKButtonSpecifier without implementing the delegate method is deprecated");
      }
    }
  } else if ([[specifier type] isEqualToString:kIASKMailComposeSpecifier]) {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    if ([MFMailComposeViewController canSendMail]) {
      MFMailComposeViewController *mailViewController = [[MFMailComposeViewController alloc] init];
      mailViewController.navigationBar.barStyle = self.navigationController.navigationBar.barStyle;
      mailViewController.navigationBar.tintColor = self.navigationController.navigationBar.tintColor;
      
      NSString *subject = [specifier localizedObjectForKey:kIASKMailComposeSubject];
      if (subject) {
        [mailViewController setSubject:subject];
      }
      if ([specifier specifierDict][kIASKMailComposeToRecipents]) {
        [mailViewController setToRecipients:[specifier specifierDict][kIASKMailComposeToRecipents]];
      }
      if ([specifier specifierDict][kIASKMailComposeCcRecipents]) {
        [mailViewController setCcRecipients:[specifier specifierDict][kIASKMailComposeCcRecipents]];
      }
      if ([specifier specifierDict][kIASKMailComposeBccRecipents]) {
        [mailViewController setBccRecipients:[specifier specifierDict][kIASKMailComposeBccRecipents]];
      }
      if ([specifier localizedObjectForKey:kIASKMailComposeBody]) {
        BOOL isHTML = NO;
        if ([specifier specifierDict][kIASKMailComposeBodyIsHTML]) {
          isHTML = [[specifier specifierDict][kIASKMailComposeBodyIsHTML] boolValue];
        }
        
        if ([self.delegate respondsToSelector:@selector(mailComposeBodyForSubject:)]) {
          [mailViewController setMessageBody:[self.delegate mailComposeBodyForSubject:subject] isHTML:isHTML];
        }
        else {
          [mailViewController setMessageBody:[specifier localizedObjectForKey:kIASKMailComposeBody] isHTML:isHTML];
        }
      }
      
      UIViewController<MFMailComposeViewControllerDelegate> *vc = nil;
      
      if ([self.delegate respondsToSelector:@selector(viewControllerForMailComposeView)]) {
        vc = [self.delegate viewControllerForMailComposeView];
      }
      
      if (vc == nil) {
        vc = self;
      }
      mailViewController.modalPresentationStyle = UIModalPresentationPageSheet;
      mailViewController.mailComposeDelegate = vc;
      if(iOS6())
        [vc presentModalViewController:mailViewController animated:YES];
      else
        [vc presentViewController:mailViewController animated:YES completion:^{
          [[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleLightContent];
        }];
    } else {
      UIAlertView *alert = [[UIAlertView alloc]
                            initWithTitle:NSLocalizedString(@"Mail not configured", @"InAppSettingsKit")
                            message:NSLocalizedString(@"This device is not configured for sending Email. Please configure the Mail settings in the Settings app.", @"InAppSettingsKit")
                            delegate: nil
                            cancelButtonTitle:NSLocalizedString(@"OK", @"InAppSettingsKit")
                            otherButtonTitles:nil];
      [alert show];
    }
    
  } else {
    [tableView deselectRowAtIndexPath:indexPath animated:NO];
  }
}


#pragma mark -
#pragma mark MFMailComposeViewControllerDelegate Function

-(void)mailComposeController:(MFMailComposeViewController*)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError*)error {
  // NOTE: No error handling is done here
  [self dismissModalViewControllerAnimated:YES];
}

#pragma mark -
#pragma mark UITextFieldDelegate Functions

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField {
  self.currentFirstResponder = textField;
  return YES;
}

- (void)_textChanged:(id)sender {
  IASKTextField *text = (IASKTextField*)sender;
  [_settingsStore setObject:[text text] forKey:[text key]];
  [[NSNotificationCenter defaultCenter] postNotificationName:kIASKAppSettingChanged
                                                      object:[text key]
                                                    userInfo:@{[text key]: [text text]}];
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField{
  [textField resignFirstResponder];
  self.currentFirstResponder = nil;
  return YES;
}


#pragma mark Notifications

- (void)synchronizeSettings {
  [_settingsStore synchronize];
}

- (void)reload {
  // wait 0.5 sec until UI is available after applicationWillEnterForeground
  [self.tableView performSelector:@selector(reloadData) withObject:nil afterDelay:0.5];
}

#pragma mark CGRect Utility function
CGRect IASKCGRectSwap(CGRect rect) {
  CGRect newRect;
  newRect.origin.x = rect.origin.y;
  newRect.origin.y = rect.origin.x;
  newRect.size.width = rect.size.height;
  newRect.size.height = rect.size.width;
  return newRect;
}
@end
