/*
 * Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
 * http://www.opensource.org/licenses/artistic-license-2.0.php
 */

/**
 * \file MacOS-specific implementation of utf8conv.h
 */

#include "../typedefs.h"
#include "../utf8conv.h"
#include <cstdlib>
#include <clocale>
#include <cassert>
#include <cstring>
#include <string>
#import "NSStringUtils.h"

using namespace std;

class Startup {
public:
  Startup() {
    char *sl = std::setlocale(LC_ALL, "");
    if (sl == NULL)
      throw "Couldn't initialize locale - bailing out";
  }
};

static Startup startup;

size_t pws_os::wcstombs(char *dst, size_t maxdstlen,
                        const wchar_t *src, size_t src_len, bool )
{
  if(src_len == size_t(-1))
    src_len = wcslen(src) + 1;
  bool nullTerminated = src_len > 0 && src[src_len - 1] == 0;
  return ::wcstombs(dst, src, maxdstlen) + (nullTerminated ? 1 : 0);
}

size_t pws_os::mbstowcs(wchar_t *dst, size_t maxdstlen,
                        const char *src, size_t src_len, bool )
{
  if(src_len == size_t(-1))
    src_len = strlen(src) + 1;
  bool nullTerminated = src_len > 0 && src[src_len - 1] == 0;
  return ::mbstowcs(dst, src, maxdstlen) + (nullTerminated ? 1 : 0);
}

wstring pws_os::towc(const char *val)
{
  if(!val)
    return wstring();
  
  NSString *str = @(val);
  NSData* asData = [str dataUsingEncoding:kEncoding_wchar_t];
  const wchar_t *wData = (const wchar_t *)[asData bytes];
  wstring ret = wstring(wData, [asData length] / sizeof(wchar_t));
  return ret;
}

#ifdef UNICODE
std::string pws_os::tomb(const stringT& val)
{
  if(val.empty())
    return string();
  NSString *str = [NSString stringWithWstring:val];
  NSData *utf8Data = [str dataUsingEncoding:NSUTF8StringEncoding];
  std::string ret = string((const char *)utf8Data.bytes, utf8Data.length);
  return ret;
}

#else
std::string pws_os::tomb(const stringT& val)
{
  return val;
}
#endif


