/*
* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
* All rights reserved. Use of the code is allowed under the
* Artistic License 2.0 terms, as specified in the LICENSE file
* distributed with this code, or available from
* http://www.opensource.org/licenses/artistic-license-2.0.php
*/
#ifndef _PWS_TCHAR_H
/**
 * Use Windows' tchar.h for Windows build,
 * roll our own for others.
 *
 */
#ifdef _WIN32
#include <tchar.h>
#else
#define _gmtime64_s(ts64, tm64) gmtime64_r(tm64, ts64)
#define _mkgmtime32(ts) mktime(ts)
#define localtime_s(st, t) (localtime_r(t, st) != NULL ? 0 : 1)
#ifdef UNICODE
#include <wctype.h>
#include <xlocale.h>

#ifdef __cplusplus
extern "C" {
#endif

int ios_iswalpha(wint_t _wc);
int ios_iswalnum(wint_t _wc);
wint_t ios_towupper(wint_t _wc);
wint_t ios_towlower(wint_t _wc);
int ios_iswlower(wint_t _wc);
int ios_iswupper(wint_t _wc);
int ios_iswdigit(wint_t _wc);
#ifdef __PWS_MACINTOSH__
int vaswprintf(wchar_t * __restrict * __restrict out, const wchar_t * __restrict fmt, __darwin_va_list va);
#endif

#ifdef __cplusplus
}
#endif

#define _istalpha(x) ios_iswalpha(x)
#define _istalnum(x) ios_iswalnum(x)
#define _totupper(x) ios_towupper(x)
#define _totlower(x) ios_towlower(x)
#define _istlower(x) ios_iswlower(x)
#define _istupper(x) ios_iswupper(x)
#define _istdigit(x) ios_iswdigit(x)
#define _istspace(x) iswspace(x)
#include <wchar.h>
#include "mac/pws_time.h"
#define _tcsncpy(t, s, sc) wcsncpy(t, s, sc)
#define _tcslen(s) wcslen(s)
#define _tcsclen(s) wcslen(s)
#define _tcscmp(s1, s2) wcscmp(s1, s2)
#define _tcsncmp(s1, s2, n) wcsncmp(s1, s2, n)
#define _tcschr(s, c) wcschr(s, c)
#define _tcsftime(wcs, maxsize, format, timeptr) wcsftime(wcs, maxsize, format, timeptr)
#define _tcsdup wcsdup
#define _tasctime_s(s, N, st) pws_os::asctime(s, N, st)
#define _vsctprintf(fmt, args) vswprintf_l(NULL, utf8_locale, fmt, args)
#define _vstprintf_s(str, size, fmt, args) vswprintf(str, size, fmt, args)
#define _ftprintf(stream, format, ...) fwprintf_l(stream, utf8_locale, format, ...)
#define _stprintf_s(ws, n, fmt, ...) swprintf(ws, utf8_locale, n, fmt, ...)
#define _stscanf(str, fmt, ...) swscanf_l(str, utf8_locale, fmt, ...)
#ifdef __PWS_MACINTOSH__
# include "./mac/pws_str.h"
#define _tcsicmp pws_os::wcscasecmp
#else
#define _tcsicmp(s1, s2) wcscasecmp(s1, s2)
# include "linux/pws_str.h"
#endif
#define _ttoi(s) pws_os::wctoi(s)
#define _tstoi(s) pws_os::wctoi(s)
#define _tstof(s) pws_os::wctof(s)
#ifdef __linux__
#include "./file.h"
#define _tfopen(f,m) pws_os::FOpen(f,m)
#endif
#else /* !UNICODE */
#include <ctype.h>
#define _istalpha(x) isalpha(x)
#define _istalnum(x) isalnum(x)
#define _totupper(x) toupper(x)
#define _totlower(x) tolower(x)
#define _istlower(x) islower(x)
#define _istupper(x) isupper(x)
#define _istdigit(x) isdigit(x)
#define _istspace(x) isspace(x)
#include <string.h>
#define _tcsncpy(t, s, sc) strncpy(t, s, sc)
#define _tcslen(s) strlen(s)
#define _tcsclen(s) strlen(s)
#define _tcsicmp(s1, s2) strcasecmp(s1, s2)
#define _tcscmp(s1, s2) strcmp(s1, s2)
#define _tcsncmp(s1, s2, n) strncmp(s1, s2, n)
#define _tcschr(s, c) strchr(s, c)
#include <time.h>
#define _tcsftime strftime
#define _tasctime_s(s, N, st) (asctime_r(st, s) != NULL ? 0 : 1)
#define _vsctprintf(fmt, args) vsnprintf(NULL, 0, fmt, args)
#define _vstprintf_s(str, size, fmt, args) vsnprintf(str, size, fmt, args)
#define _ftprintf fprintf
#define _stprintf_s snprintf
#define _stscanf sscanf
#define _tstoi(s) atoi(s)
#define _ttoi(s) atoi(s)
#define _tstof(s) atof(s)
#define _tfopen(s) fopen(s)
#endif /* UNICODE */
#define _itot(i, buf, base) pws_os::pws_itot(i, buf, base)
#endif /* _WIN32 */
#endif /* _PWS_TCHAR_H */
